@@ -1,5 +1,5 @@
 /*
- *  $Id: data-browser.h 24090 2021-09-07 12:43:58Z yeti-dn $
+ *  $Id: data-browser.h 24553 2021-12-02 15:10:33Z yeti-dn $
  *  Copyright (C) 2006-2021 David Necas (Yeti), Petr Klapetek, Chris Anderson
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, sidewinderasu@gmail.com.
  *
@@ -323,11 +323,16 @@ GdkPixbuf* gwy_app_get_curve_map_thumbnail(GwyContainer *data,
                                            gint id,
                                            gint max_width,
                                            gint max_height);
-
 void gwy_app_data_browser_select_data_field (GwyContainer *data,
                                              gint id);
 void gwy_app_data_browser_select_graph_model(GwyContainer *data,
                                              gint id);
+void gwy_app_data_browser_select_surface    (GwyContainer *data,
+                                             gint id);
+void gwy_app_data_browser_select_brick      (GwyContainer *data,
+                                             gint id);
+void gwy_app_data_browser_select_lawn       (GwyContainer *data,
+                                             gint id);
 /* XXX */
 void     gwy_app_data_browser_show_3d       (GwyContainer *data,
                                              gint id);
