@@ -1,5 +1,5 @@
 /*
- *  $Id: file.c 24584 2022-02-05 10:48:18Z yeti-dn $
+ *  $Id: file.c 25860 2023-10-15 14:18:23Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -262,25 +262,27 @@ gwy_app_file_load_real(const gchar *filename_utf8,
         if (do_add_loaded)
             _gwy_app_log_discard_captured_messages();
 
-        if (err && !g_error_matches(err, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_CANCELLED)) {
-            gchar *filename_disp = g_filename_display_basename(filename_utf8);
-            GString *message = g_string_new(NULL);
+        if (err) {
+            if (!g_error_matches(err, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_CANCELLED)) {
+                gchar *filename_disp = g_filename_display_basename(filename_utf8);
+                GString *message = g_string_new(NULL);
 
-            dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                                            _("Opening of `%s' failed"), filename_disp);
-            g_free(filename_disp);
-            g_string_append(message, err->message);
-            g_string_append(message, "\n\n");
-            g_string_append_printf(message, _("Full file path: %s."), filename_utf8);
-            if (name) {
+                dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                                                _("Opening of `%s' failed"), filename_disp);
+                g_free(filename_disp);
+                g_string_append(message, err->message);
                 g_string_append(message, "\n\n");
-                g_string_append_printf(message, _("Loaded using: %s."), name);
+                g_string_append_printf(message, _("Full file path: %s."), filename_utf8);
+                if (name) {
+                    g_string_append(message, "\n\n");
+                    g_string_append_printf(message, _("Loaded using: %s."), name);
+                }
+                gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), "%s", message->str);
+                g_string_free(message, TRUE);
+                g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
+                gtk_widget_show_all(dialog);
+                gtk_window_present(GTK_WINDOW(dialog));
             }
-            gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), "%s", message->str);
-            g_string_free(message, TRUE);
-            g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
-            gtk_widget_show_all(dialog);
-            gtk_window_present(GTK_WINDOW(dialog));
             g_clear_error(&err);
         }
         else if (!err)
