@@ -1,5 +1,5 @@
 /*
- *  $Id: gwymoduleutils.h 25224 2023-02-21 17:35:52Z yeti-dn $
+ *  $Id: gwymoduleutils.h 25438 2023-06-15 08:15:29Z yeti-dn $
  *  Copyright (C) 2007-2023 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -111,6 +111,11 @@ void          gwy_preview_surface_to_datafield          (GwySurface *surface,
 GtkWidget*    gwy_app_wait_preview_data_field           (GwyDataField *dfield,
                                                          GwyContainer *data,
                                                          gint id);
+void          gwy_set_gradient_for_residuals            (GwyGradient *gradient,
+                                                         gdouble fullmin,
+                                                         gdouble fullmax,
+                                                         gdouble *dispmin,
+                                                         gdouble *dispmax);
 gint          gwy_run_subdialog                         (GtkDialog *dialog);
 gboolean      gwy_app_data_id_verify_channel            (GwyAppDataId *id);
 gboolean      gwy_app_data_id_verify_graph              (GwyAppDataId *id);
