@@ -1,5 +1,5 @@
 /*
- *  $Id: gwytool.c 24709 2022-03-21 17:31:45Z yeti-dn $
+ *  $Id: gwytool.c 24885 2022-08-12 14:33:14Z yeti-dn $
  *  Copyright (C) 2006-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -233,7 +233,7 @@ gwy_tool_configure_event(GwyTool *tool,
     gtk_window_get_position(GTK_WINDOW(widget), &cpriv->x, &cpriv->y);
     gwy_debug("saving %s on screen #%d at (%d,%d)",
               G_OBJECT_TYPE_NAME(tool),
-              gdk_screen_get_number(cpriv->screen),
+              cpriv->screen ? gdk_screen_get_number(cpriv->screen) : -1,
               cpriv->x, cpriv->y);
 
     screen = gtk_widget_get_screen(widget);
