@@ -1,22 +1,18 @@
 /*
- *  $Id: logging.c 25416 2023-06-07 14:14:42Z yeti-dn $
- *  Copyright (C) 2016-2018 David Necas (Yeti).
+ *  $Id: logging.c 25432 2023-06-14 13:30:43Z yeti-dn $
+ *  Copyright (C) 2016-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -41,9 +37,10 @@
 
 /* Our handler should not get G_LOG_LEVEL_ERROR errors but it does not hurt. */
 enum {
-    ALERT_LEVELS = (G_LOG_LEVEL_ERROR
-                    | G_LOG_LEVEL_CRITICAL
-                    | G_LOG_LEVEL_WARNING)
+    ALERT_LEVELS = G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING,
+    SEVERE_LEVELS = G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL,
+    IGNORABLE_LEVELS = G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_INFO,
+    STDERR_LEVELS = G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING | G_LOG_LEVEL_MESSAGE,
 };
 
 typedef struct {
@@ -85,7 +82,6 @@ static void     emit_log_message      (LoggingSetup *setup,
                                        GLogLevelFlags log_level);
 static void     append_level_prefix   (GString *str,
                                        GLogLevelFlags log_level);
-static FILE*    get_console_stream    (GLogLevelFlags log_level);
 static gboolean skip_annoying_message (const gchar *message);
 
 static LoggingSetup log_setup;
@@ -96,9 +92,8 @@ static LoggingSetup log_setup;
  *
  * Sets up Gwyddion GLib log handler.
  *
- * The log handler sends the messages to a log file or console, as Gwyddion
- * usually does.  This function may not be useful in Gwyddion-based programs
- * unless they try to emulate Gwyddion behaviour closely.
+ * The log handler sends the messages to a log file or console, as Gwyddion usually does.  This function may not be
+ * useful in Gwyddion-based programs unless they try to emulate Gwyddion behaviour closely.
  *
  * Since: 2.45
  **/
@@ -106,8 +101,7 @@ void
 gwy_app_setup_logging(GwyAppLoggingFlags flags)
 {
     static const gchar *our_domains[] = {
-        "Gwyddion", "GwyProcess", "GwyDraw", "Gwydgets", "GwyModule", "GwyApp",
-        "Module", "Gwyddion-Program", NULL
+        "Gwyddion", "GwyProcess", "GwyDraw", "Gwydgets", "GwyModule", "GwyApp", "Module", "Gwyddion-Program", NULL
     };
     static const gchar *other_domains[] = {
         "GLib", "GLib-GObject", "GLib-GIO", "GModule", "GThread",
@@ -138,35 +132,20 @@ gwy_app_setup_logging(GwyAppLoggingFlags flags)
     log_setup.last = g_string_new(NULL);
     log_setup.last_domain = 0;
     log_setup.last_count = G_MAXUINT;
-    log_setup.message_history = g_array_new(FALSE, FALSE,
-                                            sizeof(GwyAppLogMessage));
+    log_setup.message_history = g_array_new(FALSE, FALSE, sizeof(GwyAppLogMessage));
     log_setup.capturing_from = G_MAXUINT;
-    /* NB: We must not initialise the text buffer here because Gtk+ may not be
-     * initialised yet.  Only do that on demand. */
+    /* NB: We must not initialise the text buffer here because Gtk+ may not be initialised yet.  Only do that on
+     * demand. */
 
-    for (i = 0; i < G_N_ELEMENTS(our_domains); i++) {
-        g_log_set_handler(our_domains[i],
-                          G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE
-                          | G_LOG_LEVEL_INFO | G_LOG_LEVEL_WARNING
-                          | G_LOG_LEVEL_CRITICAL,
-                          logger, &log_setup);
-    }
-    for (i = 0; i < G_N_ELEMENTS(other_domains); i++) {
-        g_log_set_handler(other_domains[i],
-                          G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL,
-                          logger, &log_setup);
-    }
-    /* Send messages at levels lower than WARNING to a black hole because
-     * the developers of the libraries think printing DEBUG messages in
-     * production code is a good idea and hence they cannot be trusted.  We
-     * could make an effort to pass some specifically to the default handler,
-     * but I do not care. */
-    for (i = 0; i < G_N_ELEMENTS(other_domains); i++) {
-        g_log_set_handler(other_domains[i],
-                          G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE
-                          | G_LOG_LEVEL_INFO,
-                          black_hole, NULL);
-    }
+    for (i = 0; i < G_N_ELEMENTS(our_domains); i++)
+        g_log_set_handler(our_domains[i], SEVERE_LEVELS | IGNORABLE_LEVELS, logger, &log_setup);
+    for (i = 0; i < G_N_ELEMENTS(other_domains); i++)
+        g_log_set_handler(other_domains[i], SEVERE_LEVELS, logger, &log_setup);
+    /* Send messages at levels lower than WARNING to a black hole because the developers of the libraries think
+     * printing DEBUG messages in production code is a good idea and hence they cannot be trusted.  We could make an
+     * effort to pass some specifically to the default handler, but I do not care. */
+    for (i = 0; i < G_N_ELEMENTS(other_domains); i++)
+        g_log_set_handler(other_domains[i], IGNORABLE_LEVELS, black_hole, NULL);
 }
 
 void
@@ -204,8 +183,7 @@ _gwy_app_log_get_captured_messages(guint *nmesg)
     n = message_history->len - capturing_from;
     messages = g_new(GwyAppLogMessage, n);
     for (i = 0; i < n; i++) {
-        messages[i] = g_array_index(message_history, GwyAppLogMessage,
-                                    capturing_from + i);
+        messages[i] = g_array_index(message_history, GwyAppLogMessage, capturing_from + i);
         messages[i].message = g_strdup(messages[i].message);
     }
     *nmesg = n;
@@ -234,10 +212,8 @@ flush_last_message(LoggingSetup *setup)
     }
 
     just_log_level = (setup->last_level & G_LOG_LEVEL_MASK);
-    g_string_printf(setup->last, "Last message repeated %u times",
-                    setup->last_count);
-    format_log_message(setup->message, setup->last_domain, just_log_level,
-                       setup->last->str);
+    g_string_printf(setup->last, "Last message repeated %u times", setup->last_count);
+    format_log_message(setup->message, setup->last_domain, just_log_level, setup->last->str);
     emit_log_message(setup, setup->last_domain, just_log_level);
     setup->last_count = G_MAXUINT;
 }
@@ -258,9 +234,7 @@ logger_impl(const gchar *log_domain_str,
 {
     LoggingSetup *setup = (LoggingSetup*)user_data;
     GLogLevelFlags just_log_level = (log_level & G_LOG_LEVEL_MASK);
-    GQuark log_domain = (log_domain_str
-                         ? g_quark_from_string(log_domain_str)
-                         : 0);
+    GQuark log_domain = (log_domain_str ? g_quark_from_string(log_domain_str) : 0);
 
     /* Can't use gwy_debug() for debugging here... */
     /*
@@ -305,10 +279,8 @@ emit_log_message(LoggingSetup *setup,
     gchar *message = setup->message->str;
 
     if (setup->to_console) {
-        if (!skip_annoying_message(message)) {
-            emit_message_to_file(get_console_stream(log_level),
-                                 log_level, message, TRUE);
-        }
+        if (!skip_annoying_message(message))
+            emit_message_to_file((log_level & STDERR_LEVELS) ? stderr : stdout, log_level, message, TRUE);
     }
 
     if (setup->to_file)
@@ -379,8 +351,7 @@ append_escaped_message(GString *str, const gchar *message)
     g_utf8_validate(message, -1, &p);
     utf8_valid_only_to = p - message;
     for (pos = good_from = 0; (c = (guchar)message[pos]); pos++) {
-        /* First find out if this character breaks the sequence of characters
-         * we just copy as-is. */
+        /* First find out if this character breaks the sequence of characters we just copy as-is. */
         escape_this = FALSE;
         if (pos == utf8_valid_only_to) {
             escape_this = TRUE;
@@ -390,9 +361,8 @@ append_escaped_message(GString *str, const gchar *message)
         else if (g_ascii_iscntrl(c) && !g_ascii_isspace(c))
             escape_this = TRUE;
 
-        /* If it does not just increment the counter.  If we have to escape
-         * this one then first copy as-is the entire good segment, and only
-         * after that add the bad character. */
+        /* If it does not just increment the counter.  If we have to escape this one then first copy as-is the entire
+         * good segment, and only after that add the bad character. */
         if (escape_this) {
             if (pos > good_from)
                 g_string_append_len(str, message + good_from, pos - good_from);
@@ -404,8 +374,9 @@ append_escaped_message(GString *str, const gchar *message)
         g_string_append_len(str, message + good_from, pos - good_from);
 }
 
-/* Similar to GLib's function but we do not handle recursive and fatal errors
- * so we do not have to handle the difficult cases and can use GLib functions.
+/*
+ * Similar to GLib's function but we do not handle recursive and fatal errors so we do not have to handle the
+ * difficult cases and can use GLib functions.
  */
 static void
 append_level_prefix(GString *str,
@@ -429,24 +400,13 @@ append_level_prefix(GString *str,
         g_string_append(str, "LOG");
 }
 
-static FILE*
-get_console_stream(GLogLevelFlags log_level)
-{
-    if (log_level == G_LOG_LEVEL_ERROR
-        || log_level == G_LOG_LEVEL_CRITICAL
-        || log_level == G_LOG_LEVEL_WARNING
-        || log_level == G_LOG_LEVEL_MESSAGE)
-        return stderr;
-    return stdout;
-}
-
 /* Annoying messages we keep in the log but do not print to console. */
 static gboolean
 skip_annoying_message(const gchar *message)
 {
     static const gchar *skip_message_prefixes[] = {
-        "GtkSourceView-WARNING: Cannot create a regex for all the transitions, "
-            "the syntax highlighting process will be slower than usual.",
+        "GtkSourceView-WARNING: Cannot create a regex for all the transitions, the syntax highlighting process "
+            "will be slower than usual.",
     };
     guint i;
 
@@ -462,13 +422,11 @@ skip_annoying_message(const gchar *message)
  *
  * Obtains a text buffer with program log messages.
  *
- * This functions may only be called after gwy_app_setup_logging() and,
- * obviously, after GTK+ was intialised.
+ * This functions may only be called after gwy_app_setup_logging() and, obviously, after GTK+ was intialised.
  *
- * The text buffer is owned by the library and must not be modified nor
- * destroyed.  It will be already filled with messages occurring between
- * gwy_app_setup_logging() and this function call.  New messages will be
- * appended to the buffer as they arrive.
+ * The text buffer is owned by the library and must not be modified nor destroyed.  It will be already filled with
+ * messages occurring between gwy_app_setup_logging() and this function call.  New messages will be appended to the
+ * buffer as they arrive.
  *
  * Returns: Text buffer with the program log messages.
  *
@@ -478,8 +436,7 @@ GtkTextBuffer*
 gwy_app_get_log_text_buffer(void)
 {
     if (!log_setup.message_history) {
-        g_warning("Obtaining program log text buffer requires "
-                  "gwy_app_setup_logging() being called first.");
+        g_warning("Obtaining program log text buffer requires gwy_app_setup_logging() being called first.");
         return _gwy_app_log_create_textbuf();
     }
 
@@ -489,11 +446,8 @@ gwy_app_get_log_text_buffer(void)
 
         log_setup.textbuf = _gwy_app_log_create_textbuf();
         for (i = 0; i < message_history->len; i++) {
-            GwyAppLogMessage *message = &g_array_index(message_history,
-                                                       GwyAppLogMessage, i);
-            _gwy_app_log_add_message_to_textbuf(log_setup.textbuf,
-                                                message->message,
-                                                message->log_level);
+            GwyAppLogMessage *message = &g_array_index(message_history, GwyAppLogMessage, i);
+            _gwy_app_log_add_message_to_textbuf(log_setup.textbuf, message->message, message->log_level);
         }
     }
 
@@ -557,10 +511,8 @@ _gwy_app_log_add_message_to_textbuf(GtkTextBuffer *textbuf,
         tagname = "DEBUG";
 
     gtk_text_buffer_get_end_iter(textbuf, &iter);
-    if (tagname) {
-        gtk_text_buffer_insert_with_tags_by_name(textbuf, &iter, message, -1,
-                                                 tagname, NULL);
-    }
+    if (tagname)
+        gtk_text_buffer_insert_with_tags_by_name(textbuf, &iter, message, -1, tagname, NULL);
     else
         gtk_text_buffer_insert(textbuf, &iter, message, -1);
 }
@@ -575,10 +527,9 @@ _gwy_app_log_add_message_to_textbuf(GtkTextBuffer *textbuf,
 
 /**
  * GwyAppLoggingFlags:
- * @GWY_APP_LOGGING_TO_FILE: Messages go to a log file, either gwyddion.log or
- *                           given by environment variable GWYDDION_LOGFILE.
- * @GWY_APP_LOGGING_TO_CONSOLE: Messages go to standard output and standard
- *                              error depending on message type (emulating
+ * @GWY_APP_LOGGING_TO_FILE: Messages go to a log file, either gwyddion.log or given by environment variable
+ *                           GWYDDION_LOGFILE.
+ * @GWY_APP_LOGGING_TO_CONSOLE: Messages go to standard output and standard error depending on message type (emulating
  *                              where GLib sends them).
  *
  * Flags controlling where program messages are written.
@@ -586,4 +537,4 @@ _gwy_app_log_add_message_to_textbuf(GtkTextBuffer *textbuf,
  * Since: 2.45
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
