@@ -1,5 +1,5 @@
 /*
- *  $Id: menu.h 24181 2021-09-23 08:29:39Z yeti-dn $
+ *  $Id: menu.h 24709 2022-03-21 17:31:45Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -70,7 +70,10 @@ void       gwy_app_run_curve_map_func_in_mode  (const gchar *name,
 
 void         gwy_app_menu_recent_files_update(GList *recent_files);
 GtkWidget*   gwy_app_menu_recent_files_get   (void);
+
+#ifndef GWY_DISABLE_DEPRECATED
 GtkTooltips* gwy_app_get_tooltips            (void);
+#endif
 GwySensitivityGroup* gwy_app_sensitivity_get_group   (void);
 void         gwy_app_sensitivity_add_widget  (GtkWidget *widget,
                                               GwyMenuSensFlags mask);
