@@ -1,6 +1,6 @@
 /*
- *  $Id: param-def.c 25214 2023-01-30 14:13:23Z yeti-dn $
- *  Copyright (C) 2021-2022 David Necas (Yeti).
+ *  $Id: param-def.c 25756 2023-10-02 14:07:38Z yeti-dn $
+ *  Copyright (C) 2021-2023 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
@@ -16,6 +16,7 @@
  */
 
 #include "config.h"
+#include <stdlib.h>
 #include "libgwyddion/gwymacros.h"
 #include "libprocess/gwyprocesstypes.h"
 #include "libprocess/gwyprocessenums.h"
@@ -312,7 +313,8 @@ gwy_param_def_add_gwyflags(GwyParamDef *pardef,
  * Defines a new parameter with enumerated values from a standard Gwyddion enum.
  *
  * Supported enums include #GwyMaskingType, #GwyInterpolationType, #GwyOrientation, #GwyMergeType,
- * #GwyDistanceTransformType, #GwyWindowingType.  They have standard labels allowing to pass %NULL as @desc.
+ * #GwyDistanceTransformType, #GwyWindowingType and GwyGraphCurveType.  They have standard labels allowing to pass
+ * %NULL as @desc.
  *
  * See the introduction for @name and @desc lifetime rules.
  *
@@ -384,7 +386,7 @@ gwy_param_def_append_enum(GwyParamDef *pardef,
     GwyParamDefEnum *e = &item.def.e;
 
     if (!nvalues || !values) {
-        g_warning("Enum param %s (%s) has no values.", name ? name : "???", desc ? desc : "");
+        g_warning("Enum param %s (%s) has no values.", name ? name : "(anonymous)", desc ? desc : "");
         nvalues = 0;
         values = NULL;
     }
@@ -398,7 +400,7 @@ gwy_param_def_append_enum(GwyParamDef *pardef,
     e->default_value_index = find_enum_value(values, nvalues, default_value);
     if (e->default_value_index < 0) {
         g_warning("Enum param %s (%s) default value %d is not in the enum.",
-                  name ? name : "???", desc ? desc : "", default_value);
+                  name ? name : "(anonymous)", desc ? desc : "", default_value);
         e->default_value_index = 0;
     }
 
@@ -421,7 +423,7 @@ gwy_param_def_append_flags(GwyParamDef *pardef,
     gboolean warned = FALSE;
 
     if (!nvalues || !values) {
-        g_warning("Flags param %s (%s) has no values.", name ? name : "???", desc ? desc : "");
+        g_warning("Flags param %s (%s) has no values.", name ? name : "(anonymous)", desc ? desc : "");
         nvalues = 0;
         values = NULL;
     }
@@ -438,7 +440,7 @@ gwy_param_def_append_flags(GwyParamDef *pardef,
         if (b & (b - 1)) {
             /* Can't fix this one. */
             g_warning("Flags param %s (%s) flag %u does not have exactly 1 bit set.",
-                      name ? name : "???", desc ? desc : "", values[i].value);
+                      name ? name : "(anonymous)", desc ? desc : "", values[i].value);
             warned = TRUE;
         }
         allset |= b;
@@ -454,13 +456,13 @@ gwy_param_def_append_flags(GwyParamDef *pardef,
         if (b != nvalues) {
             /* Can't fix this one. */
             g_warning("Flags param %s (%s) value bits are not independent.",
-                      name ? name : "???", desc ? desc : "");
+                      name ? name : "(anonymous)", desc ? desc : "");
         }
     }
     f->default_value = _gwy_param_def_rectify_flags(&item, default_value);
     if (f->default_value != default_value) {
         g_warning("Flags param %s (%s) default value %u has bits not among the flags.",
-                  name ? name : "???", desc ? desc : "", default_value);
+                  name ? name : "(anonymous)", desc ? desc : "", default_value);
     }
 
     gwy_param_def_append(pardef, id, name, desc, &item);
@@ -496,7 +498,7 @@ gwy_param_def_add_int(GwyParamDef *pardef,
 
     if (minimum > maximum) {
         g_warning("Int param %s (%s) has minimum > maximum (%d > %d).",
-                  name ? name : "???", desc ? desc : "", minimum, maximum);
+                  name ? name : "(anonymous)", desc ? desc : "", minimum, maximum);
         GWY_SWAP(gint, minimum, maximum);
     }
     item.type = GWY_PARAM_INT;
@@ -505,7 +507,7 @@ gwy_param_def_add_int(GwyParamDef *pardef,
     i->default_value = _gwy_param_def_rectify_int(&item, default_value);
     if (i->default_value != default_value) {
         g_warning("Int param %s (%s) default value %d is out of range [%d..%d].",
-                  name ? name : "???", desc ? desc : "", default_value, minimum, maximum);
+                  name ? name : "(anonymous)", desc ? desc : "", default_value, minimum, maximum);
     }
     gwy_param_def_append(pardef, id, name, desc, &item);
 }
@@ -689,7 +691,7 @@ gwy_param_def_add_double(GwyParamDef *pardef,
     gwy_clear(d, 1);
     if (minimum > maximum) {
         g_warning("Double param %s (%s) has minimum > maximum (%.14g > %.14g).",
-                  name ? name : "???", desc ? desc : "", minimum, maximum);
+                  name ? name : "(anonymous)", desc ? desc : "", minimum, maximum);
         GWY_SWAP(gdouble, minimum, maximum);
     }
     item.type = GWY_PARAM_DOUBLE;
@@ -698,7 +700,7 @@ gwy_param_def_add_double(GwyParamDef *pardef,
     d->default_value = _gwy_param_def_rectify_double(&item, default_value);
     if (d->default_value != default_value) {
         g_warning("Double param %s (%s) default value %.14g is out of range [%.14g..%.14g].",
-                  name ? name : "???", desc ? desc : "", default_value, minimum, maximum);
+                  name ? name : "(anonymous)", desc ? desc : "", default_value, minimum, maximum);
     }
     gwy_param_def_append(pardef, id, name, desc, &item);
 }
@@ -756,7 +758,7 @@ gwy_param_def_add_angle(GwyParamDef *pardef,
     d->default_value = _gwy_param_def_rectify_double(&item, default_value);
     if (d->default_value != default_value) {
         g_warning("Double param %s (%s) default value %.14g is out of range [%.14g..%.14g].",
-                  name ? name : "???", desc ? desc : "", default_value, d->minimum, d->maximum);
+                  name ? name : "(anonymous)", desc ? desc : "", default_value, d->minimum, d->maximum);
     }
     gwy_param_def_append(pardef, id, name, desc, &item);
 }
@@ -800,7 +802,7 @@ gwy_param_def_add_percentage(GwyParamDef *pardef,
     d->default_value = _gwy_param_def_rectify_double(&item, default_value);
     if (d->default_value != default_value) {
         g_warning("Double param %s (%s) default value %.14g is out of range [%.14g..%.14g].",
-                  name ? name : "???", desc ? desc : "", default_value, d->minimum, d->maximum);
+                  name ? name : "(anonymous)", desc ? desc : "", default_value, d->minimum, d->maximum);
     }
     gwy_param_def_append(pardef, id, name, desc, &item);
 }
@@ -817,6 +819,8 @@ gwy_param_def_add_percentage(GwyParamDef *pardef,
  * This is usually a parameter not stored in settings, but handled specially by taking the mask colour from the image
  * and then setting mask colours on the output.  The default value is the default mask colour.
  *
+ * See gwy_param_def_add_color() for a generic colour parameter.
+ *
  * See the introduction for @name and @desc lifetime rules.
  *
  * Since: 2.59
@@ -843,6 +847,43 @@ gwy_param_def_add_mask_color(GwyParamDef *pardef,
     gwy_param_def_append(pardef, id, name, desc, &item);
 }
 
+/**
+ * gwy_param_def_add_color:
+ * @pardef: A set of parameter definitions.
+ * @id: Parameter identifier.
+ * @name: Parameter name for settings.  Usually it is %NULL because the colour is taken from data, not settings.
+ * @desc: Parameter description which will be used for a label in GUI.  Usually %NULL for the default label.
+ * @has_alpha: %TRUE if semi-transparent colours are allowed; %FALSE for fully opaque colours.
+ * @default_value: Default value of the parameter.
+ *
+ * Defines a new colour parameter.
+ *
+ * See gwy_param_def_add_mask_color() for the mask colour parameter which is handled specially.
+ *
+ * See the introduction for @name and @desc lifetime rules.
+ *
+ * Since: 2.64
+ **/
+void
+gwy_param_def_add_color(GwyParamDef *pardef,
+                        gint id,
+                        const gchar *name,
+                        const gchar *desc,
+                        gboolean has_alpha,
+                        GwyRGBA default_value)
+{
+    GwyParamDefItem item;
+    GwyParamDefColor *c = &item.def.c;
+
+    if (!desc)
+        desc = _("Colo_r");
+    item.type = GWY_PARAM_COLOR;
+    c->is_mask = FALSE;
+    c->has_alpha = has_alpha;
+    c->default_value = _gwy_param_def_rectify_color(&item, default_value);
+    gwy_param_def_append(pardef, id, name, desc, &item);
+}
+
 /**
  * gwy_param_def_add_target_graph:
  * @pardef: A set of parameter definitions.
@@ -1176,14 +1217,14 @@ gwy_param_def_add_report_type(GwyParamDef *pardef,
         && style != GWY_RESULTS_EXPORT_TABULAR_DATA
         && style != GWY_RESULTS_EXPORT_FIXED_FORMAT) {
         g_warning("Report type param %s (%s) has invalid style (%u).",
-                  name ? name : "???", desc ? desc : "", style);
+                  name ? name : "(anonymous)", desc ? desc : "", style);
         style = GWY_RESULTS_EXPORT_PARAMETERS;
     }
     rt->style = style;
     rt->default_value = _gwy_param_def_rectify_report_type(&item, default_value);
     if (rt->default_value != default_value) {
         g_warning("Report type param %s (%s) default value %u is not among allowed values.",
-                  name ? name : "???", desc ? desc : "", default_value);
+                  name ? name : "(anonymous)", desc ? desc : "", default_value);
     }
     gwy_param_def_append(pardef, id, name, desc, &item);
 }
@@ -1288,7 +1329,7 @@ gwy_param_def_add_string(GwyParamDef *pardef,
     s->default_value = _gwy_param_def_rectify_string(&item, default_value);
     if (g_strcmp0(s->default_value, default_value)) {
         g_warning("String param %s (%s) default value \"%s\" does not rectify to itself but to \"%s\".",
-                  name ? name : "???", desc ? desc : "", default_value, s->default_value);
+                  name ? name : "(anonymous)", desc ? desc : "", default_value, s->default_value);
     }
     gwy_param_def_append(pardef, id, name, desc, &item);
 }
@@ -1362,7 +1403,7 @@ gwy_param_def_add_resource(GwyParamDef *pardef,
     res->default_value = g_intern_string(default_value);
     if (!gwy_inventory_get_item(inventory, res->default_value)) {
         g_warning("Resource param %s (%s) default value \"%s\" is not in the inventory.",
-                  name ? name : "???", desc ? desc : "", default_value);
+                  name ? name : "(anonymous)", desc ? desc : "", default_value);
     }
     gwy_param_def_append(pardef, id, name, desc, &item);
 }
@@ -1474,12 +1515,27 @@ static gint
 find_param_def(GwyParamDefPrivate *priv, gint id)
 {
     gint *ids = priv->ids;
-    gint i, n = priv->ndefs;
+    gint a, b, c, n = priv->ndefs;
 
-    for (i = 0; i < n; i++) {
-        if (ids[i] == id)
-            return i;
+    a = 0;
+    b = n-1;
+    if (id == ids[a])
+        return a;
+    if (id == ids[b])
+        return b;
+    if (id < ids[a] || id > ids[b])
+        return -1;
+
+    while (b - a > 1) {
+        c = (a + b)/2;
+        if (id == ids[c])
+            return c;
+        if (id < ids[c])
+            b = c;
+        else
+            a = c;
     }
+
     return -1;
 }
 
@@ -1493,13 +1549,52 @@ remove_current_user(gpointer user_data, GObject *where_the_object_was)
 }
 #endif
 
+static int
+compare_int(const void *pa, const void *pb)
+{
+    int a = *(const int*)pa;
+    int b = *(const int*)pb;
+
+    if (a < b)
+        return -1;
+    if (a > b)
+        return 1;
+    return 0;
+}
+
+/* Sort ids[] and defs[] in sync by ascending id. */
+static void
+sort_definitions(GwyParamDefPrivate *priv)
+{
+    GwyParamDefItem *newdefs;
+    guint i, n = priv->ndefs;
+    gint *idx = g_new(gint, 2*n);
+
+    for (i = 0; i < n; i++) {
+        idx[2*i] = priv->ids[i];
+        idx[2*i + 1] = i;
+    }
+
+    qsort(idx, n, 2*sizeof(gint), compare_int);
+
+    newdefs = g_new(GwyParamDefItem, n);
+    for (i = 0; i < n; i++) {
+        priv->ids[i] = idx[2*i];
+        newdefs[i] = priv->defs[idx[2*i + 1]];
+    }
+    g_free(idx);
+
+    GWY_SWAP(GwyParamDefItem*, priv->defs, newdefs);
+    g_free(newdefs);
+    priv->nalloc = n;
+}
+
 void
 _gwy_param_def_use(GwyParamDef *pardef, GwyParams *params)
 {
     GwyParamDefPrivate *priv = pardef->priv;
 
-    /* XXX: Here we could sort the definitions by id because they cannot change any more.  Then find_param_def() could
-     * use binary search. */
+    sort_definitions(priv);
     priv->is_used = TRUE;
 #ifdef DEBUG_USERS
     /* When presets are implemented using GwyParams then multiple GwyParams instances created for one GwyParamDef
@@ -1508,7 +1603,9 @@ _gwy_param_def_use(GwyParamDef *pardef, GwyParams *params)
         /* XXX: Hardcoded warning suppression for known modules with GwyParamResource presets and internal
          * parameter set duplication. */
         if (priv->users && !gwy_stramong(priv->function_name,
-                                         "rawfile", "terracefit", "graph_terraces", "logistic",
+                                         "rawfile", "imgexport", "pixmap",
+                                         "terracefit", "graph_terraces", "logistic",
+                                         "volume_psf",
                                          NULL)) {
             g_warning("Parameter definitions for %s are used multiple times.  "
                       "Check module function %s; it is probably leaking GwyParams objects!",
