@@ -1,5 +1,5 @@
 /*
- *  $Id: settings.c 24546 2021-11-26 10:44:01Z yeti-dn $
+ *  $Id: settings.c 25204 2023-01-25 15:03:59Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -150,7 +150,9 @@ gwy_app_settings_save(const gchar *filename,
  * @filename: A filename to read settings from.
  * @error: Location to store loading error to, or %NULL.
  *
- * Loads settings file.
+ * Initialises settings by loading a settings file.
+ *
+ * Any existing settings are discarded. Consider gwy_app_settings_merge_string() for piecewise settings creation.
  *
  * Returns: Whether it succeeded.  In either case you can call gwy_app_settings_get() then to obtain either the loaded
  *          settings or the old ones (if failed), or an empty #GwyContainer.
@@ -173,8 +175,7 @@ gwy_app_settings_load(const gchar *filename,
     }
 
     if (!size || !buffer) {
-        g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_EMPTY,
-                    _("File is empty."));
+        g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_EMPTY, _("File is empty."));
         g_free(buffer);
         return FALSE;
     }
@@ -189,8 +190,7 @@ gwy_app_settings_load(const gchar *filename,
     }
     new_settings = gwy_container_deserialize_from_text(buffer + strlen(magic_header));
     g_free(buffer);
-    if (!GWY_IS_CONTAINER(new_settings)) {
-        GWY_OBJECT_UNREF(new_settings);
+    if (!new_settings) {
         g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_CORRUPT,
                     _("File is corrupted, deserialization failed."));
         return FALSE;
@@ -207,14 +207,61 @@ gwy_app_settings_load(const gchar *filename,
     return TRUE;
 }
 
+/**
+ * gwy_app_settings_merge_string:
+ * @s: String contaning a part of Gwyddion settings file.
+ * @overwrite: %TRUE to replace existing settings; %FALSE to only set values which do not exist.
+ * @error: Location to store loading error to, or %NULL.
+ *
+ * Loads settings from a string and adds them to the global settings.
+ *
+ * The contents of string @s should look like a Gwyddion settings file, including the header line "Gwyddion Settings
+ * 1.0", even though it is optional.
+ *
+ * If no settings exist when this function is called, they are initialised like calling gwy_app_settings_get().
+ *
+ * Returns: Whether it succeeded.
+ *
+ * Since: 2.63
+ **/
+gboolean
+gwy_app_settings_merge_string(const gchar *s,
+                              gboolean overwrite,
+                              GError **error)
+{
+    GwyContainer *settings, *new_settings;
+    const gchar *s_to_parse;
+    gchar *buffer = NULL;
+
+    /* Do it first for determinisic behaviour. */
+    settings = gwy_app_settings_get();
+
+    if (!s || !*s)
+        return TRUE;
+
+    buffer = gwy_strkill(g_strdup(s), "\r");
+    s_to_parse = (g_str_has_prefix(buffer, magic_header) ? buffer + strlen(magic_header) : buffer);
+    new_settings = gwy_container_deserialize_from_text(s_to_parse);
+    g_free(buffer);
+    if (!new_settings) {
+        g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_CORRUPT,
+                    _("File is corrupted, deserialization failed."));
+        return FALSE;
+    }
+
+    gwy_container_transfer(new_settings, settings, "/", "/", overwrite);
+    g_object_unref(new_settings);
+
+    return TRUE;
+}
+
 /**
  * gwy_app_settings_create_config_dir:
  * @error: Location to store loading error to, or %NULL.
  *
  * Create gwyddion config directory.
  *
- * Returns: Whether it succeeded (also returns %TRUE if the directory already
- * exists).
+ * Returns: Whether it succeeded (also returns %TRUE if the directory already exists).
  **/
 gboolean
 gwy_app_settings_create_config_dir(GError **error)
@@ -319,11 +366,9 @@ gwy_app_settings_set_defaults(GwyContainer *settings)
         gwy_container_set_double_by_name(settings, "/mask/alpha", default_mask_color.a);
     }
     if (!gwy_container_contains_by_name(settings, "/app/gradients/preferred"))
-        gwy_container_set_string_by_name(settings, "/app/gradients/preferred", g_strdup(default_preferred_gradients));
-    if (!gwy_container_contains_by_name(settings, "/app/glmaterials/preferred")) {
-        gwy_container_set_string_by_name(settings, "/app/glmaterials/preferred",
-                                         g_strdup(default_preferred_gl_materials));
-    }
+        gwy_container_set_const_string_by_name(settings, "/app/gradients/preferred", default_preferred_gradients);
+    if (!gwy_container_contains_by_name(settings, "/app/glmaterials/preferred"))
+        gwy_container_set_const_string_by_name(settings, "/app/glmaterials/preferred", default_preferred_gl_materials);
     if (!gwy_container_contains_by_name(settings, "/app/toolbox/visible/graph"))
         gwy_container_set_boolean_by_name(settings, "/app/toolbox/visible/graph", TRUE);
     if (!gwy_container_contains_by_name(settings, "/app/toolbox/visible/proc"))
