@@ -1,22 +1,18 @@
 /*
- *  $Id: toolbox-spec.c 21345 2018-08-27 18:48:04Z yeti-dn $
- *  Copyright (C) 2016-2018 David Necas (Yeti).
+ *  $Id: toolbox-spec.c 24455 2021-11-03 13:29:04Z yeti-dn $
+ *  Copyright (C) 2016-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -37,19 +33,20 @@
 #define TOOLBOX_INDENT "  "
 
 static const GwyEnum modes[] = {
-    { N_("default"),         0,                   },
-    { N_("interactive"),     GWY_RUN_INTERACTIVE, },
-    { N_("non-interactive"), GWY_RUN_IMMEDIATE,   },
+    { "default",         0,                   },
+    { "interactive",     GWY_RUN_INTERACTIVE, },
+    { "non-interactive", GWY_RUN_IMMEDIATE,   },
 };
 
 static const GwyEnum action_types[] = {
-    { N_("empty"),   GWY_APP_ACTION_TYPE_PLACEHOLDER, },
-    { N_("builtin"), GWY_APP_ACTION_TYPE_BUILTIN,     },
-    { N_("proc"),    GWY_APP_ACTION_TYPE_PROC,        },
-    { N_("graph"),   GWY_APP_ACTION_TYPE_GRAPH,       },
-    { N_("volume"),  GWY_APP_ACTION_TYPE_VOLUME,      },
-    { N_("xyz"),     GWY_APP_ACTION_TYPE_XYZ,         },
-    { N_("tool"),    GWY_APP_ACTION_TYPE_TOOL,        },
+    { "empty",   GWY_APP_ACTION_TYPE_PLACEHOLDER, },
+    { "builtin", GWY_APP_ACTION_TYPE_BUILTIN,     },
+    { "proc",    GWY_APP_ACTION_TYPE_PROC,        },
+    { "graph",   GWY_APP_ACTION_TYPE_GRAPH,       },
+    { "volume",  GWY_APP_ACTION_TYPE_VOLUME,      },
+    { "xyz",     GWY_APP_ACTION_TYPE_XYZ,         },
+    { "cmap",    GWY_APP_ACTION_TYPE_CMAP,        },
+    { "tool",    GWY_APP_ACTION_TYPE_TOOL,        },
 };
 
 static GwyToolboxSpec* gwy_toolbox_parse              (const gchar *ui,
@@ -116,18 +113,15 @@ toolbox_ui_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
                     g_warning("Ignoring non-identifier id=\"%s\".", attval);
             }
             else if (gwy_strequal(attname, "title")) {
-                if ((l = strlen(attval))
-                    && g_utf8_validate(attval, l, NULL))
+                if ((l = strlen(attval)) && g_utf8_validate(attval, l, NULL))
                     title = attval;
                 else
                     g_warning("Ignoring invalid group title.");
             }
             else if (gwy_strequal(attname, "translatable")) {
-                if (g_ascii_strcasecmp(attval, "true") == 0
-                    || g_ascii_strcasecmp(attval, "yes") == 0)
+                if (g_ascii_strcasecmp(attval, "true") == 0 || g_ascii_strcasecmp(attval, "yes") == 0)
                     translatable = TRUE;
-                else if (g_ascii_strcasecmp(attval, "false") == 0
-                    || g_ascii_strcasecmp(attval, "no") == 0)
+                else if (g_ascii_strcasecmp(attval, "false") == 0 || g_ascii_strcasecmp(attval, "no") == 0)
                     translatable = FALSE;
                 else
                     g_warning("Ignoring invalid group translatable attribute.");
@@ -170,12 +164,9 @@ toolbox_ui_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
 
         for (i = 0; (attname = attribute_names[i]); i++) {
             attval = attribute_values[i];
-            if (gwy_strequal(attname, "type")
-                && (tmptype = gwy_toolbox_find_action_type(attval)) != -1)
+            if (gwy_strequal(attname, "type") && (tmptype = gwy_toolbox_find_action_type(attval)) != -1)
                 type = tmptype;
-            else if (gwy_strequal(attname, "run")
-                     && (tmpmode = gwy_toolbox_find_mode(attval))
-                        != (GwyRunType)-1)
+            else if (gwy_strequal(attname, "run") && (tmpmode = gwy_toolbox_find_mode(attval)) != (GwyRunType)-1)
                 mode = tmpmode;
             else if (gwy_strequal(attname, "function"))
                 function = attval;
@@ -198,11 +189,8 @@ toolbox_ui_start_element(G_GNUC_UNUSED GMarkupParseContext *context,
             }
             spec->seen_tool_placeholder = TRUE;
         }
-        if (type == GWY_APP_ACTION_TYPE_TOOL
-            && function
-            && !gwy_strisident(function, "_-", NULL)) {
-            g_warning("Ignoring tool item with invalid function=\"%s\"",
-                      function);
+        if (type == GWY_APP_ACTION_TYPE_TOOL && function && !gwy_strisident(function, "_-", NULL)) {
+            g_warning("Ignoring tool item with invalid function=\"%s\"", function);
             return;
         }
         if (type == GWY_APP_ACTION_TYPE_PLACEHOLDER
@@ -262,8 +250,7 @@ gwy_toolbox_spec_free(GwyToolboxSpec *spec)
 
     if (group) {
         for (i = 0; i < group->len; i++) {
-            GwyToolboxGroupSpec *gspec = &g_array_index(group,
-                                                        GwyToolboxGroupSpec, i);
+            GwyToolboxGroupSpec *gspec = &g_array_index(group, GwyToolboxGroupSpec, i);
 
             g_free(gspec->name);
             if (gspec->item)
@@ -362,14 +349,12 @@ gwy_toolbox_spec_move_item(GwyToolboxSpec *spec,
     ispec = g_array_index(gspec->item, GwyToolboxItemSpec, j);
     if (up) {
         g_return_if_fail(j > 0);
-        g_array_index(gspec->item, GwyToolboxItemSpec, j)
-            = g_array_index(gspec->item, GwyToolboxItemSpec, j-1);
+        g_array_index(gspec->item, GwyToolboxItemSpec, j) = g_array_index(gspec->item, GwyToolboxItemSpec, j-1);
         g_array_index(gspec->item, GwyToolboxItemSpec, j-1) = ispec;
     }
     else {
         g_return_if_fail(j+1 < gspec->item->len);
-        g_array_index(gspec->item, GwyToolboxItemSpec, j)
-            = g_array_index(gspec->item, GwyToolboxItemSpec, j+1);
+        g_array_index(gspec->item, GwyToolboxItemSpec, j) = g_array_index(gspec->item, GwyToolboxItemSpec, j+1);
         g_array_index(gspec->item, GwyToolboxItemSpec, j+1) = ispec;
     }
 }
@@ -385,14 +370,12 @@ gwy_toolbox_spec_move_group(GwyToolboxSpec *spec,
     gspec = g_array_index(spec->group, GwyToolboxGroupSpec, i);
     if (up) {
         g_return_if_fail(i > 0);
-        g_array_index(spec->group, GwyToolboxGroupSpec, i)
-            = g_array_index(spec->group, GwyToolboxGroupSpec, i-1);
+        g_array_index(spec->group, GwyToolboxGroupSpec, i) = g_array_index(spec->group, GwyToolboxGroupSpec, i-1);
         g_array_index(spec->group, GwyToolboxGroupSpec, i-1) = gspec;
     }
     else {
         g_return_if_fail(i+1 < spec->group->len);
-        g_array_index(spec->group, GwyToolboxGroupSpec, i)
-            = g_array_index(spec->group, GwyToolboxGroupSpec, i+1);
+        g_array_index(spec->group, GwyToolboxGroupSpec, i) = g_array_index(spec->group, GwyToolboxGroupSpec, i+1);
         g_array_index(spec->group, GwyToolboxGroupSpec, i+1) = gspec;
     }
 }
@@ -488,8 +471,7 @@ gwy_save_toolbox_ui(GwyToolboxSpec *spec, GError **error)
     for (i = 0; i < group->len; i++) {
         gspec = &g_array_index(group, GwyToolboxGroupSpec, i);
         g_string_append_printf(xml, "%s<group id='%s' title='%s'",
-                               TOOLBOX_INDENT,
-                               g_quark_to_string(gspec->id), gspec->name);
+                               TOOLBOX_INDENT, g_quark_to_string(gspec->id), gspec->name);
         if (gspec->translatable)
             g_string_append(xml, " translatable='yes'");
         g_string_append(xml, ">\n");
@@ -498,19 +480,15 @@ gwy_save_toolbox_ui(GwyToolboxSpec *spec, GError **error)
         for (j = 0; j < item->len; j++) {
             ispec = &g_array_index(item, GwyToolboxItemSpec, j);
             g_string_append_printf(xml, "%s<item type='%s'",
-                                   TOOLBOX_INDENT TOOLBOX_INDENT,
-                                   gwy_toolbox_action_type_name(ispec->type));
+                                   TOOLBOX_INDENT TOOLBOX_INDENT, gwy_toolbox_action_type_name(ispec->type));
             if (ispec->function) {
-                g_string_append_printf(xml, " function='%s'",
-                                       g_quark_to_string(ispec->function));
+                g_string_append_printf(xml, " function='%s'", g_quark_to_string(ispec->function));
             }
             if (ispec->icon) {
-                g_string_append_printf(xml, " icon='%s'",
-                                       g_quark_to_string(ispec->icon));
+                g_string_append_printf(xml, " icon='%s'", g_quark_to_string(ispec->icon));
             }
             if (ispec->mode) {
-                g_string_append_printf(xml, " run='%s'",
-                                       gwy_toolbox_mode_name(ispec->mode));
+                g_string_append_printf(xml, " run='%s'", gwy_toolbox_mode_name(ispec->mode));
             }
             g_string_append(xml, "/>\n");
         }
@@ -518,21 +496,17 @@ gwy_save_toolbox_ui(GwyToolboxSpec *spec, GError **error)
     }
     g_string_append(xml, "</toolbox>\n");
 
-    filename = g_build_filename(gwy_get_user_dir(), "ui", TOOLBOX_UI_FILE_NAME,
-                                NULL);
+    filename = g_build_filename(gwy_get_user_dir(), "ui", TOOLBOX_UI_FILE_NAME, NULL);
     fh = gwy_fopen(filename, "w");
     g_free(filename);
     if (!fh) {
-        g_set_error(error,
-                    GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
-                    _("Cannot open file for writing: %s."),
-                    g_strerror(errno));
+        g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
+                    _("Cannot open file for writing: %s."), g_strerror(errno));
         g_string_free(xml, TRUE);
         return FALSE;
     }
     if (fwrite(xml->str, 1, xml->len, fh) != xml->len) {
-        g_set_error(error,
-                    GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
+        g_set_error(error, GWY_APP_SETTINGS_ERROR, GWY_APP_SETTINGS_ERROR_FILE,
                     _("Cannot write to file: %s."), g_strerror(errno));
         fclose(fh);
         g_string_free(xml, TRUE);
@@ -555,8 +529,7 @@ gwy_toolbox_spec_duplicate(GwyToolboxSpec *spec)
     dup->seen_tool_placeholder = spec->seen_tool_placeholder;
     dup->group = g_array_new(FALSE, FALSE, sizeof(GwyToolboxGroupSpec));
     for (i = 0; i < spec->group->len; i++) {
-        GwyToolboxGroupSpec *gspec = &g_array_index(spec->group,
-                                                    GwyToolboxGroupSpec, i);
+        GwyToolboxGroupSpec *gspec = &g_array_index(spec->group, GwyToolboxGroupSpec, i);
         GwyToolboxGroupSpec gdup;
 
         gdup.item = g_array_new(FALSE, FALSE, sizeof(GwyToolboxItemSpec));
@@ -566,8 +539,7 @@ gwy_toolbox_spec_duplicate(GwyToolboxSpec *spec)
         g_array_append_val(dup->group, gdup);
 
         for (j = 0; j < gspec->item->len; j++) {
-            GwyToolboxItemSpec *ispec = &g_array_index(gspec->item,
-                                                       GwyToolboxItemSpec, j);
+            GwyToolboxItemSpec *ispec = &g_array_index(gspec->item, GwyToolboxItemSpec, j);
             GwyToolboxItemSpec idup;
 
             idup.type = ispec->type;
@@ -641,6 +613,8 @@ gwy_toolbox_action_nice_name(GwyAppActionType type, const gchar *name)
         menupath = gwy_volume_func_get_menu_path(name);
     else if (type == GWY_APP_ACTION_TYPE_XYZ)
         menupath = gwy_xyz_func_get_menu_path(name);
+    else if (type == GWY_APP_ACTION_TYPE_CMAP)
+        menupath = gwy_curve_map_func_get_menu_path(name);
 
     if (menupath) {
         const gchar *p;
@@ -673,12 +647,12 @@ gwy_toolbox_action_stock_id(GwyAppActionType type, const gchar *name)
         return gwy_volume_func_get_stock_id(name);
     if (type == GWY_APP_ACTION_TYPE_XYZ)
         return gwy_xyz_func_get_stock_id(name);
-    if (type == GWY_APP_ACTION_TYPE_TOOL && name) {
-        if (name) {
-            GType gtype = g_type_from_name(name);
-            GwyToolClass *tool_class = g_type_class_peek(gtype);
-            return gwy_tool_class_get_stock_id(tool_class);
-        }
+    if (type == GWY_APP_ACTION_TYPE_CMAP)
+        return gwy_curve_map_func_get_stock_id(name);
+    if (type == GWY_APP_ACTION_TYPE_TOOL) {
+        GType gtype = g_type_from_name(name);
+        GwyToolClass *tool_class = g_type_class_peek(gtype);
+        return gwy_tool_class_get_stock_id(tool_class);
     }
     if (type == GWY_APP_ACTION_TYPE_BUILTIN) {
         const GwyToolboxBuiltinSpec* spec;
@@ -702,13 +676,12 @@ gwy_toolbox_action_detail(GwyAppActionType type, const gchar *name)
         return gwy_volume_func_get_tooltip(name);
     if (type == GWY_APP_ACTION_TYPE_XYZ)
         return gwy_xyz_func_get_tooltip(name);
-    if (type == GWY_APP_ACTION_TYPE_TOOL && name) {
-        if (name) {
-            GType gtype = g_type_from_name(name);
-            GwyToolClass *tool_class = g_type_class_peek(gtype);
-            return gwy_tool_class_get_tooltip(tool_class);
-        }
-        return _("All tools not placed explicitly go here.");
+    if (type == GWY_APP_ACTION_TYPE_CMAP)
+        return gwy_curve_map_func_get_tooltip(name);
+    if (type == GWY_APP_ACTION_TYPE_TOOL) {
+        GType gtype = g_type_from_name(name);
+        GwyToolClass *tool_class = g_type_class_peek(gtype);
+        return gwy_tool_class_get_tooltip(tool_class);
     }
     if (type == GWY_APP_ACTION_TYPE_BUILTIN) {
         const GwyToolboxBuiltinSpec* spec;
@@ -730,6 +703,8 @@ gwy_toolbox_action_run_modes(GwyAppActionType type, const gchar *name)
         return gwy_volume_func_get_run_types(name);
     if (type == GWY_APP_ACTION_TYPE_XYZ)
         return gwy_xyz_func_get_run_types(name);
+    if (type == GWY_APP_ACTION_TYPE_CMAP)
+        return gwy_curve_map_func_get_run_types(name);
 
     return 0;
 }
@@ -751,4 +726,4 @@ gwy_app_menu_canonicalize_label(gchar *label)
         label[j-3] = '\0';
 }
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
