@@ -1,5 +1,5 @@
 /*
- *  $Id: gwygradient.c 21680 2018-11-26 10:39:39Z yeti-dn $
+ *  $Id: gwygradient.c 25527 2023-06-30 14:38:03Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -967,33 +967,15 @@ gwy_gradient_dump(GwyResource *resource,
                   GString *str)
 {
     GwyGradient *gradient;
-    GwyGradientPoint *pt;
-    gchar buffer[G_ASCII_DTOSTR_BUF_SIZE];
-    guint i;
 
     g_return_if_fail(GWY_IS_GRADIENT(resource));
     gradient = GWY_GRADIENT(resource);
     g_return_if_fail(gradient->points->len > 0);
 
-    for (i = 0; i < gradient->points->len; i++) {
-        pt = &g_array_index(gradient->points, GwyGradientPoint, i);
-        /* this is ugly.  I hate locales */
-        g_ascii_formatd(buffer, sizeof(buffer), "%.6g", pt->x);
-        g_string_append(str, buffer);
-        g_string_append_c(str, ' ');
-        g_ascii_formatd(buffer, sizeof(buffer), "%.6g", pt->color.r);
-        g_string_append(str, buffer);
-        g_string_append_c(str, ' ');
-        g_ascii_formatd(buffer, sizeof(buffer), "%.6g", pt->color.g);
-        g_string_append(str, buffer);
-        g_string_append_c(str, ' ');
-        g_ascii_formatd(buffer, sizeof(buffer), "%.6g", pt->color.b);
-        g_string_append(str, buffer);
-        g_string_append_c(str, ' ');
-        g_ascii_formatd(buffer, sizeof(buffer), "%g", pt->color.a);
-        g_string_append(str, buffer);
-        g_string_append_c(str, '\n');
-    }
+    g_assert(sizeof(GwyGradientPoint) == 5*sizeof(gdouble));
+    gwy_append_doubles_to_gstring(str,
+                                  (const gdouble*)&g_array_index(gradient->points, GwyGradientPoint, 0),
+                                  gradient->points->len, 6, " ", TRUE);
 }
 
 static GwyResource*
@@ -1002,59 +984,24 @@ gwy_gradient_parse(const gchar *text,
 {
     GwyGradient *gradient = NULL;
     GwyGradientClass *klass;
-    GArray *points = NULL;
-    GwyGradientPoint pt;
-    gchar *str, *p, *line, *end;
+    gint npoints = -1, ncols = 5;
+    gdouble *data;
 
     g_return_val_if_fail(text, NULL);
     klass = g_type_class_peek(GWY_TYPE_GRADIENT);
     g_return_val_if_fail(klass, NULL);
 
-    p = str = g_strdup(text);
-    points = g_array_new(FALSE, FALSE, sizeof(GwyGradientPoint));
-    while (TRUE) {
-        if (!(line = gwy_str_next_line(&p)))
-            break;
-        g_strstrip(line);
-        if (!line[0] || line[0] == '#')
-            continue;
-        /* this is ugly.  I hate locales */
-        pt.x = g_ascii_strtod(line, &end);
-        if (end == line)
-            break;
-        line = end;
-        pt.color.r = g_ascii_strtod(line, &end);
-        if (end == line)
-            break;
-        line = end;
-        pt.color.g = g_ascii_strtod(line, &end);
-        if (end == line)
-            break;
-        line = end;
-        pt.color.b = g_ascii_strtod(line, &end);
-        if (end == line)
-            break;
-        line = end;
-        pt.color.a = g_ascii_strtod(line, &end);
-        if (end == line)
-            break;
-
-        g_array_append_val(points, pt);
-    }
-    if (line) {
+    g_assert(sizeof(GwyGradientPoint) == 5*sizeof(gdouble));
+    if (!(data = gwy_parse_doubles(text, NULL, 0, &npoints, &ncols, NULL, NULL))) {
         g_warning("Cannot parse color points.");
         goto fail;
     }
 
-    gradient = gwy_gradient_new("",
-                                points->len, (GwyGradientPoint*)points->data,
-                                is_const);
+    gradient = gwy_gradient_new("", npoints, (GwyGradientPoint*)data, is_const);
     gwy_gradient_sanitize(gradient);
 
 fail:
-    if (points)
-        g_array_free(points, TRUE);
-    g_free(str);
+    g_free(data);
     return (GwyResource*)gradient;
 }
 
