@@ -1,5 +1,5 @@
 /*
- *  $Id: gwypixfield.c 21661 2018-11-21 13:28:44Z yeti-dn $
+ *  $Id: gwypixfield.c 22340 2019-07-25 10:23:59Z yeti-dn $
  *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -20,7 +20,8 @@
  */
 
 #include "config.h"
-#include <libgwyddion/gwyddion.h>
+#include <libgwyddion/gwymacros.h>
+#include <libgwyddion/gwythreads.h>
 #include <libprocess/stats.h>
 #include <gwypixfield.h>
 #include "libgwyddion/gwyomp.h"
