@@ -1,6 +1,6 @@
 /*
- *  $Id: gwyddion.h 21661 2018-11-21 13:28:44Z yeti-dn $
- *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwyddion.h 22334 2019-07-25 08:18:59Z yeti-dn $
+ *  Copyright (C) 2003-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -44,13 +44,12 @@
 #include <libgwyddion/gwydebugobjects.h>
 #include <libgwyddion/gwyexpr.h>
 #include <libgwyddion/gwystringlist.h>
+#include <libgwyddion/gwythreads.h>
 #include <libgwyddion/gwyversion.h>
 
 G_BEGIN_DECLS
 
 void     gwy_type_init          (void);
-gboolean gwy_threads_are_enabled(void)              G_GNUC_PURE;
-void     gwy_threads_set_enabled(gboolean setting);
 
 G_END_DECLS
 
