@@ -1,22 +1,18 @@
 /*
- *  $Id: gwyexpr.c 22340 2019-07-25 10:23:59Z yeti-dn $
+ *  $Id: gwyexpr.c 25858 2023-10-14 07:50:48Z yeti-dn $
  *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -142,6 +138,9 @@ struct _GwyExpr {
     GString *expr;
     GScanner *scanner;
     GHashTable *constants;
+    /* String stoage like intern strings/quarks, but private to the expression. Avoids string duplication and
+     * especially having to remember when to free them. */
+    GHashTable *strings;
     /* Variables */
     GPtrArray *identifiers;    /* variable names */
     const gdouble *variables;    /* user-filled variable values */
@@ -166,11 +165,8 @@ gwy_spow(gdouble x, gdouble p)
     return -pow(-x, p);
 }
 
-#define make_function_1_1(name) \
-    static void gwy_expr_##name(gdouble **s) { **s = name(**s); }
-
-#define make_function_2_1(name) \
-    static void gwy_expr_##name(gdouble **s) { --*s; **s = name(*(*s+1), **s); }
+#define make_function_1_1(name) static void gwy_expr_##name(gdouble **s) { **s = name(**s); }
+#define make_function_2_1(name) static void gwy_expr_##name(gdouble **s) { --*s; **s = name(*(*s+1), **s); }
 
 make_function_1_1(sqrt);
 make_function_1_1(cbrt);
@@ -364,7 +360,7 @@ ensure_error_domain(G_GNUC_UNUSED gpointer arg)
 /**
  * gwy_expr_error_quark:
  *
- * Returns error domain for expression parsin and evaluation.
+ * Returns error domain for expression parsing and evaluation.
  *
  * See and use %GWY_EXPR_ERROR.
  *
@@ -413,10 +409,9 @@ gwy_expr_stack_interpret(GwyExpr *expr)
  * gwy_expr_stack_interpret_vectors:
  * @expr: An expression.
  * @n: The lenght of @result and of @data member arrays, that is vector length.
- * @data: An array of arrays of length @n.  The arrays correspond to expression
- *        variables in gwy_expr_execute().  Zeroth array can be %NULL.
- * @result: An array of length @n to store computation results to.  It may be
- *          one of those in @data.
+ * @data: An array of arrays of length @n.  The arrays correspond to expression variables in gwy_expr_execute().
+ *        The zeroth array can be %NULL.
+ * @result: An array of length @n to store computation results to.  It may be one of those in @data.
  *
  * Performs actual vectorized stack interpretation.
  *
@@ -428,9 +423,8 @@ gwy_expr_stack_interpret_vectors(GwyExpr *expr,
                                  const gdouble **data,
                                  gdouble *result)
 {
-    /* XXX: We cannot have default(none) here due to incompatibility in OpenMP
-     * specification.  Some versions then require shared(call_table), whereas
-     * some forbid it. */
+    /* XXX: We cannot have default(none) here due to incompatibility in OpenMP specification.  Some versions require
+     * shared(call_table), whereas some forbid it. */
 #ifdef _OPENMP
 #pragma omp parallel if(gwy_threads_are_enabled()) \
             shared(expr,n,data,result)
@@ -475,8 +469,7 @@ gwy_expr_stack_check_executability(GwyExpr *expr)
 
     nval = max = 0;
     for (i = 0; i < expr->in; i++) {
-        if (expr->input[i].type == GWY_EXPR_CODE_CONSTANT
-            || (gint)expr->input[i].type < 0) {
+        if (expr->input[i].type == GWY_EXPR_CODE_CONSTANT || (gint)expr->input[i].type < 0) {
             nval++;
         }
         else if (expr->input[i].type > 0) {
@@ -509,17 +502,13 @@ gwy_expr_stack_print(GwyExpr *expr)
     for (i = 0; i < expr->in; i++) {
         code = expr->input + i;
         if ((gint)code->type > 0) {
-            g_print("#%u Function %s\n",
-                    i, call_table[code->type].name);
+            g_print("#%u Function %s\n", i, call_table[code->type].name);
         }
         else if ((gint)code->type < 0) {
-            g_print("#%u Argument %s\n",
-                    i, (gchar*)g_ptr_array_index(expr->identifiers,
-                                                 -(gint)code->type));
+            g_print("#%u Argument %s\n", i, (gchar*)g_ptr_array_index(expr->identifiers, -(gint)code->type));
         }
         else
-            g_print("#%u Constant %g\n",
-                    i, code->value);
+            g_print("#%u Constant %g\n", i, code->value);
     }
 }
 
@@ -529,9 +518,8 @@ gwy_expr_stack_print(GwyExpr *expr)
  *
  * Folds foldable constants in compiled op code representation.
  *
- * This is done by selective execution, that is we go through the stack like
- * in gwy_expr_stack_interpret(), but only execute function calls on constant
- * while copying operands and operations we cannot carry out right away.
+ * This is done by selective execution, that is we go through the stack like in gwy_expr_stack_interpret(), but only
+ * execute function calls on constant while copying operands and operations we cannot carry out right away.
  *
  * Only directly foldable constants can folded this way:
  *   1+2+3+4+x is folded to 10+x, but
@@ -790,9 +778,8 @@ gwy_expr_token_new0(GwyExpr *expr)
  *
  * Actually frees token list.
  *
- * Assumes each token is either unconverted scanner token, or it has non-empty
- * RPN block.  In either case it frees the RPN block, or strings of uncoverted
- * scanner tokens.
+ * Assumes each token is either unconverted scanner token, or it has non-empty RPN block.  In either case it frees the
+ * RPN block, or strings of uncoverted scanner tokens.
  **/
 static void
 gwy_expr_token_list_delete(GwyExpr *expr,
@@ -801,28 +788,28 @@ gwy_expr_token_list_delete(GwyExpr *expr,
     GwyExprToken *t;
 
     for (t = tokens; t; t = t->next) {
-        if (t->rpn_block) {
-            expr->reservoir = gwy_expr_token_list_concat(t->rpn_block,
-                                                         expr->reservoir);
-        }
-        else {
-            switch (t->token) {
-                case G_TOKEN_STRING:
-                case G_TOKEN_IDENTIFIER:
-                case G_TOKEN_IDENTIFIER_NULL:
-                case G_TOKEN_COMMENT_SINGLE:
-                case G_TOKEN_COMMENT_MULTI:
-                g_free(t->value.v_string);
-                break;
-
-                default:
-                break;
-            }
-        }
+        if (t->rpn_block)
+            expr->reservoir = gwy_expr_token_list_concat(t->rpn_block, expr->reservoir);
     }
     expr->reservoir = gwy_expr_token_list_concat(tokens, expr->reservoir);
 }
 
+/* NB: The result is gchar because it is technically modifiable and tends to appear in gchar* variables, but the
+ * caller should not really modify it. */
+static gchar*
+gwy_expr_intern_string(GwyExpr *expr, gchar *s)
+{
+    gchar *s_intern;
+
+    if ((s_intern = g_hash_table_lookup(expr->strings, s)))
+        return s_intern;
+
+    s_intern = g_strdup(s);
+    g_hash_table_insert(expr->strings, s_intern, s_intern);
+
+    return s_intern;
+}
+
 /****************************************************************************
  *
  *  Infix -> RPN convertor, tokenizer
@@ -872,15 +859,12 @@ gwy_expr_scan_tokens(GwyExpr *expr,
             t->token = token;
             t->value = expr->scanner->value;
             tokens = gwy_expr_token_list_prepend(tokens, t);
-            /* TODO: steal token value from scanner to avoid string duplication
-             * and freeing */
-            scanner->value.v_string = NULL;
-            scanner->token = G_TOKEN_NONE;
+            if (token == G_TOKEN_IDENTIFIER)
+                t->value.v_identifier = gwy_expr_intern_string(expr, t->value.v_identifier);
             break;
 
             default:
-            g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_INVALID_TOKEN,
-                        _("Invalid token"));
+            g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_INVALID_TOKEN, _("Invalid token"));
             gwy_expr_token_list_delete(expr, tokens);
             return FALSE;
             break;
@@ -888,8 +872,7 @@ gwy_expr_scan_tokens(GwyExpr *expr,
     }
 
     if (!tokens) {
-        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_EMPTY,
-                    _("Empty expression"));
+        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_EMPTY, _("Empty expression"));
         return FALSE;
     }
 
@@ -933,9 +916,7 @@ gwy_expr_rectify_token_list(GwyExpr *expr)
                           && prev->token != G_TOKEN_RIGHT_PAREN)) {
                 prev = t;
                 t = t->next;
-                expr->tokens = gwy_expr_token_list_delete_token(expr,
-                                                                expr->tokens,
-                                                                prev);
+                expr->tokens = gwy_expr_token_list_delete_token(expr, expr->tokens, prev);
             }
             else
                 t = t->next;
@@ -951,8 +932,7 @@ gwy_expr_rectify_token_list(GwyExpr *expr)
                          || prev->token == G_TOKEN_IDENTIFIER)) {
                 prev = gwy_expr_token_new0(expr);
                 prev->token = '*';
-                expr->tokens = gwy_expr_token_list_insert(expr->tokens,
-                                                          t, prev);
+                expr->tokens = gwy_expr_token_list_insert(expr->tokens, t, prev);
             }
             t = t->next;
             break;
@@ -989,8 +969,7 @@ gwy_expr_initialize_scanner(GwyExpr *expr)
     for (i = 1; i < G_N_ELEMENTS(call_table); i++) {
         if (!call_table[i].name || !g_ascii_isalpha(call_table[i].name[0]))
             continue;
-        g_scanner_scope_add_symbol(expr->scanner, GWY_EXPR_SCOPE_GLOBAL,
-                                   call_table[i].name, GUINT_TO_POINTER(i));
+        g_scanner_scope_add_symbol(expr->scanner, GWY_EXPR_SCOPE_GLOBAL, call_table[i].name, GUINT_TO_POINTER(i));
     }
     g_scanner_set_scope(expr->scanner, GWY_EXPR_SCOPE_GLOBAL);
     expr->scanner->input_name = "expression";
@@ -1027,8 +1006,8 @@ gwy_expr_parse(GwyExpr *expr,
  *
  * Converts constants to single-items RPN lists and indexes identifiers.
  *
- * %G_TOKEN_IDENTIFIER strings are freed, they are converted to single-item
- * RPN lists too, with type as minus index in returned array of name.
+ * %G_TOKEN_IDENTIFIER strings are freed, they are converted to single-item RPN lists too, with type as minus index in
+ * returned array of name.
  *
  * Modifies @expr->tokens and fills @expr->identifiers.
  *
@@ -1063,8 +1042,7 @@ gwy_expr_transform_values(GwyExpr *expr)
 
         if (expr->constants) {
             if ((quark = g_quark_try_string(t->value.v_identifier))
-                && (cval = g_hash_table_lookup(expr->constants,
-                                               GUINT_TO_POINTER(quark)))) {
+                && (cval = g_hash_table_lookup(expr->constants, GUINT_TO_POINTER(quark)))) {
                 code = gwy_expr_token_new0(expr);
                 code->token = GWY_EXPR_CODE_CONSTANT;
                 code->value.v_float = *cval;
@@ -1073,8 +1051,7 @@ gwy_expr_transform_values(GwyExpr *expr)
             }
         }
         for (i = 1; i < expr->identifiers->len; i++) {
-            if (strcmp(t->value.v_identifier,
-                       g_ptr_array_index(expr->identifiers, i)) == 0) {
+            if (strcmp(t->value.v_identifier, g_ptr_array_index(expr->identifiers, i)) == 0) {
                 g_free(t->value.v_identifier);
                 break;
             }
@@ -1093,8 +1070,7 @@ gwy_expr_transform_values(GwyExpr *expr)
  * gwy_expr_transform_infix_ops:
  * @expr: An expression.
  * @tokens: A token list.
- * @right_to_left: %TRUE to process operators right to left, %FALSE to
- *                 left to right.
+ * @right_to_left: %TRUE to process operators right to left, %FALSE to left to right.
  * @operators: String containing one-letter operators to convert.
  * @codes: Opcodes corresponding to @operators.
  * @err: Location to store conversion error to.
@@ -1148,8 +1124,7 @@ gwy_expr_transform_infix_ops(GwyExpr *expr,
         code = gwy_expr_token_new0(expr);
         code->token = codes[i];
         prev->rpn_block = gwy_expr_token_list_concat(prev->rpn_block, code);
-        prev->rpn_block = gwy_expr_token_list_concat(next->rpn_block,
-                                                     prev->rpn_block);
+        prev->rpn_block = gwy_expr_token_list_concat(next->rpn_block, prev->rpn_block);
         prev->token = G_TOKEN_NONE;
         t = prev;
         tokens = gwy_expr_token_list_delete_token(expr, tokens, t->next);
@@ -1186,16 +1161,14 @@ gwy_expr_transform_functions(GwyExpr *expr,
         /* Check arguments */
         for (i = 0, arg = t->next; i < nargs; i++, arg = arg->next) {
             if (!arg) {
-                g_set_error(err, GWY_EXPR_ERROR,
-                            GWY_EXPR_ERROR_MISSING_ARGUMENT,
+                g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_MISSING_ARGUMENT,
                 /* TRANSLATORS: %s is replaced with sin, cos, sqrt, etc. */
                             _("Missing argument of %s"), call_table[func].name);
                 gwy_expr_token_list_delete(expr, tokens);
                 return NULL;
             }
             if (!arg->rpn_block) {
-                g_set_error(err, GWY_EXPR_ERROR,
-                            GWY_EXPR_ERROR_INVALID_ARGUMENT,
+                g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_INVALID_ARGUMENT,
                 /* TRANSLATORS: %s is replaced with sin, cos, sqrt, etc. */
                             _("Invalid argument of %s"), call_table[func].name);
                 gwy_expr_token_list_delete(expr, tokens);
@@ -1211,8 +1184,7 @@ gwy_expr_transform_functions(GwyExpr *expr,
         for (i = 0; i < nargs; i++) {
             arg = t->next;
             t->token = G_TOKEN_NONE;
-            t->rpn_block = gwy_expr_token_list_concat(arg->rpn_block,
-                                                      t->rpn_block);
+            t->rpn_block = gwy_expr_token_list_concat(arg->rpn_block, t->rpn_block);
             tokens = gwy_expr_token_list_delete_token(expr, tokens, arg);
         }
     }
@@ -1235,15 +1207,9 @@ gwy_expr_transform_to_rpn_real(GwyExpr *expr,
                                GwyExprToken *tokens,
                                GError **err)
 {
-    GwyExprOpCode pow_operators[] = {
-        GWY_EXPR_CODE_POWER,
-    };
-    GwyExprOpCode mult_operators[] = {
-        GWY_EXPR_CODE_MULTIPLY, GWY_EXPR_CODE_DIVIDE, GWY_EXPR_CODE_MODULO,
-    };
-    GwyExprOpCode add_operators[] = {
-        GWY_EXPR_CODE_ADD, GWY_EXPR_CODE_SUBTRACT,
-    };
+    GwyExprOpCode pow_operators[] = { GWY_EXPR_CODE_POWER, };
+    GwyExprOpCode mult_operators[] = { GWY_EXPR_CODE_MULTIPLY, GWY_EXPR_CODE_DIVIDE, GWY_EXPR_CODE_MODULO, };
+    GwyExprOpCode add_operators[] = { GWY_EXPR_CODE_ADD, GWY_EXPR_CODE_SUBTRACT, };
     GwyExprToken *t, *subblock, *remaindr = NULL;
 
     if (!tokens) {
@@ -1253,8 +1219,7 @@ gwy_expr_transform_to_rpn_real(GwyExpr *expr,
     expr->level++;
 
     if (tokens->token != G_TOKEN_LEFT_PAREN) {
-        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_OPENING_PARENTHESIS,
-                    _("Missing opening parenthesis"));
+        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_OPENING_PARENTHESIS, _("Missing opening parenthesis"));
         goto FAIL;
     }
     tokens = gwy_expr_token_list_delete_token(expr, tokens, tokens);
@@ -1295,13 +1260,11 @@ gwy_expr_transform_to_rpn_real(GwyExpr *expr,
     }
     /* missing right parenthesis or empty parentheses */
     if (!t) {
-        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_CLOSING_PARENTHESIS,
-                    _("Missing closing parenthesis"));
+        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_CLOSING_PARENTHESIS, _("Missing closing parenthesis"));
         goto FAIL;
     }
     if (!tokens) {
-        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_EMPTY_PARENTHESES,
-                    _("Empty parentheses"));
+        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_EMPTY_PARENTHESES, _("Empty parentheses"));
         goto FAIL;
     }
 
@@ -1309,8 +1272,7 @@ gwy_expr_transform_to_rpn_real(GwyExpr *expr,
     for (t = tokens; t; t = t->next) {
         if (t->token == G_TOKEN_COMMA) {
             if (!t->next || !t->prev) {
-                g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_STRAY_COMMA,
-                            _("Stray comma"));
+                g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_STRAY_COMMA, _("Stray comma"));
                 goto FAIL;
             }
             t = t->prev;
@@ -1325,22 +1287,19 @@ gwy_expr_transform_to_rpn_real(GwyExpr *expr,
     }
 
     /* 2. Power operator */
-    if (!(tokens = gwy_expr_transform_infix_ops(expr, tokens, TRUE, "^",
-                                                pow_operators, err))) {
+    if (!(tokens = gwy_expr_transform_infix_ops(expr, tokens, TRUE, "^", pow_operators, err))) {
         g_assert(!err || *err);
         goto FAIL;
     }
 
     /* 3. Multiplicative operators */
-    if (!(tokens = gwy_expr_transform_infix_ops(expr, tokens, FALSE, "*/%",
-                                                mult_operators, err))) {
+    if (!(tokens = gwy_expr_transform_infix_ops(expr, tokens, FALSE, "*/%", mult_operators, err))) {
         g_assert(!err || *err);
         goto FAIL;
     }
 
     /* 4. Additive operators */
-    if (!(tokens = gwy_expr_transform_infix_ops(expr, tokens, FALSE, "+-",
-                                                add_operators, err))) {
+    if (!(tokens = gwy_expr_transform_infix_ops(expr, tokens, FALSE, "+-", add_operators, err))) {
         g_assert(!err || *err);
         goto FAIL;
     }
@@ -1348,8 +1307,7 @@ gwy_expr_transform_to_rpn_real(GwyExpr *expr,
     /* Check */
     for (t = tokens; t; t = t->next) {
         if (!t->rpn_block) {
-            g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_GARBAGE,
-                        _("Stray symbol %d"), t->token);
+            g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_GARBAGE, _("Stray symbol %d"), t->token);
             goto FAIL;
         }
     }
@@ -1372,9 +1330,8 @@ FAIL:
  *
  * Converts list of tokens from parser to RPN stack.
  *
- * @expr->tokens is destroyed by the conversion and set to %NULL,
- * @expr->input is filled with opcodes, stack is checked for executability
- * and eventually resized.
+ * @expr->tokens is destroyed by the conversion and set to %NULL, @expr->input is filled with opcodes, stack is
+ * checked for executability and eventually resized.
  *
  * Returns: A newly created RPN stack, %NULL on failure.
  **/
@@ -1401,8 +1358,7 @@ gwy_expr_transform_to_rpn(GwyExpr *expr,
     }
 
     if (expr->tokens->next) {
-        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_GARBAGE,
-                    _("Trailing garbage"));
+        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_GARBAGE, _("Trailing garbage"));
         gwy_expr_token_list_delete(expr, expr->tokens);
         expr->tokens = NULL;
         return FALSE;
@@ -1421,8 +1377,7 @@ gwy_expr_transform_to_rpn(GwyExpr *expr,
     expr->tokens = NULL;
 
     if (!gwy_expr_stack_check_executability(expr)) {
-        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_NOT_EXECUTABLE,
-                    _("Stack is not executable"));
+        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_NOT_EXECUTABLE, _("Stack is not executable"));
         return FALSE;
     }
 
@@ -1463,10 +1418,10 @@ gwy_expr_new(void)
 
     expr = g_slice_new0(GwyExpr);
 
-    /* We could also put constants into scanner's symbol table, but then
-     * we have to tell them apart when we get some symbol from scanner. */
-    expr->constants = g_hash_table_new_full(g_direct_hash, g_direct_equal,
-                                            NULL, g_free);
+    /* We could also put constants into scanner's symbol table, but then we have to tell them apart when we get some
+     * symbol from scanner. */
+    expr->constants = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_free);
+    expr->strings = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, g_free);
     expr->expr = g_string_new(NULL);
 
     return expr;
@@ -1489,6 +1444,8 @@ gwy_expr_free(GwyExpr *expr)
         g_scanner_destroy(expr->scanner);
     if (expr->constants)
         g_hash_table_destroy(expr->constants);
+    if (expr->strings)
+        g_hash_table_destroy(expr->strings);
     g_string_free(expr->expr, TRUE);
     g_free(expr->input);
     g_free(expr->stack);
@@ -1501,9 +1458,8 @@ gwy_expr_free(GwyExpr *expr)
  *
  * Gets the expression string.
  *
- * Returns: The last string passed to gwy_expr_evaluate() or
- *          gwy_expr_compile().  It is owned by @expr and must not be modified
- *          or freed.
+ * Returns: The last string passed to gwy_expr_evaluate() or gwy_expr_compile().  It is owned by @expr and must not be
+ *          modified or freed.
  **/
 const gchar*
 gwy_expr_get_expression(GwyExpr *expr)
@@ -1517,8 +1473,7 @@ gwy_expr_get_expression(GwyExpr *expr)
  * @expr: An expression evaluator.
  * @text: String containing the expression to evaluate.
  * @result: Location to store result to.
- * @err: Location to store compilation or evaluation error to, or %NULL.
- *       Errors from #GwyExprError domain can occur.
+ * @err: Location to store compilation or evaluation error to, or %NULL. Errors from #GwyExprError domain can occur.
  *
  * Evaulates an arithmetic expression.
  *
@@ -1534,8 +1489,7 @@ gwy_expr_evaluate(GwyExpr *expr,
         return FALSE;
 
     if (expr->identifiers->len > 1) {
-        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_UNRESOLVED_IDENTIFIERS,
-                    _("Unresolved identifiers"));
+        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_UNRESOLVED_IDENTIFIERS, _("Unresolved identifiers"));
         return FALSE;
     }
 
@@ -1549,13 +1503,12 @@ gwy_expr_evaluate(GwyExpr *expr,
  * gwy_expr_compile:
  * @expr: An expression evaluator.
  * @text: String containing the expression to compile.
- * @err: Location to store compilation or evaluation error to, or %NULL.
- *       Errors from #GwyExprError domain can occur.
+ * @err: Location to store compilation or evaluation error to, or %NULL. Errors from #GwyExprError domain can occur.
  *
  * Compiles an expression for later execution.
  *
- * This function is useful for expressions with variables.  For normal
- * arithmetic expressions it's easier to use gwy_expr_evaluate().
+ * This function is useful for expressions with variables.  For normal arithmetic expressions it's easier to use
+ * gwy_expr_evaluate().
  *
  * Returns: %TRUE on success, %FALSE if compilation failed.
  **/
@@ -1568,9 +1521,7 @@ gwy_expr_compile(GwyExpr *expr,
     g_return_val_if_fail(text, FALSE);
 
     g_string_assign(expr->expr, text);
-    if (!gwy_expr_parse(expr, err)
-        || !gwy_expr_transform_values(expr)
-        || !gwy_expr_transform_to_rpn(expr, err)) {
+    if (!gwy_expr_parse(expr, err) || !gwy_expr_transform_values(expr) || !gwy_expr_transform_to_rpn(expr, err)) {
         g_assert(!err || *err);
         expr->in = 0;
         return FALSE;
@@ -1583,27 +1534,23 @@ gwy_expr_compile(GwyExpr *expr,
 /**
  * gwy_expr_get_variables:
  * @expr: An expression evaluator.
- * @names: Location to store pointer to array of variable names to (may be
- *         %NULL to get just number of variables).  The string array returned
- *         in this argument in owned by @expr and is valid only until next
- *         gwy_expr_compile(), gwy_expr_evaluate(), or gwy_expr_free()
- *         call.
+ * @names: Location to store pointer to array of variable names to (may be %NULL to get just number of variables).
+ *         The string array returned in this argument in owned by @expr and is valid only until next
+ *         gwy_expr_compile(), gwy_expr_evaluate(), or gwy_expr_free() call.
  *
  * Get the number, names, and indices of unresolved identifiers in @expr.
  *
  * It is an error to call this function after an unsuccessful compilation.
  *
- * If you only care about variables from a prefedined set, that is if any
- * unknown variable is an error, it's easier to use
- * gwy_expr_resolve_variables().
+ * If you only care about variables from a prefedined set, that is if any unknown variable is an error, it's easier to
+ * use gwy_expr_resolve_variables().
  *
- * The position of each variable in @names corresponds to the position of its
- * value in @values array in gwy_expr_execute() call.  Namely, the first item
- * in the array is always reserved and do not correspond to any variable.
+ * The position of each variable in @names corresponds to the position of its value in @values array in
+ * gwy_expr_execute() call.  Namely, the first item in the array is always reserved and do not correspond to any
+ * variable.
  *
- * Returns: The length of array stored to @names.  This is the number of
- *          variables plus one (for the first reserved item).
- *          On failure, 0 is returned.
+ * Returns: The length of array stored to @names.  This is the number of variables plus one (for the first reserved
+ *          item). On failure, 0 is returned.
  **/
 gint
 gwy_expr_get_variables(GwyExpr *expr,
@@ -1623,11 +1570,9 @@ gwy_expr_get_variables(GwyExpr *expr,
  * @expr: An expression evaluator.
  * @n: The length of @names and @indices arrays.
  * @names: List of variable names to get positions of.
- * @indices: Array to store variable positions to.  The positions are the same
- *           as in gwy_expr_execute().  Variables not present in the expression
- *           are assigned (reserved) position 0.  This allows to safely
- *           substitute values of all variables before execution wthout caring
- *           which variables are actually present.
+ * @indices: Array to store variable positions to.  The positions are the same as in gwy_expr_execute().  Variables
+ *           not present in the expression are assigned the reserved position 0.  This permits assigning safely values
+ *           to all variables before execution wthout caring which variables are actually present.
  *
  * Finds positions of variables in an expression.
  *
@@ -1651,8 +1596,7 @@ gwy_expr_resolve_variables(GwyExpr *expr,
     gwy_clear(indices, n);
     for (i = 0; i < n; i++) {
         for (j = 1; j < expr->identifiers->len; j++) {
-            if (gwy_strequal(names[i],
-                             (gchar*)g_ptr_array_index(expr->identifiers, j))) {
+            if (gwy_strequal(names[i], (gchar*)g_ptr_array_index(expr->identifiers, j))) {
                 indices[i] = j;
                 requested[j] = TRUE;
                 break;
@@ -1670,8 +1614,8 @@ gwy_expr_resolve_variables(GwyExpr *expr,
 /**
  * gwy_expr_execute:
  * @expr: An expression evaluator.
- * @values: Array with variable values.  Its zeroth item is always unused.
- *          Variable list can be obtained by gwy_expr_get_variables().
+ * @values: Array with variable values.  Its zeroth item is always unused. Variable list can be obtained by
+ *          gwy_expr_get_variables().
  *
  * Executes a compiled expression with variables, substituting given values.
  *
@@ -1694,12 +1638,14 @@ gwy_expr_execute(GwyExpr *expr,
  * gwy_expr_vector_execute:
  * @expr: An expression evaluator.
  * @n: The lenght of @result and of @data member arrays, that is vector length.
- * @data: An array of arrays of length @n.  The arrays correspond to expression
- *        variables in gwy_expr_execute().  Zeroth array can be %NULL.
- * @result: An array of length @n to store computation results to.  It may be
- *          one of those in @data.
+ * @data: An array of arrays of length @n.  The arrays correspond to expression variables in gwy_expr_execute().
+ *        The zeroth array can be %NULL.
+ * @result: An array of length @n to store computation results to.  It may be one of those in @data.
  *
  * Executes a compiled expression on each item of data arrays.
+ *
+ * The result is the same as repeatedly taking one element from each array and using gwy_expr_execute() to evaluate
+ * the expression with this set of values. However, it is more efficient.
  **/
 void
 gwy_expr_vector_execute(GwyExpr *expr,
@@ -1721,8 +1667,7 @@ gwy_expr_vector_execute(GwyExpr *expr,
  *
  * Checks whether constant name is a valid identifier.
  *
- * Valid identifier must start with a letter, continue with alphanumeric
- * characters (or underscores).
+ * Valid identifier must start with a letter, continue with alphanumeric characters (or underscores).
  *
  * Returns: %TRUE if @name is a possible constant name, %FALSE otherwise.
  **/
@@ -1747,14 +1692,13 @@ gwy_expr_constant_name_is_valid(const gchar *name)
  * @expr: An expression evaluator.
  * @name: Name of constant to define.
  * @value: Constant numeric value.
- * @err: Location to store error to, or %NULL.
- *       Only %GWY_EXPR_ERROR_CONSTANT_NAME error from #GwyExprError domain can
- *       occur.
+ * @err: Location to store error to, or %NULL. Only the %GWY_EXPR_ERROR_CONSTANT_NAME error from #GwyExprError domain
+ *       can occur.
  *
  * Defines a symbolic constant.
  *
- * Note the definition does not affect already compiled expression, you have
- * to recompile it (and eventually re-resolve variables).
+ * Note the definition does not affect already compiled expression, you have to recompile it (and eventually
+ * re-resolve variables).
  *
  * Returns: %TRUE on success, %FALSE if definition failed.
  **/
@@ -1770,21 +1714,18 @@ gwy_expr_define_constant(GwyExpr *expr,
     g_return_val_if_fail(name, FALSE);
 
     if (!gwy_expr_constant_name_is_valid(name)) {
-        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_CONSTANT_NAME,
-                    "Invalid constant name");
+        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_CONSTANT_NAME, "Invalid constant name");
         return FALSE;
     }
 
     gwy_expr_initialize_scanner(expr);
     if (g_scanner_lookup_symbol(expr->scanner, name)) {
-        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_CONSTANT_NAME,
-                    "Constant name clashes with function");
+        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_CONSTANT_NAME, "Constant name clashes with function");
         return FALSE;
     }
 
     quark = g_quark_from_string(name);
-    g_hash_table_insert(expr->constants, GUINT_TO_POINTER(quark),
-                        g_memdup(&value, sizeof(gdouble)));
+    g_hash_table_insert(expr->constants, GUINT_TO_POINTER(quark), g_memdup(&value, sizeof(gdouble)));
 
     return TRUE;
 }
@@ -1796,11 +1737,10 @@ gwy_expr_define_constant(GwyExpr *expr,
  *
  * Undefines a symbolic constant.
  *
- * Note the definition removal does not affect already compiled expression,
- * you have to recompile it (and eventually re-resolve variables).
+ * Note the definition removal does not affect already compiled expression, you have to recompile it (and eventually
+ * re-resolve variables).
  *
- * Returns: %TRUE if there was such a constant and was removed, %FALSE
- *          otherwise.
+ * Returns: %TRUE if there was such a constant and was removed, %FALSE otherwise.
  **/
 gboolean
 gwy_expr_undefine_constant(GwyExpr *expr,
@@ -1825,24 +1765,20 @@ gwy_expr_undefine_constant(GwyExpr *expr,
  * @title: GwyExpr
  * @short_description: Arithmetic expression parser and evaluator
  *
- * #GwyExpr is an expression evaluator, more precisely parser, compiler, and
- * evaluator. A new #GwyExpr can be created with gwy_expr_new(), then it can be
- * used to evaluate any number of expressions; when it's no longer needed, it
+ * #GwyExpr is an expression evaluator, more precisely parser, compiler, and evaluator. A new #GwyExpr can be created
+ * with gwy_expr_new(), then it can be used to evaluate any number of expressions; when it's no longer needed, it
  * should be destroyed with gwy_expr_free().
  *
  * Expression syntax is described in
  * <ulink url="http://gwyddion.net/documentation/user-guide/expression-syntax.html">Gwyddion user guide</ulink>.
  *
- * Simple arithmetic expressions without variables can be directly evaluated
- * with gwy_expr_evaluate().
+ * Simple arithmetic expressions without variables can be directly evaluated with gwy_expr_evaluate().
  *
- * Expression with variables have to be compiled first with gwy_expr_compile().
- * Then either gwy_expr_resolve_variables() or gwy_expr_get_variables() can be
- * used to obtain information what variables are present in the expression and
- * at which positions (variables are referenced by position, not name during
- * final evaluation because of effieiency reasons). Subsequent evaluation with
- * variable substitution is performed by gwy_expr_execute(). It is also
- * possible to evaluate expression on each item of value arrays with
+ * Expression with variables have to be compiled first with gwy_expr_compile(). Then either
+ * gwy_expr_resolve_variables() or gwy_expr_get_variables() can be used to obtain information what variables are
+ * present in the expression and what integer index they correspond to (variables are identified by an integer index,
+ * not name, during final evaluation for effieiency). Subsequent evaluation with variable value substitution is
+ * performed by gwy_expr_execute(). It is also possible to evaluate expression on each item of value arrays with
  * gwy_expr_vector_execute().
  *
  * One-shot evaluation of expressions without variables is easy:
@@ -1859,8 +1795,7 @@ gwy_expr_undefine_constant(GwyExpr *expr,
  * gwy_expr_free(expr);
  * </programlisting></informalexample>
  *
- * One-shot evaluation of expressions with known variables can be performed by
- * defining them as constants beforehand:
+ * One-shot evaluation of expressions with known variables can be done by defining them as constants beforehand:
  * <informalexample><programlisting>
  * GwyExpr *expr;
  * GError *err = NULL;
@@ -1880,11 +1815,9 @@ gwy_expr_undefine_constant(GwyExpr *expr,
  * gwy_expr_free(expr);
  * </programlisting></informalexample>
  *
- * When the same expression is evaluated multiple times with different
- * variable values and the variables are again from some known set, the
- * gwy_expr_resolve_variables() should be used (if the repeated evaluation
- * happens over items of some arrays, gwy_expr_vector_execute() is more
- * efficient and usually simplier to use too):
+ * When the same expression is evaluated multiple times with different variable values and the variables are again
+ * from some known set, the gwy_expr_resolve_variables() should be used (if the repeated evaluation happens over items
+ * of some arrays, gwy_expr_vector_execute() is more efficient and usually simplier to use too):
  * <informalexample><programlisting>
  * GwyExpr *expr;
  * GError *err = NULL;
@@ -1924,28 +1857,24 @@ gwy_expr_undefine_constant(GwyExpr *expr,
  * gwy_expr_free(expr);
  * </programlisting></informalexample>
  *
- * The most general case is when the variables are from a large set (or
- * completely arbitrary).  Then it is best to get the list of variables with
- * gwy_expr_get_variables() and supply only values of variables that
- * are actually present in the expression.
+ * The most general case is when the variables are from a large set (or completely arbitrary).  Then it is best to get
+ * the list of variables with gwy_expr_get_variables() and supply only values of variables that are actually present
+ * in the expression.
  **/
 
 /**
  * GwyExprError:
  * @GWY_EXPR_ERROR_CLOSING_PARENTHESIS: A closing parenthesis is missing.
  * @GWY_EXPR_ERROR_EMPTY: Expression is empty.
- * @GWY_EXPR_ERROR_EMPTY_PARENTHESES: A parentheses pair contain nothing
- *                                    inside.
+ * @GWY_EXPR_ERROR_EMPTY_PARENTHESES: A parentheses pair contain nothing inside.
  * @GWY_EXPR_ERROR_GARBAGE: An symbol unexpectedly managed to survive.
- * @GWY_EXPR_ERROR_INVALID_ARGUMENT: Function or operator argument is not a
- *                                   value.
+ * @GWY_EXPR_ERROR_INVALID_ARGUMENT: Function or operator argument is not a value.
  * @GWY_EXPR_ERROR_INVALID_TOKEN: Expression contains an invalid token.
  * @GWY_EXPR_ERROR_MISSING_ARGUMENT: Function or operator arguments is missing.
  * @GWY_EXPR_ERROR_NOT_EXECUTABLE: Compiled stack is not executable.
  * @GWY_EXPR_ERROR_OPENING_PARENTHESIS: An opening parenthesis is missing.
  * @GWY_EXPR_ERROR_STRAY_COMMA: A comma at the start or end of list.
- * @GWY_EXPR_ERROR_UNRESOLVED_IDENTIFIERS: Expression contains unresolved
- *                                         identifiers.
+ * @GWY_EXPR_ERROR_UNRESOLVED_IDENTIFIERS: Expression contains unresolved identifiers.
  * @GWY_EXPR_ERROR_CONSTANT_NAME: Constant name is invalid.
  *
  * Error codes returned by expression parsing and execution.
@@ -1954,16 +1883,14 @@ gwy_expr_undefine_constant(GwyExpr *expr,
 /**
  * GWY_EXPR_ERROR:
  *
- * Error domain for expression parsing and evaluation. Errors in this domain
- * will be from the #GwyExprError enumeration. See #GError for information on
- * error domains.
+ * Error domain for expression parsing and evaluation. Errors in this domain will be from the #GwyExprError
+ * enumeration. See #GError for information on error domains.
  **/
 
 /**
  * GwyExpr:
  *
- * #GwyExpr is an opaque data structure and should be only manipulated with the
- * functions below.
+ * #GwyExpr is an opaque data structure and should be only manipulated with the functions below.
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
