@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyfdcurvepreset.c 24422 2021-10-24 13:48:20Z klapetek $
+ *  $Id: gwyfdcurvepreset.c 24520 2021-11-13 18:45:09Z klapetek $
  *  Copyright (C) 2000-2003 Martin Siler.
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
@@ -639,7 +639,7 @@ dmt_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
 
 
 
-/******************* Sneddon ********************************/
+/******************* Sneddon conical ********************************/
 static gdouble
 sneddon_func(gdouble x,
              G_GNUC_UNUSED gint n_param,
@@ -710,6 +710,166 @@ sneddon_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
     }
 }
 
+/******************* Sneddon conical with free film correction ********************************/
+static gdouble
+sneddonhfree_func(gdouble x,
+             G_GNUC_UNUSED gint n_param,
+             const gdouble *param,
+             G_GNUC_UNUSED gpointer user_data,
+             gboolean *fres)
+{
+    /*xc, alpha, E, nu, h*/
+    double xr = (param[0] - x); //this should be deformation and should be already entered only positive
+    double zeta = 0.388;
+    double fc = 1 + zeta*2*tan(param[1])*xr/(M_PI*M_PI*param[4]) 
+              + 16*zeta*zeta*tan(param[1])*tan(param[1])*xr*xr/(param[4]*param[4]);
+
+    *fres = TRUE;
+    if (xr > 0)
+        return 2.0*param[2]/M_PI/(1-param[3]*param[3])*tan(param[1])*xr*xr*fc;
+    else
+        return 0;
+}
+
+static void
+sneddonhfree_guess(gint n_dat,
+              const gdouble *x,
+              const gdouble *y,
+              gdouble *param,
+              gboolean *fres)
+{
+    gint i;
+    gdouble ymin = G_MAXDOUBLE;
+    gdouble xmin = 0;
+
+    for (i = 0; i < n_dat; i++) {
+        if (y[i] < ymin) {
+            ymin = y[i];
+            xmin = x[i];
+        }
+    }
+    param[0] = xmin;
+    param[1] = 0.25;
+    param[2] = 5e7;
+    param[3] = 0.25;
+    param[4] = 100e-9;
+
+    *fres = TRUE;
+}
+
+static GwySIUnit*
+sneddonhfree_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
+                  guint param,
+                  GwySIUnit *siunit_x,
+                  G_GNUC_UNUSED GwySIUnit *siunit_y)
+{
+    switch (param) {
+        case 0:
+        return gwy_si_unit_duplicate(siunit_x);
+        break;
+
+        case 1:
+        return gwy_si_unit_new("rad");
+        break;
+
+        case 2:
+        return gwy_si_unit_new("Pa");
+        break;
+
+        case 3:
+        return gwy_si_unit_new(NULL);
+        break;
+
+        case 4:
+        return gwy_si_unit_new("m");
+        break;
+
+        default:
+        g_return_val_if_reached(NULL);
+        break;
+    }
+}
+
+/******************* Sneddon conical with fixed film correction ********************************/
+static gdouble
+sneddonhfix_func(gdouble x,
+             G_GNUC_UNUSED gint n_param,
+             const gdouble *param,
+             G_GNUC_UNUSED gpointer user_data,
+             gboolean *fres)
+{
+    /*xc, alpha, E, nu, h*/
+    double xr = (param[0] - x); //this should be deformation and should be already entered only positive
+    double zeta = 1.7795;
+    double fc = 1 + zeta*2*tan(param[1])*xr/(M_PI*M_PI*param[4]) 
+              + 16*zeta*zeta*tan(param[1])*tan(param[1])*xr*xr/(param[4]*param[4]);
+
+    *fres = TRUE;
+    if (xr > 0)
+        return 2.0*param[2]/M_PI/(1-param[3]*param[3])*tan(param[1])*xr*xr*fc;
+    else
+        return 0;
+}
+
+static void
+sneddonhfix_guess(gint n_dat,
+              const gdouble *x,
+              const gdouble *y,
+              gdouble *param,
+              gboolean *fres)
+{
+    gint i;
+    gdouble ymin = G_MAXDOUBLE;
+    gdouble xmin = 0;
+
+    for (i = 0; i < n_dat; i++) {
+        if (y[i] < ymin) {
+            ymin = y[i];
+            xmin = x[i];
+        }
+    }
+    param[0] = xmin;
+    param[1] = 0.25;
+    param[2] = 5e7;
+    param[3] = 0.25;
+    param[4] = 100e-9;
+
+    *fres = TRUE;
+}
+
+static GwySIUnit*
+sneddonhfix_get_units(G_GNUC_UNUSED GwyNLFitPreset *preset,
+                  guint param,
+                  GwySIUnit *siunit_x,
+                  G_GNUC_UNUSED GwySIUnit *siunit_y)
+{
+    switch (param) {
+        case 0:
+        return gwy_si_unit_duplicate(siunit_x);
+        break;
+
+        case 1:
+        return gwy_si_unit_new("rad");
+        break;
+
+        case 2:
+        return gwy_si_unit_new("Pa");
+        break;
+
+        case 3:
+        return gwy_si_unit_new(NULL);
+        break;
+
+        case 4:
+        return gwy_si_unit_new("m");
+        break;
+
+        default:
+        g_return_val_if_reached(NULL);
+        break;
+    }
+}
+
 
 /******************* sphtiptap ********************************/
 static gdouble
@@ -1141,6 +1301,23 @@ static const GwyNLFitParam sneddon_params[] = {
     { "ν", 0, 0, },
 };
 
+static const GwyNLFitParam sneddonhfix_params[] = {
+    { "xc", 1, 0, },
+    { "α", 0, 0, },
+    { "E", -2, 1, },
+    { "ν", 0, 0, },
+    { "h", 1, 0, },
+};
+
+static const GwyNLFitParam sneddonhfree_params[] = {
+    { "xc", 1, 0, },
+    { "α", 0, 0, },
+    { "E", -2, 1, },
+    { "ν", 0, 0, },
+    { "h", 1, 0, },
+};
+
+
 
 /*
 static const GwyNLFitParam hertzpar_params[] = {
@@ -1381,7 +1558,31 @@ static const GwyNLFitPresetBuiltin fitting_presets[] = {
         G_N_ELEMENTS(sneddon_params),
         sneddon_params,
     },
-       /*    {
+    {
+        N_("Sneddon: conical, fixed film"),
+        "<i>f</i>(<i>x</i>) "
+        "= 2/π E/(1-ν<sup>2</sup>) tan(α) (<i>x</i>-<i>x<sub>c</sub></i>)<sup>2</sup>f<sub>c,fix</sub>",
+        &sneddonhfix_func,
+        NULL,
+        &sneddonhfix_guess,
+        &sneddonhfix_get_units,
+        NULL,
+        G_N_ELEMENTS(sneddonhfix_params),
+        sneddonhfix_params,
+    },
+     {
+        N_("Sneddon: conical, free film"),
+        "<i>f</i>(<i>x</i>) "
+        "= 2/π E/(1-ν<sup>2</sup>) tan(α) (<i>x</i>-<i>x<sub>c</sub></i>)<sup>2</sup>f<sub>c,free</sub>",
+        &sneddonhfree_func,
+        NULL,
+        &sneddonhfree_guess,
+        &sneddonhfree_get_units,
+        NULL,
+        G_N_ELEMENTS(sneddonhfree_params),
+        sneddonhfree_params,
+    },
+        /*    {
         "vdW: sphere3",
         "<i>f</i>(<i>x</i>) "
         "= -<i>HR</i>/12 (1/(<i>x</i>-<i>x<sub>c</sub></i>)<sup>2</sup>"
