@@ -1,22 +1,18 @@
 /*
- *  $Id: gwymacros.h 21726 2018-12-14 19:15:57Z yeti-dn $
+ *  $Id: gwymacros.h 24951 2022-08-26 15:24:04Z yeti-dn $
  *  Copyright (C) 2003-2016 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GWY_MACROS_H__
@@ -45,10 +41,18 @@
 
 #define GWY_SWAP(t, x, y) \
     do { \
-    t __unsafe_swap; \
-    __unsafe_swap = x; \
+    t __gwy_unsafe_swap = x; \
     x = y; \
-    y = __unsafe_swap; \
+    y = __gwy_unsafe_swap; \
+    } while (0)
+
+#define GWY_ORDER(t, x, y) \
+    do { \
+        if ((y) < (x)) { \
+            t __gwy_unsafe_swap = x; \
+            x = y; \
+            y = __gwy_unsafe_swap; \
+        } \
     } while (0)
 
 #define gwy_strequal(a, b) \
@@ -57,7 +61,7 @@
 #define GWY_CLAMP(x, low, hi) \
     (G_UNLIKELY((x) > (hi)) ? (hi) : (G_UNLIKELY((x) < (low)) ? (low) : (x)))
 
-#define gwy_clear(array, n) \
+#define gwy_clear(array, n)\
     memset((array), 0, (n)*sizeof((array)[0]))
 
 #define gwy_assign(dest, source, n) \
@@ -83,8 +87,7 @@
 #define gwy_signal_handler_disconnect GWY_SIGNAL_HANDLER_DISCONNECT
 
 #define GWY_FIND_PSPEC(type, id, spectype) \
-    G_PARAM_SPEC_##spectype(g_object_class_find_property \
-                                (G_OBJECT_CLASS(g_type_class_peek(type)), id))
+    G_PARAM_SPEC_##spectype(g_object_class_find_property (G_OBJECT_CLASS(g_type_class_peek(type)), id))
 
 #define GWY_SI_VALUE_FORMAT_FREE(vf) \
     do { \
@@ -165,4 +168,4 @@ G_END_DECLS
 
 #endif /* __GWY_MACROS_H__ */
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
