@@ -1,5 +1,5 @@
 /*
- *  $Id: gwymath.c 25415 2023-06-06 14:11:08Z yeti-dn $
+ *  $Id: gwymath.c 25856 2023-10-14 07:30:23Z yeti-dn $
  *  Copyright (C) 2003-2022 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -2683,11 +2683,9 @@ gwy_math_sort_with_index(gsize n, gdouble *array, guint *index_array)
             guint *left_ptri;
             guint *right_ptri;
 
-            /* Select median value from among LO, MID, and HI. Rearrange
-               LO and HI so the three values are sorted. This lowers the
-               probability of picking a pathological pivot value and
-               skips a comparison for both the LEFT_PTR and RIGHT_PTR in
-               the while loops. */
+            /* Select median value from among LO, MID, and HI. Rearrange LO and HI so the three values are sorted.
+             * This lowers the probability of picking a pathological pivot value and skips a comparison for both the
+             * LEFT_PTR and RIGHT_PTR in the while loops. */
 
             gdouble *mid = lo + ((hi - lo) >> 1);
             guint *midi = loi + ((hii - loi) >> 1);
@@ -2711,9 +2709,8 @@ gwy_math_sort_with_index(gsize n, gdouble *array, guint *index_array)
           left_ptri  = loi + 1;
           right_ptri = hii - 1;
 
-          /* Here's the famous ``collapse the walls'' section of quicksort.
-             Gotta like those tight inner loops!  They are the main reason
-             that this algorithm runs much faster than others. */
+          /* Here's the famous ``collapse the walls'' section of quicksort. Gotta like those tight inner loops!  They
+           * are the main reason that this algorithm runs much faster than others. */
           do {
               while (*left_ptr < *mid) {
                   left_ptr++;
@@ -2751,10 +2748,9 @@ gwy_math_sort_with_index(gsize n, gdouble *array, guint *index_array)
           }
           while (left_ptr <= right_ptr);
 
-          /* Set up pointers for next iteration.  First determine whether
-             left and right partitions are below the threshold size.  If so,
-             ignore one or both.  Otherwise, push the larger partition's
-             bounds on the stack and continue sorting the smaller one. */
+          /* Set up pointers for next iteration.  First determine whether left and right partitions are below the
+           * threshold size.  If so, ignore one or both.  Otherwise, push the larger partition's bounds on the stack
+           * and continue sorting the smaller one. */
 
           if ((gsize)(right_ptr - lo) <= MAX_THRESH) {
               if ((gsize)(hi - left_ptr) <= MAX_THRESH) {
@@ -2799,12 +2795,9 @@ gwy_math_sort_with_index(gsize n, gdouble *array, guint *index_array)
         }
     }
 
-    /* Once the BASE_PTR array is partially sorted by quicksort the rest
-       is completely sorted using insertion sort, since this is efficient
-       for partitions below MAX_THRESH size. BASE_PTR points to the beginning
-       of the array to sort, and END_PTR points at the very last element in
-       the array (*not* one beyond it!). */
-
+    /* Once the BASE_PTR array is partially sorted by quicksort the rest is completely sorted using insertion sort,
+     * since this is efficient for partitions below MAX_THRESH size. BASE_PTR points to the beginning of the array to
+     * sort, and END_PTR points at the very last element in the array (*not* one beyond it!). */
     {
         gdouble *const end_ptr = array + (n - 1);
         gdouble *tmp_ptr = array;
@@ -2813,9 +2806,8 @@ gwy_math_sort_with_index(gsize n, gdouble *array, guint *index_array)
         gdouble *run_ptr;
         guint *run_ptri;
 
-        /* Find smallest element in first threshold and place it at the
-           array's beginning.  This is the smallest array element,
-           and the operation speeds up insertion sort's inner loop. */
+        /* Find smallest element in first threshold and place it at the array's beginning.  This is the smallest array
+         * element, and the operation speeds up insertion sort's inner loop. */
 
         for (run_ptr = tmp_ptr + 1, run_ptri = tmp_ptri + 1;
              run_ptr <= thresh;
@@ -2831,8 +2823,7 @@ gwy_math_sort_with_index(gsize n, gdouble *array, guint *index_array)
             ISWAP(*tmp_ptri, *index_array);
         }
 
-        /* Insertion sort, running from left-hand-side up to right-hand-side.
-         */
+        /* Insertion sort, running from left-hand-side up to right-hand-side. */
 
         run_ptr = array + 1;
         run_ptri = index_array + 1;
