@@ -1,5 +1,5 @@
 /*
- *  $Id: gwymath.h 24689 2022-03-19 11:29:08Z yeti-dn $
+ *  $Id: gwymath.h 25074 2022-10-11 13:46:47Z yeti-dn $
  *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -128,10 +128,10 @@ guint    gwy_math_curvature_at_apex    (const gdouble *coeffs,
                                         gdouble *yc,
                                         gdouble *zc);
 guint    gwy_math_curvature_at_origin  (const gdouble *coeffs,
-                                        gdouble *pkappa1,
-                                        gdouble *pkappa2,
-                                        gdouble *pphi1,
-                                        gdouble *pphi2);
+                                        gdouble *kappa1,
+                                        gdouble *kappa2,
+                                        gdouble *phi1,
+                                        gdouble *phi2);
 gboolean gwy_math_refine_maximum_1d    (const gdouble *y,
                                         gdouble *x);
 gboolean gwy_math_refine_maximum_2d    (const gdouble *z,
@@ -182,6 +182,8 @@ gdouble  gwy_math_trimmed_mean         (gsize n,
 gdouble  gwy_math_median_uncertainty   (gsize n,
                                         gdouble *array,
                                         gdouble *uarray);
+gint     gwy_compare_double            (gconstpointer a,
+                                        gconstpointer b);
 gdouble  gwy_xlnx_int                  (guint x)                               G_GNUC_CONST;
 gdouble  gwy_sinc                      (gdouble x)                             G_GNUC_CONST;
 gdouble  gwy_canonicalize_angle        (gdouble phi,
