@@ -1,5 +1,5 @@
 /*
- *  $Id: gwymathfallback.h 21249 2018-07-23 13:38:45Z yeti-dn $
+ *  $Id: gwymathfallback.h 21933 2019-02-27 08:58:37Z yeti-dn $
  *  Copyright (C) 2007-2016 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -38,6 +38,7 @@ _GWY_STATIC_INLINE double gwy_math_fallback_hypot(double x, double y);
 _GWY_STATIC_INLINE double gwy_math_fallback_acosh(double x);
 _GWY_STATIC_INLINE double gwy_math_fallback_asinh(double x);
 _GWY_STATIC_INLINE double gwy_math_fallback_atanh(double x);
+_GWY_STATIC_INLINE double gwy_math_fallback_powi(double x, int i);
 _GWY_STATIC_INLINE int gwy_math_fallback_isinf(double x);
 _GWY_STATIC_INLINE int gwy_math_fallback_isnan(double x);
 
@@ -136,7 +137,7 @@ gwy_math_fallback_isnan(double x)
 
 /* GCC's fast math makes isinf no-op.  Must use the fallback function.
  * Gwyddion works with finite arithmetic, but we need to detect INFs upon
- * file import and remove it. */
+ * file import. */
 #if (defined(GWY_HAVE_ISINF) && !defined(__FAST_MATH__))
 #define gwy_isinf isinf
 #else
@@ -145,13 +146,48 @@ gwy_math_fallback_isnan(double x)
 
 /* GCC's fast math makes isnan no-op.  Must use the fallback function.
  * Gwyddion works with finite arithmetic, but we need to detect NaNs upon
- * file import and remove it. */
+ * file import. */
 #if (defined(GWY_HAVE_ISNAN) && !defined(__FAST_MATH__))
 #define gwy_isnan isnan
 #else
 #define gwy_isnan gwy_math_fallback_isnan
 #endif  /* GWY_HAVE_ISNAN */
 
+/* This is a compiler thing, not system configuration.  So we cannot even
+ * specify in gwyconfig.h whether the function is avilable or not.  All
+ * solutions are bad; the cleanest one (do not ever expose the function) does
+ * not allow using it in code built on Gwyddion and thus kind of defeats the
+ * point... */
+#if (!defined(__GNUC__) && !defined(__clang__))
+#  define gwy_powi gwy_math_fallback_powi
+#else
+#  define gwy_powi __builtin_powi
+#endif
+
+/* From the precision standpoint this is not nice, but it should be used
+ * for small fixed powers where the compiler should reduce it to a couple of
+ * multiplications.  Good enough as a fallback. */
+static inline double
+gwy_math_fallback_powi(double x, int i)
+{
+    gdouble r = 1.0;
+    gboolean negative = FALSE;
+    if (!i)
+        return 1.0;
+    if (i < 0) {
+        negative = TRUE;
+        i = -i;
+    }
+    while (TRUE) {
+        if (i & 1)
+            r *= x;
+        if (!(i >>= 1))
+            break;
+        x *= x;
+    }
+    return negative ? 1.0/r : r;
+}
+
 G_END_DECLS
 
 #endif /* __GWY_MATH_FALLBACK_H__ */
