@@ -1,27 +1,25 @@
 /*
- *  $Id: gwyrandgenset.c 20677 2017-12-18 18:22:52Z yeti-dn $
- *  Copyright (C) 2014-2016 David Necas (Yeti).
+ *  $Id: gwyrandgenset.c 23775 2021-05-24 21:01:14Z yeti-dn $
+ *  Copyright (C) 2014-2021 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "gwymacros.h"
 #include "gwymath.h"
+#include "gwythreads.h"
 #include "gwyrandgenset.h"
+#include "gwyomp.h"
 
 #define GWY_SQRT6 2.449489742783178098197284074705
 
@@ -93,8 +91,8 @@ gwy_rand_gen_set_init(GwyRandGenSet *rngset,
  *
  * Destroys a set of pseudorandom number generators.
  *
- * If you obtained individual generators using gwy_rand_gen_set_rng() you may
- * not use them any more after calling this function.
+ * If you obtained individual generators using gwy_rand_gen_set_rng() you may not use them any more after calling this
+ * function.
  *
  * Since: 2.37
  **/
@@ -116,10 +114,9 @@ gwy_rand_gen_set_free(GwyRandGenSet *rngset)
  *
  * Obtains a single generator from a set of pseudorandom number generators.
  *
- * The generator can be used to produce random numbers in any way for which you
- * find the provided methods insufficient.  However, if you reseed it manually,
- * number sequence stability will be broken because sampling functions may keep
- * persistent information between calls.
+ * The generator can be used to produce random numbers in any way for which you find the provided methods
+ * insufficient.  However, if you reseed it manually, number sequence stability will be broken because sampling
+ * functions may keep persistent information between calls.
  *
  * Returns: A pseudorandom number generator from @rngset.
  *
@@ -141,13 +138,12 @@ gwy_rand_gen_set_rng(GwyRandGenSet *rngset,
  * @lower: Lower limit of the range.
  * @upper: Upper limit of the range.
  *
- * Samples from a uniform distribution over given interval using one generator
- * from a pseudorandom number generator set.
+ * Samples from a uniform distribution over given interval using one generator from a pseudorandom number generator
+ * set.
  *
- * The generated number always lies inside the interval, neither endpoint value
- * is ever returned.  Note if there are no representable real numbers between
- * @lower and @upper this function will never terminate.  You must ensure
- * @upper is sufficiently larger than @lower.
+ * The generated number always lies inside the interval, neither endpoint value is ever returned.  Note if there are
+ * no representable real numbers between @lower and @upper this function will never terminate.  You must ensure @upper
+ * is sufficiently larger than @lower.
  *
  * Returns: A pseudorandom number.
  *
@@ -175,11 +171,9 @@ gwy_rand_gen_set_range(GwyRandGenSet *rngset,
  * @i: Index of a generator from the set.
  * @sigma: Rms of the distribution.
  *
- * Samples from a centered uniform distribution using one generator from a
- * pseudorandom number generator set.
+ * Samples from a centered uniform distribution using one generator from a pseudorandom number generator set.
  *
- * The mean value of the distribution is zero, the rms value is given by
- * @sigma.
+ * The mean value of the distribution is zero, the rms value is given by @sigma.
  *
  * Returns: A pseudorandom number.
  *
@@ -206,11 +200,9 @@ gwy_rand_gen_set_uniform(GwyRandGenSet *rngset,
  * @i: Index of a generator from the set.
  * @sigma: Rms of the distribution.
  *
- * Samples from a centered Gaussian distribution using one generator from a
- * pseudorandom number generator set.
+ * Samples from a centered Gaussian distribution using one generator from a pseudorandom number generator set.
  *
- * The mean value of the distribution is zero, the rms value is given by
- * @sigma.
+ * The mean value of the distribution is zero, the rms value is given by @sigma.
  *
  * Returns: A pseudorandom number.
  *
@@ -248,11 +240,9 @@ gwy_rand_gen_set_gaussian(GwyRandGenSet *rngset,
  * @i: Index of a generator from the set.
  * @sigma: Rms of the distribution.
  *
- * Samples from a centered exponential distribution using one generator from a
- * pseudorandom number generator set.
+ * Samples from a centered exponential distribution using one generator from a pseudorandom number generator set.
  *
- * The mean value of the distribution is zero, the rms value is given by
- * @sigma.
+ * The mean value of the distribution is zero, the rms value is given by @sigma.
  *
  * Returns: A pseudorandom number.
  *
@@ -293,11 +283,9 @@ gwy_rand_gen_set_exponential(GwyRandGenSet *rngset,
  * @i: Index of a generator from the set.
  * @sigma: Rms of the distribution.
  *
- * Samples from a centered triangular distribution using one generator from a
- * pseudorandom number generator set.
+ * Samples from a centered triangular distribution using one generator from a pseudorandom number generator set.
  *
- * The mean value of the distribution is zero, the rms value is given by
- * @sigma.
+ * The mean value of the distribution is zero, the rms value is given by @sigma.
  *
  * Returns: A pseudorandom number.
  *
@@ -315,8 +303,7 @@ gwy_rand_gen_set_triangular(GwyRandGenSet *rngset,
         x = g_rand_double(randgen->rng);
     } while (G_UNLIKELY(x == 0.0));
 
-    return (x <= 0.5 ? sqrt(2.0*x) - 1.0 : 1.0 - sqrt(2.0*(1.0 - x)))
-           *sigma*GWY_SQRT6;
+    return (x <= 0.5 ? sqrt(2.0*x) - 1.0 : 1.0 - sqrt(2.0*(1.0 - x)))*sigma*GWY_SQRT6;
 }
 
 /**
@@ -325,11 +312,9 @@ gwy_rand_gen_set_triangular(GwyRandGenSet *rngset,
  * @i: Index of a generator from the set.
  * @range: Half-range of the distribution.
  *
- * Samples from a multiplier distribution using one generator from a
- * pseudorandom number generator set.
+ * Samples from a multiplier distribution using one generator from a pseudorandom number generator set.
  *
- * The multiplier distribution is triangular distribution centered at 1,
- * with values from [1-@range, 1+@range].
+ * The multiplier distribution is triangular distribution centered at 1, with values from [1-@range, 1+@range].
  *
  * Returns: A pseudorandom number.
  *
@@ -351,8 +336,7 @@ gwy_rand_gen_set_multiplier(GwyRandGenSet *rngset,
  * @rngset: A set of pseudorandom number generators.
  * @i: Index of a generator from the set.
  *
- * Samples uniform distribution over [0,1) using one generator from a
- * pseudorandom number generator set.
+ * Samples uniform distribution over [0,1) using one generator from a pseudorandom number generator set.
  *
  * Returns: A pseudorandom number.
  *
@@ -370,8 +354,7 @@ gwy_rand_gen_set_double(GwyRandGenSet *rngset,
  * @rngset: A set of pseudorandom number generators.
  * @i: Index of a generator from the set.
  *
- * Samples a 32bit integer using a generator from a pseudorandom number
- * generator set.
+ * Samples a 32bit integer using a generator from a pseudorandom number generator set.
  *
  * Returns: A pseudorandom number.
  *
@@ -389,13 +372,13 @@ gwy_rand_gen_set_int(GwyRandGenSet *rngset,
  * @rngset: A set of pseudorandom number generators.
  * @i: Index of a generator from the set.
  * @n: Total number of possible indices.
- * @nchoose: Number of values to choose (at most equal to @n).  It is permitted
- *           to pass zero; the function returns %NULL then.
+ * @nchoose: Number of values to choose (at most equal to @n).  It is permitted to pass zero; the function returns
+ *           %NULL then.
  *
  * Chooses randomly a subset of indices, in random order.
  *
- * The function creates an array containing integers from the set
- * {0, 1, 2, ..., n-1}, each at most once, in random order.
+ * The function creates an array containing integers from the set {0, 1, 2, ..., n-1}, each at most once, in random
+ * order.
  *
  * To generate a permutation, simply pass @nchoose equal to @n.
  *
@@ -420,8 +403,8 @@ gwy_rand_gen_set_choose_shuffle(GwyRandGenSet *rngset,
         return NULL;
 
     rng = rngset->rngs[i].rng;
-    /* XXX: This is not a good theoretical threshold.  We should do some
-     * benchmarking because rng is a comparatively slow operation. */
+    /* XXX: This is not a good theoretical threshold.  We should do some benchmarking because rng is a comparatively
+     * slow operation. */
     if (nchoose < (guint)sqrt(n)) {
         /* Generate indices directly and check for repetition. */
         indices = g_new(guint, nchoose);
@@ -461,6 +444,98 @@ gwy_rand_gen_set_choose_shuffle(GwyRandGenSet *rngset,
     return indices;
 }
 
+/**
+ * gwy_rand_gen_set_fill_doubles:
+ * @rngset: A set of pseudorandom number generators.
+ * @random_numbers: Array to be filled with random numbers.
+ * @n: How many numbers to generated.
+ *
+ * Fill an array with random doubles from a pseudorandom number generator set
+ *
+ * If @rngset has @N generators then the array is split into @N equally sized blocks (at least approximately) and
+ * each block is filled from a different generator.  The filling of individual blocks is run multi-threaded (if
+ * parallelisation is enabled).  The result does not depend on the number of threads, allowing parallelisation of
+ * random number generation.
+ *
+ * Note that the number of generators @N must be a fixed constant for reproducible results.  It must not depend on the
+ * actual number of processor cores or a similar variable quantity.
+ *
+ * Since: 2.59
+ **/
+void
+gwy_rand_gen_set_fill_doubles(GwyRandGenSet *rngset,
+                              gdouble *random_numbers,
+                              gint n)
+{
+    guint ngenerators;
+
+    g_return_if_fail(rngset);
+    g_return_if_fail(!n || random_numbers);
+    ngenerators = rngset->n;
+
+#ifdef _OPENMP
+#pragma omp parallel if (gwy_threads_are_enabled()) default(none) \
+            shared(random_numbers,n,rngset,ngenerators)
+#endif
+    {
+        guint irfrom = gwy_omp_chunk_start(ngenerators);
+        guint irto = gwy_omp_chunk_end(ngenerators);
+        guint ir, ifrom, ito, i;
+        GRand *rng;
+
+        for (ir = irfrom; ir < irto; ir++) {
+            rng = gwy_rand_gen_set_rng(rngset, ir);
+            ifrom = ir*n/ngenerators;
+            ito = (ir + 1)*n/ngenerators;
+            for (i = ifrom; i < ito; i++)
+                random_numbers[i] = g_rand_double(rng);
+        }
+    }
+}
+
+/**
+ * gwy_rand_gen_set_fill_ints:
+ * @rngset: A set of pseudorandom number generators.
+ * @random_numbers: Array to be filled with random numbers.
+ * @n: How many numbers to generated.
+ *
+ * Fill an array with random integers from a pseudorandom number generator set
+ *
+ * See gwy_rand_gen_set_fill_doubles() for discussion and caveats.
+ *
+ * Since: 2.59
+ **/
+void
+gwy_rand_gen_set_fill_ints(GwyRandGenSet *rngset,
+                           guint32 *random_numbers,
+                           gint n)
+{
+    guint ngenerators;
+
+    g_return_if_fail(rngset);
+    g_return_if_fail(!n || random_numbers);
+    ngenerators = rngset->n;
+
+#ifdef _OPENMP
+#pragma omp parallel if (gwy_threads_are_enabled()) default(none) \
+            shared(random_numbers,n,rngset,ngenerators)
+#endif
+    {
+        guint irfrom = gwy_omp_chunk_start(ngenerators);
+        guint irto = gwy_omp_chunk_end(ngenerators);
+        guint ir, ifrom, ito, i;
+        GRand *rng;
+
+        for (ir = irfrom; ir < irto; ir++) {
+            rng = gwy_rand_gen_set_rng(rngset, ir);
+            ifrom = ir*n/ngenerators;
+            ito = (ir + 1)*n/ngenerators;
+            for (i = ifrom; i < ito; i++)
+                random_numbers[i] = g_rand_int(rng);
+        }
+    }
+}
+
 /************************** Documentation ****************************/
 
 /**
@@ -468,17 +543,14 @@ gwy_rand_gen_set_choose_shuffle(GwyRandGenSet *rngset,
  * @title: GwyRandGenSet
  * @short_description: Set of random number generators
  *
- * #GwyRandGenSet represents a set of pseudoradnom number generators
- * initialised together, but each producing a different sequence of numbers.
- * This is useful when you use pseudorandom number generators to optionally
- * randomize several different things.  Using a common generator would require
- * always generating exactly the same number of random numbers, even for
- * quantities you do not want randomized, in order to keep the random number
- * sequences stable.
+ * #GwyRandGenSet represents a set of pseudoradnom number generators initialised together, but each producing
+ * a different sequence of numbers. This is useful when you use pseudorandom number generators to optionally randomize
+ * several different things.  Using a common generator would require always generating exactly the same number of
+ * random numbers, even for quantities you do not want randomized, in order to keep the random number sequences
+ * stable.
  *
- * #GwyRandGenSet also provides functions sample a few common distributions
- * such as Gaussian or exponential.  It should be noted that the individual
- * sampling functions may advance the generator state differently.  This means
+ * #GwyRandGenSet also provides functions sample a few common distributions such as Gaussian or exponential.  It
+ * should be noted that the individual sampling functions may advance the generator state differently.  This means
  * |[
  * x = gwy_rand_gen_set_gaussian(rngset, 0, 1.0);
  * y = gwy_rand_gen_set_gaussian(rngset, 0, 1.0);
@@ -494,10 +566,9 @@ gwy_rand_gen_set_choose_shuffle(GwyRandGenSet *rngset,
 /**
  * GwyRandGenSet:
  *
- * #GwyRandGenSet is an opaque data structure and should be only manipulated
- * with the functions below.
+ * #GwyRandGenSet is an opaque data structure and should be only manipulated with the functions below.
  *
  * Since: 2.37
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
