@@ -1,22 +1,18 @@
 /*
- *  $Id: gwyresource.c 21692 2018-11-26 13:29:49Z yeti-dn $
+ *  $Id: gwyresource.c 25036 2022-09-22 12:24:13Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -49,10 +45,9 @@ enum {
     PROP_LAST
 };
 
-/* Use a static propery -> trait map.  We could do it generically, too.
- * g_param_spec_pool_list() is ugly and slow is the minor problem, the major
- * is that it does g_hash_table_foreach() so we would get different orders
- * on different invocations and have to sort it. */
+/* Use a static propery -> trait map.  We could do it generically, too. g_param_spec_pool_list() is ugly and slow is
+ * the minor problem, the major is that it does g_hash_table_foreach() so we would get different orders on different
+ * invocations and have to sort it. */
 /* Threads: only modified in class init functon.  */
 static GType gwy_resource_trait_types[PROP_LAST-1];          /* Threads: OK */
 static const gchar *gwy_resource_trait_names[PROP_LAST-1];   /* Threads: OK */
@@ -85,6 +80,11 @@ static void         gwy_resource_modified       (GwyResource *resource);
 static void         gwy_resource_class_load_dir (const gchar *path,
                                                  GwyResourceClass *klass,
                                                  gboolean is_system);
+static void         save_resource_if_modified   (gpointer key,
+                                                 gpointer value,
+                                                 gpointer user_data);
+static gboolean     gwy_resource_save_real      (GwyResource *resource,
+                                                 GError **error);
 
 static guint resource_signals[LAST_SIGNAL] = { 0 };
 
@@ -121,30 +121,22 @@ gwy_resource_class_init(GwyResourceClass *klass)
     klass->item_type.type = G_TYPE_FROM_CLASS(klass);
     klass->data_changed = gwy_resource_modified;
 
-    pspec = g_param_spec_string("name",
-                                "Name",
-                                "Resource name",
-                                NULL, /* What is the default value good for? */
-                                G_PARAM_READABLE);
+    /* What is the default value good for? */
+    pspec = g_param_spec_string("name", "Name", "Resource name", NULL,
+                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
     g_object_class_install_property(gobject_class, PROP_NAME, pspec);
     gwy_resource_trait_types[gwy_resource_ntraits] = pspec->value_type;
     gwy_resource_trait_names[gwy_resource_ntraits] = pspec->name;
     gwy_resource_ntraits++;
 
-    pspec = g_param_spec_boolean("is-preferred",
-                                 "Is preferred",
-                                 "Whether a resource is preferred",
-                                 FALSE,
+    pspec = g_param_spec_boolean("is-preferred", "Is preferred", "Whether a resource is preferred", FALSE,
                                  G_PARAM_READWRITE);
     g_object_class_install_property(gobject_class, PROP_IS_PREFERRED, pspec);
     gwy_resource_trait_types[gwy_resource_ntraits] = pspec->value_type;
     gwy_resource_trait_names[gwy_resource_ntraits] = pspec->name;
     gwy_resource_ntraits++;
 
-    pspec = g_param_spec_boolean("is-const",
-                                 "Is constant",
-                                 "Whether a resource is constant (system)",
-                                 FALSE,
+    pspec = g_param_spec_boolean("is-const", "Is constant", "Whether a resource is constant (system)", FALSE,
                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
     g_object_class_install_property(gobject_class, PROP_IS_CONST, pspec);
     gwy_resource_trait_types[gwy_resource_ntraits] = pspec->value_type;
@@ -157,14 +149,13 @@ gwy_resource_class_init(GwyResourceClass *klass)
      *
      * The ::data-changed signal is emitted when resource data changes.
      */
-    resource_signals[DATA_CHANGED]
-        = g_signal_new("data-changed",
-                       G_OBJECT_CLASS_TYPE(gobject_class),
-                       G_SIGNAL_RUN_FIRST,
-                       G_STRUCT_OFFSET(GwyResourceClass, data_changed),
-                       NULL, NULL,
-                       g_cclosure_marshal_VOID__VOID,
-                       G_TYPE_NONE, 0);
+    resource_signals[DATA_CHANGED] = g_signal_new("data-changed",
+                                                  G_OBJECT_CLASS_TYPE(gobject_class),
+                                                  G_SIGNAL_RUN_FIRST,
+                                                  G_STRUCT_OFFSET(GwyResourceClass, data_changed),
+                                                  NULL, NULL,
+                                                  g_cclosure_marshal_VOID__VOID,
+                                                  G_TYPE_NONE, 0);
 }
 
 static void
@@ -195,6 +186,13 @@ gwy_resource_set_property(GObject *object,
     GwyResource *resource = GWY_RESOURCE(object);
 
     switch (prop_id) {
+        case PROP_NAME:
+        /* Only happens upon construction, and usually we just get NULL. GLib interprets G_PARAM_CONSTRUCT_ONLY as
+         * *must* be set upon construction, not *may* be only set upon construction. */
+        if (g_value_get_string(value))
+            g_string_assign(resource->name, g_value_get_string(value));
+        break;
+
         case PROP_IS_CONST:
         resource->is_const = g_value_get_boolean(value);
         break;
@@ -348,8 +346,7 @@ gwy_resource_get_is_preferred(GwyResource *resource)
 /**
  * gwy_resource_set_is_preferred:
  * @resource: A resource.
- * @is_preferred: %TRUE to make @resource preferred, %FALSE to make it not
- *                preferred.
+ * @is_preferred: %TRUE to make @resource preferred, %FALSE to make it not preferred.
  *
  * Sets preferability of a resource.
  **/
@@ -370,8 +367,7 @@ gwy_resource_set_is_preferred(GwyResource *resource,
  *
  * This is an simple identifier usable for example as directory name.
  *
- * Returns: Resource class name, as a constant string that must not be modified
- *          nor freed.
+ * Returns: Resource class name, as a constant string that must not be modified nor freed.
  **/
 const gchar*
 gwy_resource_class_get_name(GwyResourceClass *klass)
@@ -416,28 +412,21 @@ gwy_resource_class_get_item_type(GwyResourceClass *klass)
  *
  * Starts using a resource.
  *
- * Call to this function is necessary to use a resource properly.
- * It makes the resource to create any auxiliary structures that consume
- * considerable amount of memory and perform other initialization to
- * ready-to-use form.
+ * Call to this function is necessary to use a resource properly. It makes the resource to create any auxiliary
+ * structures that consume considerable amount of memory and perform other initialization to ready-to-use form.
  *
- * When a resource is no longer used, it should be released with
- * gwy_resource_release().
+ * When a resource is no longer used, it should be released with gwy_resource_release().
  *
  * In addition, it calls g_object_ref() on the resource.
  *
- * Resources usually exist through almost whole program lifetime from
- * #GObject perspective, but from the viewpoint of use this method is the
- * constructor and gwy_resource_release() is the destructor.
+ * Resources usually exist through almost whole program lifetime from #GObject perspective, but from the viewpoint of
+ * use this method is the constructor and gwy_resource_release() is the destructor.
  **/
 void
 gwy_resource_use(GwyResource *resource)
 {
     g_return_if_fail(GWY_IS_RESOURCE(resource));
-    gwy_debug("%s %p<%s> %d",
-              G_OBJECT_TYPE_NAME(resource),
-              resource, resource->name->str,
-              resource->use_count);
+    gwy_debug("%s %p<%s> %d", G_OBJECT_TYPE_NAME(resource), resource, resource->name->str, resource->use_count);
 
     g_object_ref(resource);
     if (!resource->use_count++) {
@@ -455,18 +444,14 @@ gwy_resource_use(GwyResource *resource)
  *
  * Releases a resource.
  *
- * When the number of resource uses drops to zero, it frees all auxiliary data
- * and returns back to `latent' form.  In addition, it calls g_object_unref()
- * on it.  See gwy_resource_use() for more.
+ * When the number of resource uses drops to zero, it frees all auxiliary data and returns back to `latent' form.  In
+ * addition, it calls g_object_unref() on it.  See gwy_resource_use() for more.
  **/
 void
 gwy_resource_release(GwyResource *resource)
 {
     g_return_if_fail(GWY_IS_RESOURCE(resource));
-    gwy_debug("%s %p<%s> %d",
-              G_OBJECT_TYPE_NAME(resource),
-              resource, resource->name->str,
-              resource->use_count);
+    gwy_debug("%s %p<%s> %d", G_OBJECT_TYPE_NAME(resource), resource, resource->name->str, resource->use_count);
     g_return_if_fail(resource->use_count);
 
     if (!--resource->use_count) {
@@ -525,8 +510,8 @@ gwy_resource_dump(GwyResource *resource)
 /**
  * gwy_resource_parse:
  * @text: Textual resource representation.
- * @expected_type: Resource object type.  If not 0, only resources of give type
- *                 are allowed.  Zero value means any #GwyResource is allowed.
+ * @expected_type: Resource object type.  If not 0, only resources of give type are allowed.  Zero value means any
+ *                 #GwyResource is allowed.
  *
  * Reconstructs a resource from human readable form.
  *
@@ -574,9 +559,16 @@ gwy_resource_parse_real(const gchar *text,
         goto fail;
     }
     klass = GWY_RESOURCE_CLASS(g_type_class_peek_static(type));
-    g_return_val_if_fail(klass && klass->parse, NULL);
+    g_return_val_if_fail(klass, resource);
 
-    resource = klass->parse(text, is_const);
+    if (klass->parse_with_type)
+        resource = klass->parse_with_type(type, text, is_const);
+    else if (klass->parse)
+        resource = klass->parse(text, is_const);
+    else {
+        g_assert_not_reached();
+        goto fail;
+    }
     if (resource)
         g_string_assign(resource->name, name);
 
@@ -592,8 +584,7 @@ fail:
  *
  * Emits signal "data-changed" on a resource.
  *
- * It can be called only on non-constant resources.  The default handler
- * sets @is_modified flag on the resource.
+ * It can be called only on non-constant resources.  The default handler sets @is_modified flag on the resource.
  *
  * Mostly useful in resource implementation.
  **/
@@ -635,8 +626,7 @@ gwy_resource_data_saved(GwyResource *resource)
  *
  * Loads resources of a resources class from disk.
  *
- * Resources are loaded from system directory (and marked constant) and from
- * user directory (marked modifiable).
+ * Resources are loaded from system directory (and marked constant) and from user directory (marked modifiable).
  **/
 void
 gwy_resource_class_load(GwyResourceClass *klass)
@@ -700,8 +690,7 @@ gwy_resource_class_load_dir(const gchar *path,
         }
         g_free(filename);
 
-        resource = gwy_resource_parse_real(text, G_TYPE_FROM_CLASS(klass),
-                                           is_system);
+        resource = gwy_resource_parse_real(text, G_TYPE_FROM_CLASS(klass), is_system);
         if (resource) {
             g_string_assign(resource->name, name);
             resource->is_modified = FALSE;
@@ -714,17 +703,45 @@ gwy_resource_class_load_dir(const gchar *path,
     g_dir_close(dir);
 }
 
+/**
+ * gwy_resource_class_save_modified:
+ * @klass: A resource class.
+ *
+ * Saves all modified resource of given class to disk.
+ *
+ * Since: 2.62
+ **/
+void
+gwy_resource_class_save_modified(GwyResourceClass *klass)
+{
+    g_return_if_fail(GWY_IS_RESOURCE_CLASS(klass));
+    g_return_if_fail(klass->inventory);
+    if (!gwy_resource_class_mkdir(klass)) {
+        g_warning("Resource directory cannot be created.");
+        return;
+    }
+    gwy_inventory_foreach(klass->inventory, save_resource_if_modified, NULL);
+}
+
+static void
+save_resource_if_modified(gpointer key, gpointer value, G_GNUC_UNUSED gpointer user_data)
+{
+    G_GNUC_UNUSED guint i = GPOINTER_TO_UINT(key);
+    GwyResource *resource = (GwyResource*)value;
+
+    if (resource->is_modified && !resource->is_const)
+        gwy_resource_save_real(resource, NULL);
+}
+
 /**
  * gwy_resource_build_filename:
  * @resource: A resource.
  *
  * Builds file name a resource should be saved to.
  *
- * If the resource has not been newly created, renamed, or system it was
- * probably loaded from file of the same name.
+ * If the resource has not been newly created, renamed, or system it was probably loaded from file of the same name.
  *
- * Returns: Resource file name as a newly allocated string that must be freed
- *          by caller.
+ * Returns: Resource file name as a newly allocated string that must be freed by caller.
  **/
 gchar*
 gwy_resource_build_filename(GwyResource *resource)
@@ -733,12 +750,10 @@ gwy_resource_build_filename(GwyResource *resource)
 
     g_return_val_if_fail(GWY_IS_RESOURCE(resource), NULL);
     if (resource->is_const)
-        g_warning("Filename of a constant resource `%s' should not be needed",
-                  resource->name->str);
+        g_warning("Filename of a constant resource `%s' should not be needed", resource->name->str);
 
     klass = GWY_RESOURCE_GET_CLASS(resource);
-    return g_build_filename(gwy_get_user_dir(),
-                            klass->name, resource->name->str, NULL);
+    return g_build_filename(gwy_get_user_dir(), klass->name, resource->name->str, NULL);
 }
 
 /**
@@ -747,8 +762,7 @@ gwy_resource_build_filename(GwyResource *resource)
  *
  * Deletes a resource, including removal from disk.
  *
- * The method deletes the resource both in the inventory and on disk.
- * Constant resources cannot be deleted.
+ * The method deletes the resource both in the inventory and on disk. Constant resources cannot be deleted.
  *
  * Returns: %TRUE if the removal succeeded.
  *
@@ -788,10 +802,9 @@ gwy_resource_delete(GwyResource *resource)
  *
  * Renames a resource, including renaming it on disk.
  *
- * The method renames the resource both in the inventory and on disk. The
- * renaming must not conflict with an existing resource, constant resources
- * cannot be renamed, etc.  It is OK to rename a resource to the same name
- * (nothing happens then).
+ * The method renames the resource both in the inventory and on disk. The renaming must not conflict with an existing
+ * resource, constant resources cannot be renamed, etc.  It is OK to rename a resource to the same name (nothing
+ * happens then).
  *
  * Returns: %TRUE if the renaming succeeded.
  *
@@ -832,8 +845,7 @@ gwy_resource_rename(GwyResource *resource,
     retval = g_rename(oldfilename, newfilename);
     if (retval != 0) {
         /* FIXME: GUIze this */
-        g_warning("Cannot rename resource file: %s to %s",
-                  oldfilename, newfilename);
+        g_warning("Cannot rename resource file: %s to %s", oldfilename, newfilename);
         gwy_inventory_rename_item(inventory, newname, oldname);
     }
     g_free(newfilename);
@@ -843,15 +855,68 @@ gwy_resource_rename(GwyResource *resource,
     return retval == 0;
 }
 
+/**
+ * gwy_resource_save:
+ * @resource: A resource.
+ * @error: Return location for a #GError.
+ *
+ * Saves a resource to disk.
+ *
+ * Only non-constant resources can be saved. The file name is determined by gwy_resource_build_filename(). The
+ * resource data are saved even if the modified flag is not set. Upon successful save, the modified flag is cleared.
+ *
+ * Instead of saving individual resources, consider also using gwy_resource_class_save_modified().
+ *
+ * Returns: %TRUE if resource was saved to disk, %FALSE otherwise.
+ *
+ * Since: 2.62
+ **/
+gboolean
+gwy_resource_save(GwyResource *resource,
+                  GError **error)
+{
+    /* Return TRUE on assertions as we do not set the error. */
+    g_return_val_if_fail(GWY_IS_RESOURCE(resource), TRUE);
+    if (resource->is_const) {
+        g_warning("Constant resource is being passed to gwy_resource_save()");
+        return TRUE;
+    }
+
+    if (!gwy_resource_class_mkdir(GWY_RESOURCE_GET_CLASS(resource))) {
+        g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOTDIR, _("Resource directory cannot be created."));
+        return FALSE;
+    }
+
+    return gwy_resource_save_real(resource, error);
+}
+
+static gboolean
+gwy_resource_save_real(GwyResource *resource,
+                       GError **error)
+{
+    gchar *filename;
+    gboolean ok;
+    GString *str;
+
+    filename = gwy_resource_build_filename(resource);
+    str = gwy_resource_dump(resource);
+    if ((ok = g_file_set_contents(filename, str->str, str->len, error)))
+        gwy_resource_data_saved(resource);
+
+    g_string_free(str, TRUE);
+    g_free(filename);
+
+    return ok;
+}
+
 /**
  * gwy_resource_class_mkdir:
  * @klass: A resource class.
  *
  * Creates directory for user resources if it does not exist.
  *
- * Returns: %TRUE if the directory exists or has been successfully created.
- *          %FALSE if it doesn't exist and cannot be created, consult errno
- *          for reason.
+ * Returns: %TRUE if the directory exists or has been successfully created. %FALSE if it doesn't exist and cannot be
+ *          created, consult errno for reason.
  **/
 gboolean
 gwy_resource_class_mkdir(GwyResourceClass *klass)
@@ -878,12 +943,10 @@ gwy_resource_class_mkdir(GwyResourceClass *klass)
  *
  * Destroys the inventories of all resource classes.
  *
- * This function makes the affected resource classes unusable.  Its purpose is
- * to faciliate reference leak debugging by destroying a large number of
- * objects that normally live forever.
+ * This function makes the affected resource classes unusable.  Its purpose is to faciliate reference leak debugging
+ * by destroying a large number of objects that normally live forever.
  *
- * Note static resource classes that never called gwy_resource_class_load()
- * are excluded.
+ * Note static resource classes that never called gwy_resource_class_load() are excluded.
  *
  * Since: 2.8
  **/
@@ -913,35 +976,34 @@ gwy_resource_classes_finalize(void)
  * @short_description: Built-in and/or user supplied application resources
  * @see_also: #GwyInventory
  *
- * #GwyResource is a base class for various application resources.  It defines
- * common interface: questioning resource name (gwy_resource_get_name()),
- * modifiability (gwy_resource_get_is_modifiable()), loading resources from
- * files and saving them.
+ * #GwyResource is a base class for various application resources.  It defines common interface: questioning resource
+ * name (gwy_resource_get_name()), modifiability (gwy_resource_get_is_modifiable()), loading resources from files and
+ * saving them.
  **/
 
 /**
  * GwyResource:
  *
- * The #GwyResource struct contains private data only and should be accessed
- * using the functions below.
+ * The #GwyResource struct contains private data only and should be accessed using the functions below.
  **/
 
 /**
  * GwyResourceClass:
  * @inventory: Inventory with resources.
- * @name: Resource class name, usable as resource directory name for on-disk
- *        resources.
- * @item_type: Inventory item type.  Most fields are pre-filled, but namely
- *             @type and @copy must be filled by particular resource type.
+ * @name: Resource class name, usable as resource directory name for on-disk resources.
+ * @item_type: Inventory item type.  Most fields are pre-filled, but namely @type and @copy must be filled by
+ *             particular resource type.
  * @data_changed: "data-changed" signal method.
  * @use: gwy_resource_use() virtual method.
  * @release: gwy_resource_release() virtual method.
- * @dump: gwy_resource_dump() virtual method, it only cares about resource
- *        data itself, the envelope is handled by #GwyResource.
- * @parse: gwy_resource_parse() virtual method, in only cares about resource
- *         data itself, the envelope is handled by #GwyResource.
+ * @dump: gwy_resource_dump() virtual method. It only cares about resource data itself, the envelope is handled by
+ *        #GwyResource.
+ * @parse: gwy_resource_parse() virtual method. It only cares about resource data itself, the envelope is handled by
+ *         #GwyResource. This is the usual method as the parse normally knows its type.
+ * @parse_with_type: gwy_resource_parse() virtual method. It only cares about resource data itself, the envelope is
+ *                   handled by #GwyResource. It takes precedence over @parse if both are defined. (Since 2.62)
  *
  * Resource class.
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
