@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyresults.c 24606 2022-02-16 14:45:59Z yeti-dn $
+ *  $Id: gwyresults.c 25196 2023-01-10 12:59:20Z yeti-dn $
  *  Copyright (C) 2017-2022 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -91,6 +91,7 @@ typedef struct {
     gshort power_x;
     gshort power_y;
     gshort power_z;
+    gshort precision;
     guint bind_group;
     guint label_width;   /* Just a cache; varies according to translation. */
     guint covar_n;
@@ -496,6 +497,8 @@ handle_value_properties(ResultLine *line, va_list ap)
             line->power_y = va_arg(ap, gint);
         else if (gwy_strequal(propname, "power-z"))
             line->power_z = va_arg(ap, gint);
+        else if (gwy_strequal(propname, "precision"))
+            line->precision = va_arg(ap, gint);
         else if (gwy_strequal(propname, "is-angle"))
             line->is_angle = !!va_arg(ap, gint);  /* promotion */
         else if (gwy_strequal(propname, "is-fitting-param"))
@@ -544,6 +547,7 @@ add_result_line_base(GwyResults *results,
     line->parent = priv;
     line->label = label;
     line->type = type;
+    line->precision = -1;
     priv->n++;
 
     return line;
@@ -568,6 +572,7 @@ add_result_line_base(GwyResults *results,
  * <member><literal>"power-y"</literal>, power of unit @y (#gint)</member>
  * <member><literal>"power-z"</literal>, power of unit @z (#gint)</member>
  * <member><literal>"power-w"</literal>, power of unit @w (#gint)</member>
+ * <member><literal>"precision"</literal>, the mininum number of significant digits (#gint, since 2.63)</member>
  * <member><literal>"is-angle"</literal>, %TRUE if the quantity should converted to degrees for humans, this overrides any unit powers (#gboolean)</member>
  * <member><literal>"is-fitting-param"</literal>, %TRUE if the quantity should be reported as fixed parameter when it does not have error set (#gboolean)</member>
  * <member><literal>"is-percents"</literal>, %TRUE if the quantity should converted to percents for humans, this overrides any unit powers (#gboolean)</member>
@@ -1783,6 +1788,7 @@ construct_value_format(GwySIValueFormat *vf, ResultLine *line,
     gchar *s = NULL;
 
     vf->magnitude = 1.0;
+    digits = MAX(digits, line->precision);
     vf->precision = (for_machine ? 5 : digits);
     if (line->unit_str) {
         sc = maybe_translate_string(line->unit_str, line->translate_unit, !for_machine);
@@ -1794,7 +1800,7 @@ construct_value_format(GwySIValueFormat *vf, ResultLine *line,
         else {
             gwy_si_unit_value_format_set_units(vf, _("deg"));
             vf->magnitude = G_PI/180.0;
-            vf->precision = 2;
+            vf->precision = MAX(2, line->precision-2);
         }
     }
     else if (line->is_percents) {
@@ -1803,7 +1809,7 @@ construct_value_format(GwySIValueFormat *vf, ResultLine *line,
         else {
             gwy_si_unit_value_format_set_units(vf, "%");
             vf->magnitude = 0.01;
-            vf->precision = 3;
+            vf->precision = MAX(3, line->precision-1);
         }
     }
     else {
