@@ -1,22 +1,18 @@
 /*
- *  $Id: gwystringlist.c 21680 2018-11-26 10:39:39Z yeti-dn $
+ *  $Id: gwystringlist.c 25969 2023-10-27 14:47:05Z yeti-dn $
  *  Copyright (C) 2004,2014 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /* FIXME: we should use a better backend and also add more methods, for the
@@ -38,18 +34,18 @@ enum {
     LAST_SIGNAL
 };
 
-static void        gwy_string_list_finalize       (GObject *object);
-static void      gwy_string_list_serializable_init(GwySerializableIface *iface);
-static GByteArray* gwy_string_list_serialize      (GObject *obj,
-                                                   GByteArray *buffer);
-static gsize       gwy_string_list_get_size       (GObject *obj);
-static GObject*    gwy_string_list_deserialize    (const guchar *buffer,
-                                                   gsize size,
-                                                   gsize *position);
-static GObject*    gwy_string_list_duplicate_real (GObject *object);
-static void        gwy_string_list_clone_real     (GObject *source,
-                                                   GObject *copy);
-static void        clear_strings                  (GPtrArray *strings);
+static void        gwy_string_list_finalize         (GObject *object);
+static void        gwy_string_list_serializable_init(GwySerializableIface *iface);
+static GByteArray* gwy_string_list_serialize        (GObject *obj,
+                                                     GByteArray *buffer);
+static gsize       gwy_string_list_get_size         (GObject *obj);
+static GObject*    gwy_string_list_deserialize      (const guchar *buffer,
+                                                     gsize size,
+                                                     gsize *position);
+static GObject*    gwy_string_list_duplicate_real   (GObject *object);
+static void        gwy_string_list_clone_real       (GObject *source,
+                                                     GObject *copy);
+static void        clear_strings                    (GPtrArray *strings);
 
 static guint string_list_signals[LAST_SIGNAL] = { 0 };
 
@@ -132,9 +128,7 @@ gwy_string_list_serialize(GObject *obj,
         };
         guint n = len ? G_N_ELEMENTS(spec) : 0;
 
-        retval = gwy_serialize_pack_object_struct(buffer,
-                                                  GWY_STRING_LIST_TYPE_NAME,
-                                                  n, spec);
+        retval = gwy_serialize_pack_object_struct(buffer, GWY_STRING_LIST_TYPE_NAME, n, spec);
         return retval;
     }
 }
@@ -157,8 +151,7 @@ gwy_string_list_get_size(GObject *obj)
         };
         guint n = len ? G_N_ELEMENTS(spec) : 0;
 
-        size = gwy_serialize_get_struct_size(GWY_STRING_LIST_TYPE_NAME,
-                                             n, spec);
+        size = gwy_serialize_get_struct_size(GWY_STRING_LIST_TYPE_NAME, n, spec);
     }
 
     return size;
@@ -180,8 +173,7 @@ gwy_string_list_deserialize(const guchar *buffer,
 
     g_return_val_if_fail(buffer, NULL);
 
-    if (!gwy_serialize_unpack_object_struct(buffer, size, position,
-                                            GWY_STRING_LIST_TYPE_NAME,
+    if (!gwy_serialize_unpack_object_struct(buffer, size, position, GWY_STRING_LIST_TYPE_NAME,
                                             G_N_ELEMENTS(spec), spec)) {
         return NULL;
     }
@@ -212,8 +204,7 @@ gwy_string_list_duplicate_real(GObject *object)
     dupstrings = (GPtrArray*)duplicate->strings;
     g_ptr_array_set_size(dupstrings, strings->len);
     for (i = 0; i < strings->len; i++)
-        g_ptr_array_index(dupstrings, i)
-            = g_strdup(g_ptr_array_index(strings, i));
+        g_ptr_array_index(dupstrings, i) = g_strdup(g_ptr_array_index(strings, i));
 
     return (GObject*)duplicate;
 }
@@ -237,8 +228,7 @@ gwy_string_list_clone_real(GObject *source, GObject *copy)
         g_free(g_ptr_array_index(copystrings, i));
     g_ptr_array_set_size(copystrings, strings->len);
     for (i = 0; i < strings->len; i++)
-        g_ptr_array_index(copystrings, i)
-            = g_strdup(g_ptr_array_index(strings, i));
+        g_ptr_array_index(copystrings, i) = g_strdup(g_ptr_array_index(strings, i));
 
     g_signal_emit(copy, string_list_signals[VALUE_CHANGED], 0);
 }
@@ -328,8 +318,7 @@ gwy_string_list_get_length(GwyStringList *strlist)
  *
  * Gets a string from a string list by position.
  *
- * Returns: The string, owned by @strlist.  It is valid only until @strlist
- *          changes.
+ * Returns: The string, owned by @strlist.  It is valid only until @strlist changes.
  **/
 const gchar*
 gwy_string_list_get(GwyStringList *strlist,
@@ -403,4 +392,4 @@ clear_strings(GPtrArray *strings)
  * typecasting.
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
