@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyutils.c 25373 2023-05-26 13:57:05Z yeti-dn $
+ *  $Id: gwyutils.c 25721 2023-09-26 15:37:05Z yeti-dn $
  *  Copyright (C) 2003-2023 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -104,321 +104,6 @@ gwy_hash_table_to_list_cb(G_GNUC_UNUSED gpointer unused_key,
     *list = g_list_prepend(*list, value);
 }
 
-/**
- * gwy_strkill:
- * @s: A NUL-terminated string.
- * @killchars: A string containing characters to kill.
- *
- * Removes characters in @killchars from string @s, modifying it in place.
- *
- * Use gwy_strkill(g_strdup(@s), @killchars) to get a modified copy.
- *
- * Returns: @s itself, the return value is to allow function call nesting.
- **/
-gchar*
-gwy_strkill(gchar *s,
-            const gchar *killchars)
-{
-    gchar *p, *q;
-    gchar killc;
-
-    if (!killchars || !*killchars)
-        return s;
-    killc = *killchars;
-    if (killchars[1])
-        g_strdelimit(s, killchars+1, killc);
-    if ((p = strchr(s, killc))) {
-        for (q = p; *p; p++) {
-            if (*p != killc) {
-                *q = *p;
-                q++;
-            }
-        }
-        *q = '\0';
-    }
-
-    return s;
-}
-
-/**
- * gwy_strreplace:
- * @haystack: A NUL-terminated string to search in.
- * @needle: A NUL-terminated string to search for.
- * @replacement: A NUL-terminated string to replace @needle with.
- * @maxrepl: Maximum number of occurences to replace (use (gsize)-1 to replace all occurences).
- *
- * Replaces occurences of string @needle in @haystack with @replacement.
- *
- * See gwy_gstring_replace() for a function which does in-place replacement on a #GString.
- *
- * Returns: A newly allocated string.
- **/
-gchar*
-gwy_strreplace(const gchar *haystack,
-               const gchar *needle,
-               const gchar *replacement,
-               gsize maxrepl)
-{
-    gsize n, hlen, nlen, rlen, newlen;
-    const gchar *p, *pp;
-    gchar *dest, *q;
-
-    nlen = strlen(needle);
-    g_return_val_if_fail(nlen, NULL);
-    n = 0;
-    p = haystack;
-    while ((p = strstr(p, needle)) && n < maxrepl) {
-        p += nlen;
-        n++;
-    }
-    if (!n)
-        return g_strdup(haystack);
-
-    hlen = strlen(haystack);
-    rlen = strlen(replacement);
-    newlen = hlen + n*rlen - n*nlen;
-
-    dest = g_new(gchar, newlen+1);
-    pp = haystack;
-    q = dest;
-    n = 0;
-    while ((p = strstr(pp, needle)) && n < maxrepl) {
-        memcpy(q, pp, p - pp);
-        q += p - pp;
-        memcpy(q, replacement, rlen);
-        q += rlen;
-        pp = p + nlen;
-        n++;
-    }
-    strcpy(q, pp);
-
-    return dest;
-}
-
-/**
- * gwy_strdiffpos:
- * @s1: A string.
- * @s2: A string.
- *
- * Finds position where two strings differ.
- *
- * Returns: The last position where the strings do not differ yet.  Possibly -1 can be returned if either string is
- *          %NULL, zero-length, or they differ in the very first character.
- **/
-gint
-gwy_strdiffpos(const gchar *s1, const gchar *s2)
-{
-    const gchar *ss = s1;
-
-    if (!s1 || !s2)
-        return -1;
-
-    while (*s1 && *s2 && *s1 == *s2) {
-        s1++;
-        s2++;
-    }
-
-    return (s1 - ss) - 1;
-}
-
-/**
- * gwy_strisident:
- * @s: A NUL-terminated string.
- * @more: List of additional ASCII characters allowed inside identifier, empty list can be passed as %NULL.
- * @startmore: List of additional ASCII characters allowed as the first identifier characters, empty list can be
- *             passed as %NULL.
- *
- * Checks whether a string is valid identifier.
- *
- * Valid identifier must start with an alphabetic character or a character from @startmore, and it must continue with
- * alphanumeric characters or characters from @more.
- *
- * Note underscore is not allowed by default, you have to pass it in @more and/or @startmore.
- *
- * Returns: %TRUE if @s is valid identifier, %FALSE otherwise.
- **/
-gboolean
-gwy_strisident(const gchar *s,
-               const gchar *more,
-               const gchar *startmore)
-{
-    const gchar *m;
-
-    g_return_val_if_fail(s, FALSE);
-    if (!g_ascii_isalpha(*s)) {
-        if (!startmore)
-            return FALSE;
-        for (m = startmore; *m; m++) {
-            if (*s == *m)
-                break;
-        }
-        if (!*m)
-            return FALSE;
-    }
-    s++;
-
-    while (*s) {
-        if (!g_ascii_isalnum(*s)) {
-            if (!more)
-                return FALSE;
-            for (m = more; *m; m++) {
-                if (*s == *m)
-                    break;
-            }
-            if (!*m)
-                return FALSE;
-        }
-        s++;
-    }
-
-    return TRUE;
-}
-
-/**
- * gwy_ascii_strcase_equal:
- * @v1: String key.
- * @v2: String key to compare with @v1.
- *
- * Compares two strings for equality, ignoring case.
- *
- * The case folding is performed only on ASCII characters.
- *
- * This function is intended to be passed to g_hash_table_new() as @key_equal_func, namely in conjuction with
- * gwy_ascii_strcase_hash() hashing function.
- *
- * Returns: %TRUE if the two string keys match, ignoring case.
- *
- * Since: 2.26
- */
-gboolean
-gwy_ascii_strcase_equal(gconstpointer v1,
-                        gconstpointer v2)
-{
-    const gchar *s1 = (const gchar*)v1, *s2 = (const gchar*)v2;
-
-    while (*s1 && *s2) {
-        if (TOLOWER(*s1) != TOLOWER(*s2))
-            return FALSE;
-        s1++, s2++;
-    }
-    return !*s1 && !*s2;
-}
-
-/**
- * gwy_ascii_strcase_hash:
- * @v: String key.
- *
- * Converts a string to a hash value, ignoring case.
- *
- * The case folding is performed only on ASCII characters.
- *
- * This function is intended to be passed to g_hash_table_new() as @hash_func, namely in conjuction with
- * gwy_ascii_strcase_equal() comparison function.
- *
- * Returns: The hash value corresponding to the key @v.
- *
- * Since: 2.26
- */
-guint
-gwy_ascii_strcase_hash(gconstpointer v)
-{
-    const signed char *p;
-    guint32 h = 5381;
-
-    for (p = v; *p != '\0'; p++)
-        h = (h << 5) + h + TOLOWER(*p);
-
-    return h;
-}
-
-/**
- * gwy_stramong:
- * @str: A string to find.
- * @...: %NULL-terminated list of string to compare @str with.
- *
- * Checks whether a string is equal to any from given list.
- *
- * Returns: Zero if @str does not equal to any string from the list, nozero othwerise.  More precisely, the position
- *          + 1 of the first string @str equals to is returned in the latter case.
- *
- * Since: 2.11
- **/
-guint
-gwy_stramong(const gchar *str,
-             ...)
-{
-    va_list ap;
-    const gchar *s;
-    guint i = 1;
-
-    va_start(ap, str);
-    while ((s = va_arg(ap, const gchar*))) {
-        if (gwy_strequal(str, s)) {
-            va_end(ap);
-            return i;
-        }
-        i++;
-    }
-    va_end(ap);
-
-    return 0;
-}
-
-/**
- * gwy_memmem:
- * @haystack: Memory block to search in.
- * @haystack_len: Size of @haystack, in bytes.
- * @needle: Memory block to find.
- * @needle_len: Size of @needle, in bytes.
- *
- * Find a block of memory in another block of memory.
- *
- * This function is very similar to strstr(), except that it works with arbitrary memory blocks instead of
- * NUL-terminated strings.
- *
- * If @needle_len is zero, @haystack is always returned.
- *
- * On GNU systems with glibc at least 2.1 this is a just a trivial memmem() wrapper.  On other systems it emulates
- * memmem() behaviour but may be a bit slower.
- *
- * Returns: Pointer to the first byte of memory block in @haystack that matches @needle; %NULL if no such block
- *          exists.
- *
- * Since: 2.12
- **/
-gpointer
-gwy_memmem(gconstpointer haystack,
-           gsize haystack_len,
-           gconstpointer needle,
-           gsize needle_len)
-{
-#ifdef HAVE_MEMMEM
-    return memmem(haystack, haystack_len, needle, needle_len);
-#else
-    const guchar *h = haystack;
-    const guchar *s, *p;
-    guchar n0;
-
-    /* Empty needle can be found anywhere */
-    if (!needle_len)
-        return (gpointer)haystack;
-
-    /* Needle that doesn't fit cannot be found anywhere */
-    if (needle_len > haystack_len)
-        return NULL;
-
-    /* The general case */
-    n0 = *(const guchar*)needle;
-    s = h + haystack_len - needle_len;
-    for (p = h; p && p <= s; p = memchr(p, n0, s-p + 1)) {
-        if (memcmp(p, needle, needle_len) == 0)
-            return (gpointer)p;
-        p++;
-    }
-    return NULL;
-#endif
-}
-
 /**
  * gwy_file_get_contents:
  * @filename: A file to read contents of.
@@ -931,282 +616,6 @@ gwy_filename_ignore(const gchar *filename_sys)
     return FALSE;
 }
 
-/**
- * gwy_str_next_line:
- * @buffer: Text buffer.
- *
- * Extracts a next line from a character buffer, modifying it in place.
- *
- * @buffer is updated to point after the end of the line and the "\n" (or "\r" or "\r\n") is replaced with "\0", if
- * present.
- *
- * The final line may or may not be terminated with an EOL marker, its contents is returned in either case.  Note,
- * however, that the empty string "" is not interpreted as an empty unterminated line.  Instead, %NULL is immediately
- * returned.
- *
- * The typical usage of gwy_str_next_line() is:
- * |[
- * gchar *p = text;
- * for (gchar *line = gwy_str_next_line(&p);
- *      line;
- *      line = gwy_str_next_line(&p)) {
- *     g_strstrip(line);
- *     // Do something more with line
- * }
- * ]|
- *
- * Returns: The start of the line.  %NULL if the buffer is empty or %NULL. The return value is
- *          <emphasis>not</emphasis> a new string; the normal return value is the previous value of @buffer.
- **/
-gchar*
-gwy_str_next_line(gchar **buffer)
-{
-    gchar *p, *q;
-
-    if (!buffer || !*buffer)
-        return NULL;
-
-    q = *buffer;
-    if (!*q) {
-        *buffer = NULL;
-        return NULL;
-    }
-
-    for (p = q; *p != '\n' && *p != '\r' && *p; p++)
-        ;
-    if (p[0] == '\r' && p[1] == '\n') {
-        *(p++) = '\0';
-        *(p++) = '\0';
-    }
-    else if (p[0]) {
-        *(p++) = '\0';
-    }
-
-    *buffer = p;
-    return q;
-}
-
-/**
- * gwy_str_fixed_font_width:
- * @str: UTF-8 encoded string.
- *
- * Measures the width of UTF-8 encoded string in fixed-width font.
- *
- * This corresponds to width of the string displayed on a text terminal, for instance.  Zero and double width
- * characters are taken into account.  It is not guaranteed all terminals display the string with the calculated
- * width.
- *
- * Returns: String width in fixed font, in character cells.
- *
- * Since: 2.52
- **/
-guint
-gwy_str_fixed_font_width(const gchar *str)
-{
-    guint w = 0;
-    gunichar u;
-
-    while (str && *str) {
-        u = g_utf8_get_char(str);
-        if (g_unichar_iswide(u))
-            w += 2;
-        else if (g_unichar_iszerowidth(u))
-            w += 0;
-        else
-            w += 1;
-        str = g_utf8_next_char(str);
-    }
-
-    return w;
-}
-
-/**
- * gwy_gstring_replace:
- * @str: A #GString string to modify in place.
- * @old: The character sequence to find and replace.  Passing %NULL is the same as passing the empty string.
- * @replacement: The character sequence that should replace @old.  Passing %NULL is the same as passing the empty
- *               string.
- * @count: The maximum number of replacements to make.  A negative number means replacing all occurrences of @old.
- *         Note zero means just zero, i.e. no replacements are made.
- *
- * Replaces non-overlapping occurrences of one string with another in a #GString.
- *
- * Passing %NULL or the empty string for @replacement will cause the occurrences of @old to be removed.
- *
- * Passing %NULL or the empty string for @old means a match occurs at every position in the string, including after
- * the last character.  So @replacement will be inserted at every position in this case.
- *
- * See gwy_strreplace() for a function which creates a new plain C string with substring replacement.
- *
- * Returns: The number of replacements made.  A non-zero value means the string has been modified, no-op replacements
- *          do not count.
- *
- * Since: 2.36
- **/
-guint
-gwy_gstring_replace(GString *str,
-                    const gchar *old,
-                    const gchar *replacement,
-                    gint count)
-{
-    guint oldlen, repllen, newlen, ucount, n, i;
-    gchar *p, *q, *newp, *newstr;
-
-    g_return_val_if_fail(str, 0);
-
-    if (!old)
-        old = "";
-    oldlen = strlen(old);
-
-    /* Do we need to do anywork at all? */
-    if (!count)
-        return 0;
-    ucount = (count < 0) ? G_MAXUINT : (guint)count;
-
-    p = str->str;
-    if (oldlen && !(p = strstr(str->str, old)))
-        return 0;
-
-    if (!replacement)
-        replacement = "";
-    repllen = strlen(replacement);
-
-    /* Equal lengths, we can do the replacement in place easily. */
-    if (oldlen == repllen) {
-        if (gwy_strequal(old, replacement))
-            return 0;
-
-        n = 0;
-        while (p) {
-            memcpy(p, replacement, repllen);
-            if (++n == ucount)
-                break;
-            p = strstr(p + oldlen, old);
-        }
-
-        return n;
-    }
-
-    /* Empty old string: the slightly silly case.  It has a different oldlen semantics so handle it specially. */
-    if (!oldlen) {
-        gchar *oldcopy;
-        guint len;
-
-        ucount = MIN(ucount, str->len + 1);
-
-        if (ucount == 1) {
-            g_string_prepend(str, replacement);
-            return 1;
-        }
-
-        oldcopy = g_strdup(str->str);
-        len = str->len;
-        g_string_set_size(str, str->len + ucount*repllen);
-        g_string_truncate(str, 0);
-        p = str->str;
-        for (i = 0; i < ucount; i++) {
-            memcpy(p, replacement, repllen);
-            p += repllen;
-            *p = oldcopy[i];
-            p++;
-        }
-
-        if (ucount < len)
-            memcpy(p, oldcopy + ucount, len - ucount);
-
-        g_free(oldcopy);
-
-        return ucount;
-    }
-
-    /* The general case.  Count the actual replacement number. */
-    n = 0;
-    for (q = p; q; q = strstr(q + oldlen, old)) {
-        if (++n == ucount)
-            break;
-    }
-    ucount = MIN(n, ucount);
-
-    newlen = str->len;
-    if (repllen >= oldlen)
-        newlen += ucount*(repllen - oldlen);
-    else
-        newlen -= ucount*(oldlen - repllen);
-
-    if (!newlen) {
-        g_string_truncate(str, 0);
-        return ucount;
-    }
-
-    /* For just one replacement, do the operation directly. */
-    if (ucount == 1) {
-        guint pos = p - str->str;
-
-        if (repllen > oldlen) {
-            g_string_insert_len(str, pos, replacement, repllen - oldlen);
-            memcpy(str->str + pos + (repllen - oldlen), replacement + (repllen - oldlen), oldlen);
-        }
-        else {
-            g_string_erase(str, pos, oldlen - repllen);
-            memcpy(str->str + pos, replacement, repllen);
-        }
-        return 1;
-    }
-
-    /* For more replacements, rebuild the string from scratch in a buffer. */
-    newstr = g_new(gchar, newlen);
-
-    memcpy(newstr, str->str, p - str->str);
-    newp = newstr + (p - str->str);
-
-    n = 0;
-    for (q = p; q; p = q) {
-        if (repllen) {
-            memcpy(newp, replacement, repllen);
-            newp += repllen;
-        }
-
-        if (++n == ucount)
-            break;
-        if (!(q = strstr(q + oldlen, old)))
-            break;
-
-        memcpy(newp, p + oldlen, (q - p) - oldlen);
-        newp += (q - p) - oldlen;
-    }
-
-    memcpy(newp, p + oldlen, str->len - oldlen - (p - str->str));
-
-    g_string_truncate(str, 0);
-    g_string_append_len(str, newstr, newlen);
-    g_free(newstr);
-
-    return ucount;
-}
-
-/**
- * gwy_gstring_to_native_eol:
- * @str: A #GString string to modify in place.
- *
- * Converts "\n" in a string to operating system native line terminators.
- *
- * Text files are most easily written by opening them in the text mode.  This function can be useful for writing text
- * files using functions such as g_file_set_contents() that do not permit the conversion to happen automatically.
- *
- * It is a no-op on all POSIX systems, including OS X.  So at present, it actually performs any conversion at all only
- * on MS Windows.
- *
- * Since: 2.36
- **/
-void
-gwy_gstring_to_native_eol(GString *str)
-{
-    g_return_if_fail(str);
-#ifdef G_OS_WIN32
-    gwy_gstring_replace(str, "\n", "\r\n", -1);
-#endif
-}
-
 /**
  * gwy_memcpy_byte_swap:
  * @source: Source memory block.
@@ -1534,193 +943,6 @@ gwy_raw_data_size(GwyRawDataType datatype)
     return sizes[datatype];
 }
 
-/**
- * gwy_utf16_to_utf8:
- * @str: A UTF-16 encoded string.
- * @len: Maximum string length (number of #gunichar2 items in @str, NOT the number of bytes).  May be -1 if @str is
- *       nul-terminated.
- * @byteorder: Byte order of @str.
- *
- * Convert a string from UTF-16 to UTF-8. The result will be terminated with a 0 byte.
- *
- * This functions differs from g_utf16_to_utf8() mainly by the handling of byte order.  In particular, the caller
- * specifies the byte order explicitly and it can differ from the native byte order.
- *
- * It is possible to pass %GWY_BYTE_ORDER_IMPLICIT as @byteorder.  In such case @str is checked for a byte-order mark.
- * When one is present it is used for the byte order.  Otherwise the behaviour is the same as for
- * %GWY_BYTE_ORDER_NATIVE.  The output string never begins with a byte-order mark.
- *
- * Conversion failure is indicated just by a %NULL return value.  Conversion is either completely successful or fails.
- * Use g_utf16_to_utf8() if you need to handle partial conversions and failure details.
- *
- * Returns: A newly allocated string on success; %NULL on failure.
- *
- * Since: 2.60
- **/
-gchar*
-gwy_utf16_to_utf8(const gunichar2 *str,
-                  glong len,
-                  GwyByteOrder byteorder)
-{
-    gboolean byteswap = (byteorder && byteorder != G_BYTE_ORDER && byteorder != GWY_BYTE_ORDER_IMPLICIT);
-    gunichar2 maybebom;
-    gunichar2 *swapped;
-    gchar *retval;
-    gboolean isrealbom = FALSE;
-
-    /* Whatever... */
-    if (!str)
-        return NULL;
-
-    if (len < 0) {
-        while (str[len])
-            len++;
-    }
-    if (!len)
-        return g_strdup("");
-
-    maybebom = str[0];
-    if (maybebom == 0xfeff || maybebom == 0xfffe) {
-        if (byteorder == GWY_BYTE_ORDER_IMPLICIT) {
-            isrealbom = TRUE;
-            byteswap = (maybebom == 0xfffe);
-        }
-        else if (byteswap && maybebom == 0xfffe)
-            isrealbom = TRUE;
-        else if (!byteswap && maybebom == 0xfeff)
-            isrealbom = TRUE;
-        /* In all other cases the caller is perhaps confused, but the conversion should fail.  Do not guess. */
-    }
-    if (isrealbom) {
-        str++;
-        len--;
-    }
-
-    if (!byteswap)
-        return g_utf16_to_utf8(str, len, NULL, NULL, NULL);
-
-    swapped = g_new(gunichar2, len+1);
-    gwy_memcpy_byte_swap((const guint8*)str, (guint8*)swapped, 2, len, 1);
-    swapped[len] = 0;
-    retval = g_utf16_to_utf8(swapped, len, NULL, NULL, NULL);
-    g_free(swapped);
-
-    return retval;
-}
-
-/**
- * gwy_utf8_strescape:
- * @source: A string to escape.
- * @exceptions: A string containing characters to not escape, or %NULL.
- *
- * Escapes special characters in a string.
- *
- * Escapes the special characters '\b', '\f', '\n', '\r', '\t', '\v', '\' and '"' in the string source by inserting
- * a '\' before them. Additionally all characters in the range 0x01-0x1F (everything below SPACE), the character 0x7F,
- * characters 0xC0 and 0xC1, and all characters in the range 0xF5-0xFF are replaced with a '\' followed by their octal
- * representation. Characters supplied in exceptions are not escaped.
- *
- * In essence, this functions differs from g_strescape() by preserving valid UTF-8 strings, unless they contain ‘odd’
- * characters. Passing strings which are not valid UTF-8 is possible, but generally not advised.
- *
- * Function g_strcompress() does the reverse conversion.
- *
- * Returns: A newly allocated string with special characters escaped.
- *
- * Since: 2.62
- **/
-gchar*
-gwy_utf8_strescape(const gchar *source,
-                   const gchar *exceptions)
-{
-    static const gchar stdescapes[] = { 'b', 't', 'n', 'v', 'f', 'r' };
-    const guchar *p = (guchar*)source;
-    guchar *dest, *q;
-    guchar c, excmap[256];
-
-    g_return_val_if_fail(source, NULL);
-
-    /* Each source byte needs maximally four destination chars (\777) */
-    q = dest = g_malloc(4*strlen(source) + 1);
-
-    if (exceptions && *exceptions) {
-        guchar *e = (guchar*)exceptions;
-
-        gwy_clear(excmap, 256);
-        do {
-            excmap[*(e++)] = 1;
-        } while (*e);
-    }
-    else
-        exceptions = NULL;
-
-    while ((c = *p)) {
-        if (exceptions && excmap[c])
-            *(q++) = c;
-        else if (c >= '\b' && c <= '\r') {
-            *(q++) = '\\';
-            *(q++) = stdescapes[c - '\b'];
-        }
-        else if (c == '\\' || c == '"') {
-            *(q++) = '\\';
-            *(q++) = c;
-        }
-        else if (c < ' ' || c == 0x7f || c == 0xc0 || c == 0xc1 || c >= 0xf5) {
-            *(q++) = '\\';
-            *(q++) = '0' + ((c >> 6) & 0x07);
-            *(q++) = '0' + ((c >> 3) & 0x07);
-            *(q++) = '0' + (c & 0x07);
-        }
-        else
-            *(q++) = c;
-        p++;
-    }
-    *q = 0;
-
-    return dest;
-}
-
-/**
- * gwy_assign_string:
- * @target: Pointer to target string, typically a struct field.
- * @newvalue: New value of the string, may be %NULL.
- *
- * Assigns a string, checking for equality and handling %NULL<!-- -->s.
- *
- * This function simplifies handling of string value setters.
- *
- * The new value is duplicated and the old string is freed in a safe manner (it is possible to pass a pointer
- * somewhere within the old value as the new value, for instance).  Any of the old and new value can be %NULL.  If
- * both values are equal (including both unset), the function returns %FALSE.
- *
- * Returns: %TRUE if the target string has changed.
- *
- * Since: 2.59
- **/
-gboolean
-gwy_assign_string(gchar **target,
-                  const gchar *newvalue)
-{
-    if (*target && newvalue) {
-        if (!gwy_strequal(*target, newvalue)) {
-            gchar *old = *target;
-            *target = g_strdup(newvalue);
-            g_free(old);
-            return TRUE;
-        }
-    }
-    else if (*target) {
-        g_free(*target);
-        *target = NULL;
-        return TRUE;
-    }
-    else if (newvalue) {
-        *target = g_strdup(newvalue);
-        return TRUE;
-    }
-    return FALSE;
-}
-
 /**
  * gwy_object_set_or_reset:
  * @object: A #GObject.
@@ -2016,6 +1238,194 @@ gwy_append_doubles_to_gstring(GString *str,
         g_string_truncate(str, str->len - strlen(field_separator));
 }
 
+/**
+ * gwy_parse_doubles:
+ * @s: String with floating point data to parse.
+ * @values: Pre-allocated array where to store values (for a fixed number of values).
+ * @flags: Parsing flags.
+ * @nlines: Location where to store the number of lines. Initialise to the number of lines for a fixed number of
+ *          lines or to -1 for unknown. The value is only changed if initially -1.
+ * @ncols: Location where to store the number of columns. Initialise to the number of columns for a fixed number of
+ *         columns or to -1 for unknown. The value is only changed if initially -1.
+ * @endptr: Location where to store pointer after the last parsed character in @s. Possibly %NULL.
+ * @error: Return location for a #GError.
+ *
+ * Parse a block of text floating point values.
+ *
+ * Any combinations of a priori known and unknown dimensions is possible. For an unknown number of columns the number
+ * of columns must still be the same on each line, unless the flag %GWY_PARSE_DOUBLES_FREE_FORM is passed. If any
+ * dimension is unknown, @values cannot be preallocated as must be %NULL.
+ *
+ * Currently the values must be whitespace separated and in the POSIX format.
+ *
+ * Returns: On success, either @values (if non-%NULL @values was passed) or a newly allocated array with the data.
+ *          On failure, %NULL is returned.
+ *
+ * Since: 2.64
+ **/
+gdouble*
+gwy_parse_doubles(const gchar *s,
+                  gdouble *values,
+                  GwyParseDoublesFlags flags,
+                  gint *nlines,
+                  gint *ncols,
+                  gchar **endptr,
+                  GError **error)
+{
+    GArray *storage = NULL;
+    gint nreadcols, nreadlines, nreadn = -1, line, col, n;
+    gboolean free_values_on_fail = FALSE, is_newline, is_free_form = (flags & GWY_PARSE_DOUBLES_FREE_FORM);
+    gchar *end;
+    gdouble v;
+
+    g_return_val_if_fail(s, NULL);
+    g_return_val_if_fail(nlines, NULL);
+    g_return_val_if_fail(ncols, NULL);
+
+    if (values) {
+        /* Reading an unknown number of values into fixed array = buffer overflow. Refuse. */
+        g_return_val_if_fail(*nlines >= 0, NULL);
+        g_return_val_if_fail(*ncols >= 0, NULL);
+    }
+    if (!*nlines || !*ncols)
+        return NULL;
+
+    nreadlines = *nlines;
+    nreadcols = *ncols;
+    if (nreadlines > 0 && nreadcols > 0)
+        nreadn = nreadlines*nreadcols;
+    if (!values) {
+        free_values_on_fail = TRUE;
+        if (nreadn > 0)
+            values = g_new(gdouble, nreadn);
+        else
+            storage = g_array_new(FALSE, FALSE, sizeof(gdouble));
+    }
+
+    line = col = n = 0;
+    while (line < nreadlines || nreadlines < 0 || is_free_form) {
+        v = g_ascii_strtod(s, &end);
+        if (end == s) {
+            /* We finished parsing the string or garbage at the end is allowed. */
+            if (!*end || g_ascii_isspace(*end) || !(flags & GWY_PARSE_DOUBLES_COMPLETELY)) {
+                if (nreadlines < 0) {
+                    /* One long line. */
+                    if (nreadcols < 0) {
+                        *ncols = n;
+                        *nlines = 1;
+                        goto finished;
+                    }
+                    /* A full rectangular block, still possibly empty. */
+                    if (col == 0) {
+                        *ncols = (nreadcols > 0 ? nreadcols : 0);
+                        *nlines = line;
+                        goto finished;
+                    }
+                    if (is_free_form) {
+                        *ncols = 1;
+                        *nlines = n;
+                        goto finished;
+                    }
+                }
+                /* We are also done here if we are asked to read one line with some values. */
+                if (nreadlines == 1 && nreadcols < 0) {
+                    *ncols = n;
+                    *nlines = 1;
+                    goto finished;
+                }
+                g_set_error(error, GWY_PARSE_DOUBLES_ERROR, GWY_PARSE_DOUBLES_TRUNCATED,
+                            _("Data ended prematurely at line %d and column %d."), line+1, col+1);
+                goto fail;
+            }
+            else {
+                g_set_error(error, GWY_PARSE_DOUBLES_ERROR, GWY_PARSE_DOUBLES_MALFORMED,
+                            _("Malformed data encountered at line %d and column %d."), line+1, col+1);
+                goto fail;
+            }
+        }
+        col++;
+        is_newline = FALSE;
+        /* This automatically skips any number of empty lines. */
+        for (s = end; g_ascii_isspace(*s); s++) {
+            if (*s == '\n' || *s == '\r')
+                is_newline = TRUE;
+        }
+
+        if (is_newline) {
+            line++;
+            if (nreadcols < 0)
+                nreadcols = col;
+            else if (!is_free_form && nreadcols != col) {
+                g_set_error(error, GWY_PARSE_DOUBLES_ERROR, GWY_PARSE_DOUBLES_NONUNIFORM,
+                            _("Line %d has %d columns instead of expected %d."), line, col, nreadcols);
+                goto fail;
+            }
+            col = 0;
+        }
+
+        if (storage)
+            g_array_append_val(storage, v);
+        else
+            values[n] = v;
+
+        /* With free form we do not really care about line breaks. And we definitely do not want to update @nlines
+         * and @ncols to whatever random stuff we found. */
+        if (++n == nreadn && is_free_form)
+            goto finished;
+    }
+
+    /* We can only get here after parsing an explicitly given number of lines. So we must have successfully read at
+     * least one full line. */
+    g_assert(line == *nlines);
+    g_assert(nreadcols > 0);
+    if (*ncols > 0) {
+        g_assert(nreadcols == *ncols);
+    }
+    else
+        *ncols = nreadcols;
+
+finished:
+    if (endptr)
+        *endptr = end;
+    if (storage)
+        values = (gdouble*)g_array_free(storage, FALSE);
+    if (n || (flags & GWY_PARSE_DOUBLES_EMPTY_OK))
+        return values;
+
+    g_set_error(error, GWY_PARSE_DOUBLES_ERROR, GWY_PARSE_DOUBLES_EMPTY, _("Empty data."));
+
+fail:
+    if (endptr)
+        *endptr = end;
+    if (free_values_on_fail)
+        g_free(values);
+    return NULL;
+}
+
+static gpointer
+ensure_error_domain(G_GNUC_UNUSED gpointer arg)
+{
+    return GUINT_TO_POINTER(g_quark_from_static_string("gwy-parse-doubles-error-quark"));
+}
+
+/**
+ * gwy_parse_doubles_error_quark:
+ *
+ * Returns error domain for floating point value parsing and evaluation.
+ *
+ * See and use %GWY_PARSE_DOUBLES_ERROR.
+ *
+ * Returns: The error domain.
+ *
+ * Since: 2.64
+ **/
+GQuark
+gwy_parse_doubles_error_quark(void)
+{
+    static GOnce error_once = G_ONCE_INIT;
+    return GPOINTER_TO_UINT(g_once(&error_once, ensure_error_domain, NULL));
+}
+
 /**
  * gwy_fopen:
  * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
@@ -2219,14 +1629,6 @@ gwy_fprintf(FILE        *file,
  * It expands to property spec cast to correct type (@spec).
  **/
 
-/**
- * gwy_strequal:
- * @a: A string.
- * @b: Another string.
- *
- * Expands to %TRUE if strings are equal, to %FALSE otherwise.
- **/
-
 /**
  * GWY_CLAMP:
  * @x: The value to clamp.
@@ -2390,6 +1792,40 @@ gwy_fprintf(FILE        *file,
  * Since: 2.25
  **/
 
+/**
+ * GWY_PARSE_DOUBLE_ERROR:
+ *
+ * Error domain for floating point data parsing and evaluation. Errors in this domain will be from the
+ * #GwyParseDoublesError enumeration. See #GError for information on error domains.
+ *
+ * Since: 2.64
+ **/
+
+/**
+ * GwyParseDoublesFlags:
+ * @GWY_PARSE_DOUBLES_EMPTY_OK: Do not set error (and return %NULL) when there are no data.
+ * @GWY_PARSE_DOUBLES_COMPLETELY: For unknown number of values, consider an error when they are followed by
+ *                                non-numerical garbage. This flag has no effect for a known number of values.
+ * @GWY_PARSE_DOUBLES_FREE_FORM: Do not distinguish line breaks from other whitespace; just read the requested number
+ *                               of values. For an unknown number of values they will be reported as single-column.
+ *
+ * Type of flags passed to gwy_parse_doubles().
+ *
+ * Since: 2.64
+ **/
+
+/**
+ * GwyParseDoublesError:
+ * @GWY_PARSE_DOUBLES_EMPTY: There are no data at all.
+ * @GWY_PARSE_DOUBLES_TRUNCATED: There are fewer values than expected.
+ * @GWY_PARSE_DOUBLES_MALFORMED: Non-numerical garbage encounted before reading the expected number of values.
+ * @GWY_PARSE_DOUBLES_NONUNIFORM: Lines do not have the expected constant number of columns.
+ *
+ * Type of error which can occur in gwy_parse_doubles().
+ *
+ * Since: 2.64
+ **/
+
 /**
  * GwySetFractionFunc:
  * @fraction: Progress estimate as a number from the interval [0,1].
