@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyutils.h 25242 2023-02-23 16:13:10Z yeti-dn $
+ *  $Id: gwyutils.h 25679 2023-09-18 16:35:11Z yeti-dn $
  *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -45,9 +45,25 @@ typedef enum {
     GWY_BYTE_ORDER_IMPLICIT      = 9999,
 } GwyByteOrder;
 
+typedef enum {
+    GWY_PARSE_DOUBLES_EMPTY_OK   = 1 << 0,
+    GWY_PARSE_DOUBLES_COMPLETELY = 1 << 1,
+    GWY_PARSE_DOUBLES_FREE_FORM  = 1 << 2,
+} GwyParseDoublesFlags;
+
+typedef enum {
+    GWY_PARSE_DOUBLES_EMPTY,
+    GWY_PARSE_DOUBLES_TRUNCATED,
+    GWY_PARSE_DOUBLES_MALFORMED,
+    GWY_PARSE_DOUBLES_NONUNIFORM,
+} GwyParseDoublesError;
+
 typedef gboolean (*GwySetFractionFunc)(gdouble fraction);
 typedef gboolean (*GwySetMessageFunc)(const gchar *message);
 
+#define GWY_PARSE_DOUBLES_ERROR gwy_parse_doubles_error_quark()
+
+GQuark       gwy_parse_doubles_error_quark(void);
 void         gwy_hash_table_to_slist_cb   (gpointer unused_key,
                                            gpointer value,
                                            gpointer user_data);
@@ -101,6 +117,13 @@ void         gwy_append_doubles_to_gstring(GString *str,
                                            gint precision,
                                            const gchar *field_separator,
                                            gboolean force_decimal_dot);
+gdouble*     gwy_parse_doubles            (const gchar *s,
+                                           gdouble *values,
+                                           GwyParseDoublesFlags flags,
+                                           gint *nlines,
+                                           gint *ncols,
+                                           gchar **endptr,
+                                           GError **error);
 void         gwy_memcpy_byte_swap         (const guint8 *source,
                                            guint8 *dest,
                                            gsize item_size,
@@ -115,6 +138,9 @@ void         gwy_convert_raw_data         (gconstpointer data,
                                            gdouble scale,
                                            gdouble offset);
 guint        gwy_raw_data_size            (GwyRawDataType datatype);
+gchar*       gwy_convert_to_utf8          (const guchar *str,
+                                           glong len,
+                                           const gchar *codeset)         G_GNUC_MALLOC;
 gchar*       gwy_utf16_to_utf8            (const gunichar2 *str,
                                            glong len,
                                            GwyByteOrder byteorder)       G_GNUC_MALLOC;
