@@ -1,5 +1,5 @@
 /*
- *  $Id: gwy3dwindow.c 24135 2021-09-15 12:12:53Z yeti-dn $
+ *  $Id: gwy3dwindow.c 24710 2022-03-21 17:35:20Z yeti-dn $
  *  Copyright (C) 2004-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -133,8 +133,6 @@ static void          gwy_3d_window_labels_entry_activate(GtkEntry *entry,
                                                          Gwy3DWindow *window);
 static void          gwy_3d_window_labels_reset_clicked (Gwy3DWindow *window);
 static void          gwy_3d_window_reset_visualisation  (Gwy3DWindow *window);
-static void          gwy_3d_window_set_tooltip          (GtkWidget *widget,
-                                                         const gchar *tip_text);
 static gboolean      gwy_3d_window_view_clicked         (GtkWidget *gwy3dwindow,
                                                          GdkEventButton *event,
                                                          GtkWidget *gwy3dview);
@@ -605,7 +603,7 @@ gwy_3d_window_pack_buttons(Gwy3DWindow *gwy3dwindow,
                                  G_CALLBACK(gwy_3d_window_set_mode),
                                  GINT_TO_POINTER(buttons[i].mode));
         g_object_set_data(G_OBJECT(button), "gwy3dwindow", gwy3dwindow);
-        gwy_3d_window_set_tooltip(button, _(buttons[i].tooltip));
+        gtk_widget_set_tooltip_text(button, _(buttons[i].tooltip));
         gwy3dwindow->buttons[offset + buttons[i].mode] = button;
         if (!group)
             group = GTK_RADIO_BUTTON(button);
@@ -663,7 +661,7 @@ gwy_3d_window_new(Gwy3DView *gwy3dview)
     gtk_container_add(GTK_CONTAINER(button),
                       gtk_image_new_from_stock(GWY_STOCK_MORE,
                                                GTK_ICON_SIZE_LARGE_TOOLBAR));
-    gwy_3d_window_set_tooltip(button, _("Show full controls"));
+    gtk_widget_set_tooltip_text(button, _("Show full controls"));
     g_object_set_data(G_OBJECT(button), "gwy3dwindow", gwy3dwindow);
     g_signal_connect_swapped(button, "clicked",
                              G_CALLBACK(gwy_3d_window_select_controls),
@@ -688,7 +686,7 @@ gwy_3d_window_new(Gwy3DView *gwy3dview)
     gtk_container_add(GTK_CONTAINER(button),
                       gtk_image_new_from_stock(GWY_STOCK_LESS,
                                                GTK_ICON_SIZE_LARGE_TOOLBAR));
-    gwy_3d_window_set_tooltip(button, _("Hide full controls"));
+    gtk_widget_set_tooltip_text(button, _("Hide full controls"));
     g_object_set_data(G_OBJECT(button), "gwy3dwindow", gwy3dwindow);
     g_signal_connect_swapped(button, "clicked",
                              G_CALLBACK(gwy_3d_window_select_controls),
@@ -1386,7 +1384,7 @@ gwy_3d_window_add_small_toolbar_button(Gwy3DWindow *gwy3dwindow,
     gtk_container_add(GTK_CONTAINER(button),
                       gtk_image_new_from_stock(stock_id,
                                                GTK_ICON_SIZE_LARGE_TOOLBAR));
-    gwy_3d_window_set_tooltip(button, tooltip);
+    gtk_widget_set_tooltip_text(button, tooltip);
     g_object_set_data(G_OBJECT(button), "gwy3dwindow", gwy3dwindow);
     g_signal_connect_swapped(button, "clicked", G_CALLBACK(callback), cbdata);
 }
@@ -1398,6 +1396,8 @@ gwy_3d_window_add_small_toolbar_button(Gwy3DWindow *gwy3dwindow,
  *
  * Sets the tooltips object to use for adding tooltips to 3D window parts.
  *
+ * This function does not do anything useful.  Do not use it.
+ *
  * This is a class method.  It affects only newly cerated 3D windows, existing
  * 3D windows will continue to use the tooltips they were constructed with.
  *
@@ -1425,6 +1425,8 @@ gwy_3d_window_class_set_tooltips(GtkTooltips *tips)
  *
  * Gets the tooltips object used for adding tooltips to 3D window parts.
  *
+ * This function does not do anything useful.  Do not use it.
+ *
  * Returns: The #GtkTooltips object.
  **/
 GtkTooltips*
@@ -1818,14 +1820,6 @@ gwy_3d_window_reset_visualisation(Gwy3DWindow *window)
     gwy_container_remove_by_name(data, key);
 }
 
-static void
-gwy_3d_window_set_tooltip(GtkWidget *widget,
-                          const gchar *tip_text)
-{
-    if (tooltips)
-        gtk_tooltips_set_tip(tooltips, widget, tip_text, NULL);
-}
-
 static gboolean
 gwy_3d_window_view_clicked(GtkWidget *gwy3dwindow,
                            GdkEventButton *event,
