@@ -1,5 +1,5 @@
 /*
- *  $Id: gwy3dwindow.h 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: gwy3dwindow.h 24710 2022-03-21 17:35:20Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -104,8 +104,10 @@ void         gwy_3d_window_add_small_toolbar_button(Gwy3DWindow *gwy3dwindow,
                                                     gpointer cbdata);
 void         gwy_3d_window_set_overlay_chooser     (Gwy3DWindow *gwy3dwindow,
                                                     GtkWidget *chooser);
+#ifndef GWY_DISABLE_DEPRECATED
 void         gwy_3d_window_class_set_tooltips      (GtkTooltips *tips);
 GtkTooltips* gwy_3d_window_class_get_tooltips      (void);
+#endif
 
 G_END_DECLS
 
