@@ -1,22 +1,18 @@
 /*
- *  $Id: gwyadjustbar.c 23814 2021-06-08 13:17:39Z yeti-dn $
+ *  $Id: gwyadjustbar.c 24993 2022-09-02 14:39:52Z yeti-dn $
  *  Copyright (C) 2012-2017 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -182,36 +178,29 @@ gwy_adjust_bar_class_init(GwyAdjustBarClass *klass)
 
     klass->change_value = gwy_adjust_bar_change_value;
 
-    properties[PROP_ADJUSTMENT]
-        = g_param_spec_object("adjustment",
-                              "Adjustment",
-                              "Adjustment representing the value.",
-                              GTK_TYPE_ADJUSTMENT,
-                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-
-    properties[PROP_SNAP_TO_TICKS]
-        = g_param_spec_boolean("snap-to-ticks",
-                               "Snap to ticks",
-                               "Whether only values that are multiples of step "
-                               "size are allowed.",
-                               FALSE,
-                               G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-
-    properties[PROP_MAPPING]
-        = g_param_spec_enum("mapping",
-                            "Mapping",
-                            "Mapping function between values and screen "
-                            "positions.",
-                            GWY_TYPE_SCALE_MAPPING_TYPE,
-                            GWY_SCALE_MAPPING_SQRT,
-                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-
-    properties[PROP_HAS_CHECK_BUTTON]
-        = g_param_spec_boolean("has-check-button",
-                               "Has check button",
-                               "Whether the adjust bar has a check button.",
-                               FALSE,
-                               G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+    properties[PROP_ADJUSTMENT] = g_param_spec_object("adjustment",
+                                                      "Adjustment",
+                                                      "Adjustment representing the value.",
+                                                      GTK_TYPE_ADJUSTMENT,
+                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+    properties[PROP_SNAP_TO_TICKS] = g_param_spec_boolean("snap-to-ticks",
+                                                          "Snap to ticks",
+                                                          "Whether only values that are multiples of step "
+                                                          "size are allowed.",
+                                                          FALSE,
+                                                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+    properties[PROP_MAPPING] = g_param_spec_enum("mapping",
+                                                 "Mapping",
+                                                 "Mapping function between values and screen "
+                                                 "positions.",
+                                                 GWY_TYPE_SCALE_MAPPING_TYPE,
+                                                 GWY_SCALE_MAPPING_SQRT,
+                                                 G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+    properties[PROP_HAS_CHECK_BUTTON] = g_param_spec_boolean("has-check-button",
+                                                             "Has check button",
+                                                             "Whether the adjust bar has a check button.",
+                                                             FALSE,
+                                                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
     for (i = 1; i < N_PROPS; i++)
         g_object_class_install_property(gobject_class, i, properties[i]);
@@ -221,19 +210,17 @@ gwy_adjust_bar_class_init(GwyAdjustBarClass *klass)
      * @gwyadjustbar: The #GwyAdjustBar which received the signal.
      * @arg1: New value for @gwyadjustbar.
      *
-     * The ::change-value signal is emitted when the user interactively
-     * changes the value.
+     * The ::change-value signal is emitted when the user interactively changes the value.
      *
      * It is an action signal.
      **/
-    signals[SGNL_CHANGE_VALUE]
-        = g_signal_new("change-value",
-                       G_OBJECT_CLASS_TYPE(klass),
-                       G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
-                       G_STRUCT_OFFSET(GwyAdjustBarClass, change_value),
-                       NULL, NULL,
-                       g_cclosure_marshal_VOID__DOUBLE,
-                       G_TYPE_NONE, 1, G_TYPE_DOUBLE);
+    signals[SGNL_CHANGE_VALUE] = g_signal_new("change-value",
+                                              G_OBJECT_CLASS_TYPE(klass),
+                                              G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+                                              G_STRUCT_OFFSET(GwyAdjustBarClass, change_value),
+                                              NULL, NULL,
+                                              g_cclosure_marshal_VOID__DOUBLE,
+                                              G_TYPE_NONE, 1, G_TYPE_DOUBLE);
 }
 
 static void
@@ -242,9 +229,7 @@ gwy_adjust_bar_init(GwyAdjustBar *adjbar)
     GtkWidget *label;
     AdjustBar *priv;
 
-    priv = adjbar->priv = G_TYPE_INSTANCE_GET_PRIVATE(adjbar,
-                                                      GWY_TYPE_ADJUST_BAR,
-                                                      AdjustBar);
+    priv = adjbar->priv = G_TYPE_INSTANCE_GET_PRIVATE(adjbar, GWY_TYPE_ADJUST_BAR, AdjustBar);
     priv->mapping = GWY_SCALE_MAPPING_SQRT;
     priv->bar_sensitive = TRUE;
     GTK_WIDGET_SET_FLAGS(adjbar, GTK_NO_WINDOW);
@@ -340,9 +325,7 @@ gwy_adjust_bar_forall(GtkContainer *container, gboolean include_internals,
             (*callback)(priv->check, cbdata);
 
     }
-    GTK_CONTAINER_CLASS(gwy_adjust_bar_parent_class)->forall(container,
-                                                             include_internals,
-                                                             callback, cbdata);
+    GTK_CONTAINER_CLASS(gwy_adjust_bar_parent_class)->forall(container, include_internals, callback, cbdata);
 }
 
 static void
@@ -376,10 +359,7 @@ gwy_adjust_bar_map(GtkWidget *widget)
     GTK_WIDGET_CLASS(gwy_adjust_bar_parent_class)->map(widget);
     if (priv->input_window)
         gdk_window_show(priv->input_window);
-    if (check
-        && GTK_WIDGET_VISIBLE(check)
-        && gtk_widget_get_child_visible(check)
-        && !GTK_WIDGET_MAPPED(check))
+    if (check && GTK_WIDGET_VISIBLE(check) && gtk_widget_get_child_visible(check) && !GTK_WIDGET_MAPPED(check))
         gtk_widget_map(check);
 }
 
@@ -424,9 +404,8 @@ gwy_adjust_bar_size_request(GtkWidget *widget, GtkRequisition *requisition)
         requisition->width += child_req.width;
         requisition->height = MAX(requisition->height, child_req.height);
 
-        /* Reproduce checkbutton's indicator spacing internally.  It puts two
-         * spacings between the check and label.  So use that instead of
-         * our default border. */
+        /* Reproduce checkbutton's indicator spacing internally.  It puts two spacings between the check and label.
+         * So use that instead of our default border. */
         gtk_widget_style_get(priv->check, "indicator-spacing", &spacing, NULL);
         requisition->width += 2*spacing;
         requisition->width -= default_border.left;
@@ -462,32 +441,20 @@ gwy_adjust_bar_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
         gtk_widget_size_allocate(priv->check, &child_allocation);
     }
 
-    gwy_debug("ALLOCATION %dx%d at (%d,%d)",
-              allocation->width, allocation->height,
-              allocation->x, allocation->y);
+    gwy_debug("ALLOCATION %dx%d at (%d,%d)", allocation->width, allocation->height, allocation->x, allocation->y);
     if (priv->input_window) {
-        gwy_debug("INPUT WINDOW %dx%d at (%d,%d)",
-                  width, allocation->height,
-                  allocation->x + priv->x, allocation->y);
-        gdk_window_move_resize(priv->input_window,
-                               allocation->x + priv->x, allocation->y,
-                               width, allocation->height);
+        gwy_debug("INPUT WINDOW %dx%d at (%d,%d)", width, allocation->height, allocation->x + priv->x, allocation->y);
+        gdk_window_move_resize(priv->input_window, allocation->x + priv->x, allocation->y, width, allocation->height);
     }
 
     label = GTK_BIN(widget)->child;
     if (label && GTK_WIDGET_VISIBLE(label)) {
         gtk_widget_get_child_requisition(label, &child_req);
-        h = child_req.height + (2*border_width
-                                + default_border.top + default_border.bottom);
-        child_allocation.x = (allocation->x + priv->x
-                              + border_width + default_border.left);
-        child_allocation.y = (allocation->y
-                              + MAX(allocation->height - h, 0)/2
-                              + border_width + default_border.top);
-        child_allocation.width = (width - 2*border_width
-                                  - default_border.left - default_border.right);
-        child_allocation.height = (allocation->height - 2*border_width
-                                   - default_border.top - default_border.bottom);
+        h = child_req.height + (2*border_width + default_border.top + default_border.bottom);
+        child_allocation.x = (allocation->x + priv->x + border_width + default_border.left);
+        child_allocation.y = (allocation->y + MAX(allocation->height - h, 0)/2 + border_width + default_border.top);
+        child_allocation.width = (width - 2*border_width - default_border.left - default_border.right);
+        child_allocation.height = (allocation->height - 2*border_width - default_border.top - default_border.bottom);
         gtk_widget_size_allocate(label, &child_allocation);
     }
 
@@ -511,9 +478,7 @@ gwy_adjust_bar_expose(GtkWidget *widget, GdkEventExpose *event)
             cairo_fill(cr);
         }
 #endif
-        cairo_translate(cr,
-                        widget->allocation.x + adjbar->priv->x,
-                        widget->allocation.y);
+        cairo_translate(cr, widget->allocation.x + adjbar->priv->x, widget->allocation.y);
         draw_bar(adjbar, cr);
         cairo_destroy(cr);
     }
@@ -669,9 +634,8 @@ gwy_adjust_bar_change_value(GwyAdjustBar *adjbar, gdouble newvalue)
  *
  * Creates a new adjustment bar.
  *
- * The label text, if any, is set with mnemonic enabled.  However, you still
- * need to assign it to a widget (presumably a #GtkSpinButton) using
- * gtk_label_set_mnemonic_widget().
+ * The label text, if any, is set with mnemonic enabled.  However, you still need to assign it to a widget (presumably
+ * a #GtkSpinButton) using gtk_label_set_mnemonic_widget().
  *
  * Returns: A new adjustment bar.
  *
@@ -682,11 +646,8 @@ gwy_adjust_bar_new(GtkAdjustment *adjustment, const gchar *label)
 {
     GwyAdjustBar *adjbar;
 
-    if (adjustment) {
-        adjbar = g_object_new(GWY_TYPE_ADJUST_BAR,
-                              "adjustment", adjustment,
-                              NULL);
-    }
+    if (adjustment)
+        adjbar = g_object_new(GWY_TYPE_ADJUST_BAR, "adjustment", adjustment, NULL);
     else
         adjbar = g_object_new(GWY_TYPE_ADJUST_BAR, NULL);
 
@@ -725,8 +686,8 @@ gwy_adjust_bar_set_adjustment(GwyAdjustBar *adjbar,
  *
  * Obtains the adjustment that an adjustment bar visualises.
  *
- * Returns: The adjustment used by @adjbar.  If no adjustment was set
- *          and the default one is used, function returns %NULL.
+ * Returns: The adjustment used by @adjbar.  If no adjustment was set and the default one is used, function returns
+ *          %NULL.
  *
  * Since: 2.49
  **/
@@ -740,19 +701,16 @@ gwy_adjust_bar_get_adjustment(GwyAdjustBar *adjbar)
 /**
  * gwy_adjust_bar_set_snap_to_ticks:
  * @adjbar: An adjustment bar.
- * @setting: %TRUE to restrict values to multiples of step size,
- *           %FALSE to permit any values.
+ * @setting: %TRUE to restrict values to multiples of step size, %FALSE to permit any values.
  *
  * Sets the snapping behaviour of an adjustment bar.
  *
- * Note the ‘multiples of step size’ condition in fact applies to the
- * difference from the minimum value.  The maximum adjustment value is always
- * permissible, even if it does not satisfy this condition.  Values modified by
- * the user (i.e.  emission of signal "change-value") are snapped, however,
- * values set explicitly gtk_adjustment_set_value() are kept intact.
+ * Note the ‘multiples of step size’ condition in fact applies to the difference from the minimum value.  The maximum
+ * adjustment value is always permissible, even if it does not satisfy this condition.  Values modified by the user
+ * (i.e.  emission of signal "change-value") are snapped, however, values set explicitly gtk_adjustment_set_value()
+ * are kept intact.
  *
- * Setting this option to %TRUE immediately causes an adjustment value change
- * if it does not satisfy the condition.
+ * Setting this option to %TRUE immediately causes an adjustment value change if it does not satisfy the condition.
  *
  * It is usually a poor idea to enable snapping for non-linear mappings.
  *
@@ -789,8 +747,7 @@ gwy_adjust_bar_get_snap_to_ticks(GwyAdjustBar *adjbar)
 /**
  * gwy_adjust_bar_set_mapping:
  * @adjbar: An adjustment bar.
- * @mapping: Mapping function type between values and screen positions in the
- *           adjustment bar.
+ * @mapping: Mapping function type between values and screen positions in the adjustment bar.
  *
  * Sets the mapping function type for an adjustment bar.
  *
@@ -884,8 +841,7 @@ gwy_adjust_bar_get_has_check_button(GwyAdjustBar *adjbar)
  *
  * Gets the label widget inside an adjustment bar.
  *
- * Use gtk_label_set_mnemonic_widget() to set the mnemonic widget for the label
- * or change the label text.
+ * Use gtk_label_set_mnemonic_widget() to set the mnemonic widget for the label or change the label text.
  *
  * Returns: The label widget.
  *
@@ -904,8 +860,7 @@ gwy_adjust_bar_get_label(GwyAdjustBar *adjbar)
  *
  * Gets the check button of an adjustment bar.
  *
- * Connect to the "toggled" signal of the check button.  Modifying it is not
- * recommended.
+ * Connect to the "toggled" signal of the check button.  Modifying it is not recommended.
  *
  * Returns: The check button widget, or %NULL if there is none.
  *
@@ -925,12 +880,10 @@ gwy_adjust_bar_get_check_button(GwyAdjustBar *adjbar)
  *
  * Sets the sensitivity of an adjustment bar.
  *
- * The bar's sensitivity can be controlled separately.  This is useful when
- * @adjbar has a check button because otherwise the bar is the entire widget
- * and the function is not different from gtk_widget_set_sensitive().
- * However, if you want to enable and disable the adjustment bar via the check
- * button, use this function instead of gtk_widget_set_sensitive() which would
- * make insensitive also the check button.
+ * The bar's sensitivity can be controlled separately.  This is useful when @adjbar has a check button because
+ * otherwise the bar is the entire widget and the function is not different from gtk_widget_set_sensitive(). However,
+ * if you want to enable and disable the adjustment bar via the check button, use this function instead of
+ * gtk_widget_set_sensitive() which would make insensitive also the check button.
  *
  * Since: 2.49
  **/
@@ -988,14 +941,11 @@ set_adjustment(GwyAdjustBar *adjbar,
                GtkAdjustment *adjustment)
 {
     AdjustBar *priv = adjbar->priv;
-    if (!gwy_set_member_object(adjbar, adjustment, GTK_TYPE_ADJUSTMENT,
-                               &priv->adjustment,
+    if (!gwy_set_member_object(adjbar, adjustment, GTK_TYPE_ADJUSTMENT, &priv->adjustment,
                                "changed", &adjustment_changed,
-                               &priv->adjustment_changed_id,
-                               G_CONNECT_SWAPPED,
+                               &priv->adjustment_changed_id, G_CONNECT_SWAPPED,
                                "value-changed", &adjustment_value_changed,
-                               &priv->adjustment_value_changed_id,
-                               G_CONNECT_SWAPPED,
+                               &priv->adjustment_value_changed_id, G_CONNECT_SWAPPED,
                                NULL))
         return FALSE;
 
@@ -1032,7 +982,7 @@ set_mapping(GwyAdjustBar *adjbar,
     if (mapping == priv->mapping)
         return FALSE;
 
-    if (mapping > GWY_SCALE_MAPPING_LOG) {
+    if (mapping > GWY_SCALE_MAPPING_LOG1P) {
         g_warning("Wrong scale mapping %u.", mapping);
         return FALSE;
     }
@@ -1071,10 +1021,8 @@ create_input_window(GwyAdjustBar *adjbar)
                              | GDK_POINTER_MOTION_MASK
                              | GDK_POINTER_MOTION_HINT_MASK);
     attributes_mask = GDK_WA_X | GDK_WA_Y;
-    gwy_debug("CREATE INPUT WINDOW %dx%d at (%d,%d)",
-              attributes.width, attributes.height, attributes.x, attributes.y);
-    priv->input_window = gdk_window_new(gtk_widget_get_parent_window(widget),
-                                        &attributes, attributes_mask);
+    gwy_debug("CREATE INPUT WINDOW %dx%d at (%d,%d)", attributes.width, attributes.height, attributes.x, attributes.y);
+    priv->input_window = gdk_window_new(gtk_widget_get_parent_window(widget), &attributes, attributes_mask);
     gdk_window_set_user_data(priv->input_window, widget);
 }
 
@@ -1179,6 +1127,22 @@ ssqr(gdouble x)
     return x*fabs(x);
 }
 
+#ifndef HAVE_LOG1P
+static gdouble
+log1p(gdouble x)
+{
+    return log(1.0 + x);
+}
+#endif
+
+#ifndef HAVE_EXPM1
+static gdouble
+expm1(gdouble x)
+{
+    return exp(x) - 1.0;
+}
+#endif
+
 static void
 update_mapping(GwyAdjustBar *adjbar)
 {
@@ -1200,6 +1164,10 @@ update_mapping(GwyAdjustBar *adjbar)
         if (lower <= 0.0 || upper <= 0.0)
             return;
     }
+    if (priv->mapping == GWY_SCALE_MAPPING_LOG1P) {
+        if (lower < 0.0 || upper < 0.0)
+            return;
+    }
 
     priv->length = GTK_WIDGET(adjbar)->allocation.width - priv->x;
     if (priv->length < 2)
@@ -1215,6 +1183,10 @@ update_mapping(GwyAdjustBar *adjbar)
         priv->map_value = log;
         priv->map_position = exp;
     }
+    else if (priv->mapping == GWY_SCALE_MAPPING_LOG1P) {
+        priv->map_value = log1p;
+        priv->map_position = expm1;
+    }
     priv->b = priv->map_value(lower);
     priv->a = (priv->map_value(upper) - priv->b)/priv->length;
     if (gwy_isinf(priv->a) || gwy_isnan(priv->a) || !priv->a
@@ -1322,17 +1294,14 @@ snap_value(GwyAdjustBar *adjbar, gdouble value)
  * @title: GwyAdjustBar
  * @short_description: Compact adjustment visualisation and modification
  *
- * #GwyAdjustBar is a compact widget for visualisation and modification of the
- * value of an #GtkAdjustment.  It can contains a label with an overlaid bar
- * that can be clicked, dragged or modified by the scroll-wheel by the user.
- * Since the widget does not take keyboard focus, it should be paired with a
- * #GtkSpinButton, sharing the same adjustment.  This spin button would also be
- * the typical mnemonic widget for the adjustment bar.
+ * #GwyAdjustBar is a compact widget for visualisation and modification of the value of an #GtkAdjustment.  It can
+ * contains a label with an overlaid bar that can be clicked, dragged or modified by the scroll-wheel by the user.
+ * Since the widget does not take keyboard focus, it should be paired with a #GtkSpinButton, sharing the same
+ * adjustment.  This spin button would also be the typical mnemonic widget for the adjustment bar.
  *
- * #GwyAdjustBar supports several different types of mapping between screen
- * positions and values of the underlying adjustment.  Nevertheless, the
- * default mapping (signed square root, %GWY_SCALE_MAPPING_SQRT) should fit
- * most situations.
+ * #GwyAdjustBar supports several different types of mapping between screen positions and values of the underlying
+ * adjustment.  Nevertheless, the default mapping (signed square root, %GWY_SCALE_MAPPING_SQRT) should fit most
+ * situations.
  **/
 
 /**
@@ -1356,23 +1325,21 @@ snap_value(GwyAdjustBar *adjbar, gdouble value)
 
 /**
  * GwyScaleMappingType:
- * @GWY_SCALE_MAPPING_LINEAR: Linear mapping between values and screen
- *                            positions.  This recommended for signed additive
- *                            quantities of a limited range.
- * @GWY_SCALE_MAPPING_SQRT: Screen positions correspond to ‘signed square
- *                          roots’ of the value.  This is the
- *                          recommended general-purpose default mapping type as
- *                          it works with both signed and usigned quantities
- *                          and offers good sensitivity for both large and
- *                          small values.
- * @GWY_SCALE_MAPPING_LOG: Screen positions correspond to logarithms of values.
- *                         The adjustment range must contain only positive
- *                         values.  For quantities of extreme ranges this
- *                         mapping may be preferred to %GWY_SCALE_MAPPING_SQRT.
+ * @GWY_SCALE_MAPPING_LINEAR: Linear mapping between values and screen positions.  This recommended for signed
+ *                            additive quantities of a limited range.
+ * @GWY_SCALE_MAPPING_SQRT: Screen positions correspond to ‘signed square roots’ of the value.  This is the
+ *                          recommended general-purpose default mapping type as it works with both signed and usigned
+ *                          quantities and offers good sensitivity for both large and small values.
+ * @GWY_SCALE_MAPPING_LOG: Screen positions correspond to logarithms of values. The adjustment range must contain only
+ *                         positive values.  For quantities of extreme ranges this mapping may be preferred to
+ *                         %GWY_SCALE_MAPPING_SQRT.
+ * @GWY_SCALE_MAPPING_LOG1P: Screen positions correspond to logarithms of values with 1 added fist. The adjustment
+ *                           range must contain only non-negative values.  This mapping may be preferred for integer
+ *                           quantities with extreme ranges which also include zero. (Since 2.62)
  *
  * Type of adjustment bar mapping functions.
  *
  * Since: 2.49
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
