@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyaxis.c 21781 2019-01-03 12:33:06Z yeti-dn $
+ *  $Id: gwyaxis.c 25853 2023-10-13 17:24:18Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -344,6 +344,21 @@ gwy_axis_new(gint orientation)
     return GTK_WIDGET(axis);
 }
 
+static void
+clear_ticks(GwyAxis *axis)
+{
+    GwyAxisLabeledTick *mjt;
+    guint i;
+
+    for (i = 0; i < axis->mjticks->len; i++) {
+        mjt = &g_array_index(axis->mjticks, GwyAxisLabeledTick, i);
+        g_string_free(mjt->ttext, TRUE);
+    }
+    g_array_set_size(axis->mjticks, 0);
+    /* We do not have minor tick texts. */
+    g_array_set_size(axis->miticks, 0);
+}
+
 static void
 gwy_axis_finalize(GObject *object)
 {
@@ -354,6 +369,9 @@ gwy_axis_finalize(GObject *object)
     axis = GWY_AXIS(object);
 
     g_string_free(axis->label_text, TRUE);
+    if (axis->magnification_string)
+        g_string_free(axis->magnification_string, TRUE);
+    clear_ticks(axis);
     g_array_free(axis->mjticks, TRUE);
     g_array_free(axis->miticks, TRUE);
     if (axis->mjpubticks)
@@ -1374,36 +1392,28 @@ gwy_axis_logscale(GwyAxis *a)
  * < 0 if there are not enough major ticks,
  * > 0 if there area too many ticks */
 static gint
-gwy_axis_scale(GwyAxis *a)
+gwy_axis_scale(GwyAxis *axis)
 {
-    gsize i;
     gint ret;
-    GwyAxisLabeledTick *mjt;
 
     /*never use logarithmic mode for negative numbers*/
-    if (a->reqmin < 0 && a->is_logarithmic)
+    if (axis->reqmin < 0 && axis->is_logarithmic)
         return 1; /*this is an error*/
 
-    /*remove old ticks*/
-    for (i = 0; i < a->mjticks->len; i++) {
-        mjt = &g_array_index(a->mjticks, GwyAxisLabeledTick, i);
-        g_string_free(mjt->ttext, TRUE);
-    }
-    g_array_set_size(a->mjticks, 0);
-    g_array_set_size(a->miticks, 0);
+    clear_ticks(axis);
 
     /*find tick positions*/
-    if (a->is_logarithmic)
-        gwy_axis_logscale(a);
+    if (axis->is_logarithmic)
+        gwy_axis_logscale(axis);
     else
-        gwy_axis_normalscale(a);
+        gwy_axis_normalscale(axis);
 
     /* Displaying NaN or similar rubbish, just return success... */
-    if (!a->mjticks->len)
+    if (!axis->mjticks->len)
         return 0;
 
     /*label ticks*/
-    ret = gwy_axis_formatticks(a);
+    ret = gwy_axis_formatticks(axis);
 
     return ret;
 }
